"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_1 = require("@atomist/automation-client/lib/configuration");
const print = require("./print");
/**
 * Read user config and resolve workspace IDs then simplify config
 * down to a CliConfig.
 *
 * @return CliConfig populated from user config
 */
function resolveUserConfig() {
    let userConfig;
    try {
        userConfig = configuration_1.getUserConfig() || {};
    }
    catch (e) {
        print.warn(`Failed to load user configuration, ignoring: ${e.message}`);
        userConfig = {};
    }
    configuration_1.resolveWorkspaceIds(userConfig);
    if (!userConfig.workspaceIds) {
        userConfig.workspaceIds = [];
    }
    if (userConfig.teamIds) {
        delete userConfig.teamIds;
    }
    return userConfig;
}
exports.resolveUserConfig = resolveUserConfig;
/**
 * Read user config and simplify it down to a CliConfig.
 *
 * @return CliConfig populated from user config
 */
function resolveCliConfig() {
    const userConfig = resolveUserConfig();
    return {
        apiKey: userConfig.apiKey,
        workspaceIds: userConfig.workspaceIds,
        endpoints: userConfig.endpoints,
    };
}
exports.resolveCliConfig = resolveCliConfig;
//# sourceMappingURL=cliConfig.js.map