"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Determine whether sdm-local commands should be loaded.  To improve
 * startup times and eliminate client startup when unnecessary, we do
 * not load the sdm-local commands if we are just running a native CLI
 * command.
 *
 * @param args command-line arguments, typically process.argv
 * @return true if the SDM local commands should be loaded
 */
function shouldAddLocalSdmCommands(args) {
    if (args.includes("--help") || args.includes("--h") || args.includes("-h") || args.includes("-?")) {
        return true;
    }
    const command = args.slice(2).filter(a => !/^-/.test(a)).shift();
    if (!command) {
        return false;
    }
    const internalCommands = [
        "config",
        "connect",
        "execute",
        "git-hook",
        "gql-fetch",
        "install",
        "kube",
        "kube-decrypt",
        "kube-encrypt",
        "kube-fetch",
        "kube-install",
        "login",
        "start",
        "update",
    ];
    return !internalCommands.includes(command);
}
exports.shouldAddLocalSdmCommands = shouldAddLocalSdmCommands;
/**
 * Does this command start up an embedded SDM?
 * @param args command-line arguments, typically process.argv
 */
function isEmbeddedSdmCommand(args) {
    const relevant = args.slice(2);
    return relevant.length > 0 && ["create sdm", "enable local"].includes(relevant.join(" "));
}
exports.isEmbeddedSdmCommand = isEmbeddedSdmCommand;
/**
 * Call the provided function with the provided arguments and capture
 * any errors.  When the function is complete, `process.exit` will be
 * called with the appropriate code, i.e., this function will never return.
 *
 * @param fn function providing the desired command and returning a
 *           Promise of an integer exit value.
 * @param argv command-line arguments
 */
function cliCommand(fn) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const status = yield fn();
            process.exit(status);
        }
        catch (e) {
            process.stderr.write(`Unhandled Error: ${e.message}`);
            process.exit(101);
        }
        throw new Error("You will not get here. You have exited.");
    });
}
exports.cliCommand = cliCommand;
/**
 * Parse positional parameters into parameter name/value pairs.  The
 * positional parameters should be of the form NAME[=VALUE].  If
 * =VALUE is omitted, the value is set to `undefined`.  If the VALUE
 * is empty, i.e., NAME=, then the value is the empty string.
 *
 * @param args typically argv._ from yargs
 * @return array of CommandInvocation Arg
 */
function extractArgs(args) {
    return args.map(arg => {
        const split = arg.indexOf("=");
        if (split < 0) {
            return { name: arg, value: undefined };
        }
        const name = arg.slice(0, split);
        const value = arg.slice(split + 1);
        return { name, value };
    });
}
exports.extractArgs = extractArgs;
//# sourceMappingURL=command.js.map