"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const cliConfig_1 = require("./cliConfig");
const gql_1 = require("./gql");
const print = require("./print");
const spawn_1 = require("./spawn");
/**
 * Fetch the GraphQL schema for an Atomist workspace.
 *
 * @param opts see GqlFetchOptions
 * @return integer return value
 */
function gqlFetch(opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const cliConfig = cliConfig_1.resolveCliConfig();
        if (!cliConfig.apiKey) {
            print.error(`No API key set in user configuration, run 'atomist config' first`);
            return Promise.resolve(1);
        }
        if (!cliConfig.workspaceIds || cliConfig.workspaceIds.length < 1) {
            print.error(`No workspace IDs set in use configuration, run 'atomist config' first`);
            return Promise.resolve(1);
        }
        else if (cliConfig.workspaceIds.length > 1) {
            print.warn(`More than one workspace ID in user configuration, using first: ${cliConfig.workspaceIds[0]}`);
        }
        const workspaceId = cliConfig.workspaceIds[0];
        const graphQL = cliConfig.endpoints && cliConfig.endpoints.graphql
            ? cliConfig.endpoints.graphql : "https://automation.atomist.com/graphql/team";
        const outDir = path.join(gql_1.libDir(opts.cwd), "graphql");
        const outSchema = path.join(outDir, "schema.json");
        yield fs.ensureDir(outDir);
        const spawnOpts = {
            command: "apollo",
            args: [
                "schema:download",
                outSchema,
                "--endpoint", `${graphQL}/${workspaceId}`,
                "--header", `Authorization: Bearer ${cliConfig.apiKey}`,
            ],
            cwd: opts.cwd,
            install: opts.install,
            compile: false,
        };
        return spawn_1.spawnBinary(spawnOpts);
    });
}
exports.gqlFetch = gqlFetch;
//# sourceMappingURL=gqlFetch.js.map