import { Configuration } from "@atomist/automation-client";
import { K8sObject, K8sObjectApi } from "@atomist/sdm-pack-k8s/lib/kubernetes/api";
/**
 * Command-line options and arguments for kube.
 */
export interface KubeInstallOptions {
    /** Name of Kubernetes cluster */
    env?: string;
    /** Namespace to deploy Atomist Kubernetes utilities into */
    ns?: string;
    /** Only dry-run the command */
    dryRun?: boolean;
    /** Confirm all questions */
    yes?: boolean;
    /** URL of the public ingress */
    url?: string;
}
/**
 * Deploy Atomist Kubernetes utilities into a Kubernetes cluster.
 *
 * @param opts see [[KubeInstallOptions]]
 * @return integer return value, 0 if successful, non-zero otherwise
 */
export declare function kubeInstall(opts: KubeInstallOptions): Promise<number>;
/**
 * Fetch the appropriate Kubernetes specs from GitHub and parse them.
 *
 * @param ns Namespace resources are to be deployed to, if not cluster-wide
 * @return Resource specs that need to be upserted
 */
export declare function fetchSpecs(ns?: string): Promise<K8sObject[]>;
/**
 * Convert workspace IDs to Atomist Kubernetes webhook URLs.
 *
 * @param workspaceIds array of Atomist workspace/team IDs
 * @return comma-delimited list of webhook URLs
 */
export declare function kubeWebhookUrls(workspaceIds: string[]): string;
/** Options informing the k8s-sdm configuration. */
export interface K8sSdmConfigOptions {
    apiKey: string;
    environment: string;
    workspaceIds: string[];
    url?: string;
}
/**
 * Create appropriate configuration for a k8s-sdm.
 */
export declare function k8sSdmConfig(opts: K8sSdmConfigOptions): Configuration;
export declare function processSpecs(raw: string, ns?: string): K8sObject[];
/**
 * Simple Kubernetes v1.Secret interface.
 */
export interface KubeSecret {
    apiVersion: "v1";
    kind: "Secret";
    type: "Opaque";
    metadata: {
        name: string;
        namespace?: string;
    };
    data: {
        [key: string]: string;
    };
}
/**
 * Create encoded secret object from key-value pairs.
 *
 * @param secrets key-value pairs of secrets, the values are base64 encoded
 * @return Kubernetes secret object
 */
export declare function encodeSecret(name: string, ns: string, secrets: {
    [key: string]: string;
}): KubeSecret;
/**
 * Return informative string for spec.
 */
export declare function specSlug(spec: K8sObject): string;
/**
 * Create or update Kubernetes resource.
 */
export declare function applySpec(client: K8sObjectApi, spec: K8sObject): Promise<void>;
//# sourceMappingURL=kubeInstall.d.ts.map