/**
 * Options when executing a command.
 */
export interface SpawnOptions {
    /** Command to execute */
    command: string;
    /** Command-line arguments of command */
    args?: string[];
    /** Directory to run command in, must be an automation client directory */
    cwd?: string;
    /** If true or no node_modules directory exists, run "npm install" before running command */
    install?: boolean;
    /** If true, run `npm run compile` before running command */
    compile?: boolean;
    /** Checks to run after the optional install and compile */
    checks?: Array<() => number>;
}
/**
 * Run a binary automation-client dependency in a platform-independent-ish way.
 */
export declare function spawnBinary(opts: SpawnOptions): Promise<number>;
/**
 * Run an @atomist/automation-client Node.js script.
 */
export declare function spawnJs(opts: SpawnOptions & {
    nodeArgs?: string[];
}): Promise<number>;
/**
 * Options used by spawnPromise to execute a command.
 */
export interface SpawnPromiseOptions {
    /** Executable to spawn */
    command: string;
    /** Arguments to pass to executable */
    args?: string[];
    /** Directory to launch executable process in, default is process process.cwd */
    cwd?: string;
    /** If true, ensure package.json exist in cwd before running command */
    checkPackageJson?: boolean;
}
/**
 * Run a command and return a promise supplying the exit value of
 * the command.
 *
 * @param options see SpawnPromiseoptions
 * @return a Promise of the return value of the spawned command
 */
export declare function spawnPromise(options: SpawnPromiseOptions): Promise<number>;
/**
 * Create string representation of command and arguments, removing
 * sensitive information.
 *
 * @param cmd command
 * @param args command arguments
 * @return sanitized string representing command
 */
export declare function cleanCommandString(cmd: string, args?: string[]): string;
//# sourceMappingURL=spawn.d.ts.map