"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const readPkgUp = require("read-pkg-up");
const print = require("./print");
/**
 * Read package name and version from nearest package.json and return
 * standard CLI package and version string.
 *
 * @return standard version string
 */
function version() {
    try {
        // must be sync because yargs.version only accepts a string
        const pj = readPkgUp.sync({ cwd: __dirname });
        const dependencies = [];
        _.forEach(pj.package.dependencies, (v, k) => {
            if (k.startsWith("@atomist/")) {
                dependencies.push(`${k} ${v}`);
            }
        });
        return `${pj.package.name} ${pj.package.version}

${dependencies.sort((d1, d2) => d1.localeCompare(d2)).join("\n")}`;
    }
    catch (e) {
        print.error(`Failed to read package.json: ${e.message}`);
    }
    return "@atomist/cli 0.0.0";
}
exports.version = version;
//# sourceMappingURL=version.js.map