"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var atomistWebhook_1 = require("./lib/atomistWebhook");
exports.webhookBaseUrl = atomistWebhook_1.webhookBaseUrl;
var automationClient_1 = require("./lib/automationClient");
exports.AutomationClient = automationClient_1.AutomationClient;
var configuration_1 = require("./lib/configuration");
exports.configurationValue = configuration_1.configurationValue;
exports.DEFAULT_REDACTION_PATTERNS = configuration_1.DEFAULT_REDACTION_PATTERNS;
var decorators_1 = require("./lib/decorators");
exports.MappedParameter = decorators_1.MappedParameter;
exports.MappedParameters = decorators_1.MappedParameters;
exports.Parameter = decorators_1.Parameter;
exports.Parameters = decorators_1.Parameters;
exports.Secret = decorators_1.Secret;
exports.Secrets = decorators_1.Secrets;
exports.Tags = decorators_1.Tags;
exports.Value = decorators_1.Value;
var globals_1 = require("./lib/globals");
exports.automationClientInstance = globals_1.automationClientInstance;
const GraphQL = require("./lib/graph/graphQL");
exports.GraphQL = GraphQL;
const validationPatterns = require("./lib/operations/common/params/validationPatterns");
exports.validationPatterns = validationPatterns;
const editModes = require("./lib/operations/edit/editModes");
exports.editModes = editModes;
const parseUtils = require("./lib/project/util/parseUtils");
exports.parseUtils = parseUtils;
const projectUtils = require("./lib/project/util/projectUtils");
exports.projectUtils = projectUtils;
const secured = require("./lib/secured");
exports.secured = secured;
const astUtils = require("./lib/tree/ast/astUtils");
exports.astUtils = astUtils;
const matchTesters = require("./lib/tree/ast/matchTesters");
exports.matchTesters = matchTesters;
var HandlerResult_1 = require("./lib/HandlerResult");
exports.failure = HandlerResult_1.failure;
exports.Failure = HandlerResult_1.Failure;
exports.FailurePromise = HandlerResult_1.FailurePromise;
exports.reduceResults = HandlerResult_1.reduceResults;
exports.success = HandlerResult_1.success;
exports.Success = HandlerResult_1.Success;
exports.SuccessPromise = HandlerResult_1.SuccessPromise;
var shutdown_1 = require("./lib/internal/util/shutdown");
exports.registerShutdownHook = shutdown_1.registerShutdownHook;
exports.safeExit = shutdown_1.safeExit;
var string_1 = require("./lib/internal/util/string");
exports.guid = string_1.guid;
exports.toStringArray = string_1.toStringArray;
var BitBucketRepoRef_1 = require("./lib/operations/common/BitBucketRepoRef");
exports.BitBucketRepoRef = BitBucketRepoRef_1.BitBucketRepoRef;
var BitBucketServerRepoRef_1 = require("./lib/operations/common/BitBucketServerRepoRef");
exports.BitBucketServerRepoRef = BitBucketServerRepoRef_1.BitBucketServerRepoRef;
var gitHubRepoLoader_1 = require("./lib/operations/common/gitHubRepoLoader");
exports.gitHubRepoLoader = gitHubRepoLoader_1.gitHubRepoLoader;
var GitHubRepoRef_1 = require("./lib/operations/common/GitHubRepoRef");
exports.GitHubRepoRef = GitHubRepoRef_1.GitHubRepoRef;
exports.isGitHubRepoRef = GitHubRepoRef_1.isGitHubRepoRef;
var ProjectOperationCredentials_1 = require("./lib/operations/common/ProjectOperationCredentials");
exports.isTokenCredentials = ProjectOperationCredentials_1.isTokenCredentials;
var RepoId_1 = require("./lib/operations/common/RepoId");
exports.isRemoteRepoRef = RepoId_1.isRemoteRepoRef;
exports.ScmProviderType = RepoId_1.ProviderType;
exports.SimpleRepoId = RepoId_1.SimpleRepoId;
var GitlabRepoCreationParameters_1 = require("./lib/operations/generate/GitlabRepoCreationParameters");
exports.GitlabRepoCreationParameters = GitlabRepoCreationParameters_1.GitlabRepoCreationParameters;
var ReviewResult_1 = require("./lib/operations/review/ReviewResult");
exports.DefaultReviewComment = ReviewResult_1.DefaultReviewComment;
exports.reviewCommentSorter = ReviewResult_1.reviewCommentSorter;
var Tagger_1 = require("./lib/operations/tagger/Tagger");
exports.unifiedTagger = Tagger_1.unifiedTagger;
__export(require("./lib/project/fileGlobs"));
var GitCommandGitProject_1 = require("./lib/project/git/GitCommandGitProject");
exports.GitCommandGitProject = GitCommandGitProject_1.GitCommandGitProject;
exports.isValidSHA1 = GitCommandGitProject_1.isValidSHA1;
var LocalProject_1 = require("./lib/project/local/LocalProject");
exports.isLocalProject = LocalProject_1.isLocalProject;
var NodeFsLocalProject_1 = require("./lib/project/local/NodeFsLocalProject");
exports.NodeFsLocalProject = NodeFsLocalProject_1.NodeFsLocalProject;
var InMemoryFile_1 = require("./lib/project/mem/InMemoryFile");
exports.InMemoryProjectFile = InMemoryFile_1.InMemoryFile;
var InMemoryProject_1 = require("./lib/project/mem/InMemoryProject");
exports.InMemoryProject = InMemoryProject_1.InMemoryProject;
var jsonUtils_1 = require("./lib/project/util/jsonUtils");
exports.doWithJson = jsonUtils_1.doWithJson;
var AutomationEventListener_1 = require("./lib/server/AutomationEventListener");
exports.AutomationEventListenerSupport = AutomationEventListener_1.AutomationEventListenerSupport;
var SmartParameters_1 = require("./lib/SmartParameters");
exports.NoParameters = SmartParameters_1.NoParameters;
__export(require("./lib/spi/graph/GraphClient"));
__export(require("./lib/spi/http/axiosHttpClient"));
__export(require("./lib/spi/http/curlHttpClient"));
__export(require("./lib/spi/http/httpClient"));
__export(require("./lib/spi/message/MessageClient"));
var FileHits_1 = require("./lib/tree/ast/FileHits");
exports.ZapTrailingWhitespace = FileHits_1.ZapTrailingWhitespace;
var TypeScriptFileParser_1 = require("./lib/tree/ast/typescript/TypeScriptFileParser");
exports.TypeScriptES6FileParser = TypeScriptFileParser_1.TypeScriptES6FileParser;
var MicrogrammarBasedFileParser_1 = require("./lib/tree/ast/microgrammar/MicrogrammarBasedFileParser");
exports.MicrogrammarBasedFileParser = MicrogrammarBasedFileParser_1.MicrogrammarBasedFileParser;
var RegexFileParser_1 = require("./lib/tree/ast/regex/RegexFileParser");
exports.RegexFileParser = RegexFileParser_1.RegexFileParser;
__export(require("./lib/util/exec"));
var gitHub_1 = require("./lib/util/gitHub");
exports.deepLink = gitHub_1.deepLink;
exports.raiseIssue = gitHub_1.raiseIssue;
var logger_1 = require("./lib/util/logger");
exports.LoggingFormat = logger_1.LoggingFormat;
exports.NoLogging = logger_1.NoLogging;
exports.PlainLogging = logger_1.PlainLogging;
exports.MinimalLogging = logger_1.MinimalLogging;
exports.ClientLogging = logger_1.ClientLogging;
exports.configureLogging = logger_1.configureLogging;
exports.clientLoggingConfiguration = logger_1.clientLoggingConfiguration;
exports.logger = logger_1.logger;
var redact_1 = require("./lib/util/redact");
exports.addRedaction = redact_1.addRedaction;
exports.addLogRedaction = redact_1.addLogRedaction;
var retry_1 = require("./lib/util/retry");
exports.doWithRetry = retry_1.doWithRetry;
var pool_1 = require("./lib/util/pool");
exports.executeAll = pool_1.executeAll;
__export(require("./lib/util/spawn"));
__export(require("./lib/operations/common/gitlabRepoLoader"));
__export(require("./lib/operations/common/GitlabPrivateTokenCredentials"));
__export(require("./lib/operations/common/GitlabRepoRef"));
__export(require("./lib/operations/generate/GitlabRepoCreationParameters"));
var ApolloGraphClient_1 = require("./lib/graph/ApolloGraphClient");
exports.ApolloGraphClient = ApolloGraphClient_1.ApolloGraphClient;
var ApolloGraphClientFactory_1 = require("./lib/graph/ApolloGraphClientFactory");
exports.ApolloGraphClientFactory = ApolloGraphClientFactory_1.ApolloGraphClientFactory;
var GraphClientFactory_1 = require("./lib/spi/graph/GraphClientFactory");
exports.defaultGraphClientFactory = GraphClientFactory_1.defaultGraphClientFactory;
var axiosHttpClient_1 = require("./lib/spi/http/axiosHttpClient");
exports.AxiosHttpClient = axiosHttpClient_1.AxiosHttpClient;
exports.AxiosHttpClientFactory = axiosHttpClient_1.AxiosHttpClientFactory;
var curlHttpClient_1 = require("./lib/spi/http/curlHttpClient");
exports.CurlHttpClient = curlHttpClient_1.CurlHttpClient;
exports.CurlHttpClientFactory = curlHttpClient_1.CurlHttpClientFactory;
var httpClient_1 = require("./lib/spi/http/httpClient");
exports.defaultHttpClientFactory = httpClient_1.defaultHttpClientFactory;
exports.DefaultHttpClientOptions = httpClient_1.DefaultHttpClientOptions;
exports.HttpMethod = httpClient_1.HttpMethod;
var wsClient_1 = require("./lib/spi/http/wsClient");
exports.WSWebSocketFactory = wsClient_1.WSWebSocketFactory;
exports.defaultWebSocketFactory = wsClient_1.defaultWebSocketFactory;
var statsdClient_1 = require("./lib/spi/statsd/statsdClient");
exports.defaultStatsDClientFactory = statsdClient_1.defaultStatsDClientFactory;
exports.HotShotStatsDClientFactory = statsdClient_1.HotShotStatsDClientFactory;
exports.NodeStatsDClientFactory = statsdClient_1.NodeStatsDClientFactory;
//# sourceMappingURL=index.js.map