"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const axios_fetch_1 = require("axios-fetch");
const NodeCache = require("node-cache");
const http_1 = require("../internal/util/http");
const logger_1 = require("../util/logger");
const ApolloGraphClient_1 = require("./ApolloGraphClient");
/**
 * Factory for creating GraphClient instances for incoming commands and events.
 *
 * Uses a cache to store GraphClient instances for 5 mins after which new instances will be given out.
 */
class ApolloGraphClientFactory {
    create(workspaceId, configuration) {
        this.init();
        let graphClient = this.graphClients.get(workspaceId);
        if (graphClient) {
            return graphClient;
        }
        else {
            const headers = {
                "Authorization": `Bearer ${configuration.apiKey}`,
                "apollographql-client-name": `${configuration.name}/${workspaceId}`,
                "apollographql-client-version": configuration.version,
            };
            graphClient = new ApolloGraphClient_1.ApolloGraphClient(`${configuration.endpoints.graphql}/${workspaceId}`, headers, this.configure(configuration), configuration.graphql.listeners);
            this.graphClients.set(workspaceId, graphClient);
            return graphClient;
        }
        logger_1.logger.debug("Unable to create graph client for team '%s'", workspaceId);
        return null;
    }
    configure(configuration) {
        return axios_fetch_1.buildAxiosFetch(axios_1.default.create(http_1.configureProxy({})));
    }
    init() {
        if (!this.graphClients) {
            this.graphClients = new NodeCache({
                stdTTL: 1 * 60,
                checkperiod: 1 * 30,
                useClones: false,
            });
        }
    }
}
exports.ApolloGraphClientFactory = ApolloGraphClientFactory;
//# sourceMappingURL=ApolloGraphClientFactory.js.map