"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../../util/logger");
const config_1 = require("../util/config");
const string_1 = require("../util/string");
const AtomistPrefix = "atomist://";
/**
 * Local secrets: Resolve using config (resolves to /config directory).
 * Throw exception if not found.
 */
class NodeConfigSecretResolver {
    resolve(key) {
        const resolved = key.startsWith(AtomistPrefix) ?
            config_1.config(key.replace(AtomistPrefix, "")) :
            config_1.config(key);
        if (!resolved) {
            throw new Error(`Failed to resolve '${key}'`);
        }
        else {
            logger_1.logger.debug(`Resolved '${key}' to '${string_1.hideString(resolved)}'`);
            return resolved;
        }
    }
}
exports.NodeConfigSecretResolver = NodeConfigSecretResolver;
//# sourceMappingURL=NodeConfigSecretResolver.js.map