/**
 * Enum value expressing the state of a health check.
 */
export declare enum HealthStatus {
    Up = "UP",
    Down = "DOWN",
    OutOfService = "OUT_OF_SERVICE"
}
export interface Health<P> {
    status: HealthStatus;
    detail: P;
}
/**
 * A HealthIndicator is a function that determines the Health of a sub-system or service this client
 * consumes.
 */
export declare type HealthIndicator = () => Health<any>;
/**
 * Register a HealthIndicator at startup.
 * @param {HealthIndicator} indicator
 */
export declare function registerHealthIndicator(indicator: HealthIndicator): void;
/**
 * Returns the combined health of the client.
 * @returns {Health<any>}
 */
export declare function health(): Health<any>;
export declare const Indicators: HealthIndicator[];
//# sourceMappingURL=health.d.ts.map