import { ActionResult } from "../../action/ActionResult";
import { Configurable } from "../../project/git/Configurable";
import { ProjectOperationCredentials } from "./ProjectOperationCredentials";
import { ProviderType, PullRequestReviewer, RemoteRepoRef } from "./RepoId";
/**
 * Superclass for RemoteRepoRef implementations.
 * Handles parsing remote base
 *
 * This should ultimately move down to automation-client-ts and replace AbstractRepoRef.
 */
export declare abstract class AbstractRemoteRepoRef implements RemoteRepoRef {
    readonly providerType: ProviderType;
    readonly owner: string;
    readonly repo: string;
    readonly sha: string;
    readonly path?: string;
    branch?: string;
    /** URL scheme from applied to both API and remote bases. */
    readonly scheme: "http://" | "https://";
    /** API URL without the scheme or trailing '/'. */
    readonly apiBase: string;
    abstract readonly kind: string;
    /** Git remote URL not including scheme or trailing '/' */
    readonly remoteBase: string;
    /**
     * Construct a new RemoteRepoRef
     * @param {ProviderType} providerType
     * @param {string} rawRemote remote url, like for cloning or linking into the repo. Should start with a scheme.
     *                           May have a trailing slash, which will be stripped
     * @param rawApiBase API base url. Should start with a scheme. May have a trailing slash, which will be stripped.
     * @param {string} owner
     * @param {string} repo
     * @param {string} sha
     * @param {string} path
     */
    protected constructor(providerType: ProviderType, rawRemote: string, rawApiBase: string, owner: string, repo: string, sha: string, path?: string, branch?: string);
    get url(): string;
    cloneUrl(creds: ProjectOperationCredentials): string;
    get pathComponent(): string;
    abstract createRemote(creds: ProjectOperationCredentials, description: string, visibility: string): Promise<ActionResult<this>>;
    abstract setUserConfig(credentials: ProjectOperationCredentials, project: Configurable): Promise<ActionResult<any>>;
    abstract raisePullRequest(creds: ProjectOperationCredentials, title: string, body: string, head: string, base: string, reviewers?: PullRequestReviewer[]): Promise<ActionResult<this>>;
    abstract deleteRemote(creds: ProjectOperationCredentials): Promise<ActionResult<this>>;
}
//# sourceMappingURL=AbstractRemoteRepoRef.d.ts.map