import { ActionResult } from "../../action/ActionResult";
import { Configurable } from "../../project/git/Configurable";
import { AbstractRemoteRepoRef } from "./AbstractRemoteRepoRef";
import { ProjectOperationCredentials } from "./ProjectOperationCredentials";
import { PullRequestReviewerType } from "./RepoId";
/**
 * RemoteRepoRef implementation for BitBucket server (not BitBucket Cloud)
 */
export declare class BitBucketServerRepoRef extends AbstractRemoteRepoRef {
    private readonly isProject;
    readonly ownerType: "projects" | "users";
    readonly kind: string;
    /**
     * Construct a new BitBucketServerRepoRef
     * @param {string} remoteBase remote base, including scheme
     * @param {string} owner
     * @param {string} repo
     * @param {boolean} isProject
     * @param {string} sha
     * @param {string} path
     * @param {string} branch
     * @param {string}apiUrl
     */
    constructor(remoteBase: string, owner: string, repo: string, isProject?: boolean, sha?: string, path?: string, branch?: string, apiUrl?: string);
    createRemote(creds: ProjectOperationCredentials, description: string, visibility: any): Promise<ActionResult<this>>;
    deleteRemote(creds: ProjectOperationCredentials): Promise<ActionResult<this>>;
    setUserConfig(credentials: ProjectOperationCredentials, project: Configurable): Promise<ActionResult<this>>;
    raisePullRequest(creds: ProjectOperationCredentials, title: string, body: string, head: string, base: string, reviewers?: Array<{
        type: Exclude<PullRequestReviewerType, PullRequestReviewerType.team>;
        name: string;
    }>): Promise<ActionResult<this>>;
    protected getDefaultReviewers(creds: ProjectOperationCredentials, repoId: number, head: string, base: string): Promise<string[]>;
    private getRepoId;
    get url(): string;
    get pathComponent(): string;
    private get maybeTilde();
    private get apiBasePathComponent();
    get apiPathComponent(): string;
}
//# sourceMappingURL=BitBucketServerRepoRef.d.ts.map