"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../../../decorators");
const GitlabRepoRef_1 = require("../GitlabRepoRef");
const TargetsParams_1 = require("./TargetsParams");
const validationPatterns_1 = require("./validationPatterns");
let GitlabTargetsParams = class GitlabTargetsParams extends TargetsParams_1.TargetsParams {
    constructor(token) {
        super();
        this.token = token;
        this.branch = "master";
        this.repos = ".*";
    }
    get credentials() {
        const creds = { privateToken: this.token };
        return creds;
    }
    get description() {
        return "Gitlab";
    }
    /**
     * Return a single RepoRef or undefined if we're not identifying a single repo
     * @return {RepoRef}
     */
    get repoRef() {
        return (!!this.owner && !!this.repo && !this.usesRegex) ?
            GitlabRepoRef_1.GitlabRepoRef.from({
                owner: this.owner,
                repo: this.repo,
                sha: this.sha,
                branch: this.branch,
                rawApiBase: this.apiUrl,
                gitlabRemoteUrl: this.url,
            }) :
            undefined;
    }
    bindAndValidate() {
        if (!this.repo) {
            if (!this.repos) {
                return {
                    message: "If not executing in a mapped channel, must identify a repo via: `targets.owner`" +
                        "and `targets.repo`, or a repo name regex via `targets.repos`",
                };
            }
            this.repo = this.repos;
        }
    }
};
__decorate([
    decorators_1.MappedParameter(decorators_1.MappedParameters.GitHubApiUrl, false),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "apiUrl", void 0);
__decorate([
    decorators_1.MappedParameter(decorators_1.MappedParameters.GitHubOwner, false),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "owner", void 0);
__decorate([
    decorators_1.MappedParameter(decorators_1.MappedParameters.GitHubRepository, false),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "repo", void 0);
__decorate([
    decorators_1.MappedParameter(decorators_1.MappedParameters.GitHubUrl, false),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "url", void 0);
__decorate([
    decorators_1.Parameter(Object.assign(Object.assign({ description: "Ref" }, validationPatterns_1.GitShaRegExp), { required: false })),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "sha", void 0);
__decorate([
    decorators_1.Parameter(Object.assign(Object.assign({ description: "Branch Defaults to 'master'" }, validationPatterns_1.GitBranchRegExp), { required: false })),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "branch", void 0);
__decorate([
    decorators_1.Parameter({ description: "regex", required: false }),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "repos", void 0);
__decorate([
    decorators_1.Parameter({ description: "Repository visibility. 'public' or 'private", required: false }),
    __metadata("design:type", String)
], GitlabTargetsParams.prototype, "visibility", void 0);
GitlabTargetsParams = __decorate([
    decorators_1.Parameters(),
    __metadata("design:paramtypes", [String])
], GitlabTargetsParams);
exports.GitlabTargetsParams = GitlabTargetsParams;
//# sourceMappingURL=GitlabTargetsParams.js.map