"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const validationPatterns_1 = require("./validationPatterns");
/**
 * Base parameters for working with repo(s).
 * Allows use of regex.
 */
class TargetsParams {
    constructor() {
        /**
         * If we're not tied to a single repo ref, test this RepoRef
         * @param {RepoRef} rr
         * @return {boolean}
         */
        this.test = rr => {
            if (this.repoRef) {
                const my = this.repoRef;
                return my.owner === rr.owner && my.repo === rr.repo;
            }
            if (this.usesRegex) {
                if (this.owner && this.owner !== rr.owner) {
                    return false;
                }
                return new RegExp(this.repo).test(rr.repo);
            }
            return false;
        };
    }
    get usesRegex() {
        return !!this.repo && (!validationPatterns_1.GitHubNameRegExp.pattern.test(this.repo) || !this.owner);
    }
}
exports.TargetsParams = TargetsParams;
//# sourceMappingURL=TargetsParams.js.map