"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../../util/logger");
const pool_1 = require("../../util/pool");
const defaultRepoLoader_1 = require("./defaultRepoLoader");
const repoFilter_1 = require("./repoFilter");
/**
 * Perform an action against all the given repos.
 * Skip over repos that cannot be loaded, logging a warning.
 * @param {HandlerContext} ctx
 * @param credentials credentials for repo finding and loading
 * @param action action parameter
 * @param parameters optional parameters
 * @param {RepoFinder} repoFinder
 * @param {} repoFilter
 * @param {RepoLoader} repoLoader
 * @return {Promise<R[]>}
 */
function doWithAllRepos(ctx, credentials, action, parameters, repoFinder, repoFilter = repoFilter_1.AllRepos, repoLoader = defaultRepoLoader_1.defaultRepoLoader(credentials)) {
    return __awaiter(this, void 0, void 0, function* () {
        const ids = yield relevantRepos(ctx, repoFinder, repoFilter);
        const promises = ids.map(id => () => repoLoader(id)
            .catch(err => {
            logger_1.logger.debug("Unable to load repo %s/%s: %s", id.owner, id.repo, err);
            logger_1.logger.debug(err.stack);
            return undefined;
        })
            .then(p => {
            if (p) {
                return action(p, parameters);
            }
        }));
        return (yield pool_1.executeAll(promises)).filter(result => result);
    });
}
exports.doWithAllRepos = doWithAllRepos;
function relevantRepos(ctx, repoFinder, repoFilter = repoFilter_1.AllRepos) {
    return repoFinder(ctx)
        .then(rids => Promise.all(rids.map(rid => Promise.resolve(repoFilter(rid))
        .then(relevant => relevant ? rid : undefined))))
        .then(many => many.filter(s => s !== undefined));
}
exports.relevantRepos = relevantRepos;
//# sourceMappingURL=repoUtils.js.map