"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function toEditModeFactory(em) {
    return p => isEditMode(em) ? em : em(p);
}
exports.toEditModeFactory = toEditModeFactory;
function isEditMode(em) {
    return !!em.message;
}
exports.isEditMode = isEditMode;
/**
 * Merge method to use when auto merging the branches
 */
var AutoMergeMethod;
(function (AutoMergeMethod) {
    AutoMergeMethod["Merge"] = "[auto-merge-method:merge]";
    AutoMergeMethod["Rebase"] = "[auto-merge-method:rebase]";
    AutoMergeMethod["Squash"] = "[auto-merge-method:squash]";
})(AutoMergeMethod = exports.AutoMergeMethod || (exports.AutoMergeMethod = {}));
/**
 * Trigger for branch auto merge
 */
var AutoMergeMode;
(function (AutoMergeMode) {
    AutoMergeMode["ApprovedReview"] = "[auto-merge:on-approve]";
    AutoMergeMode["SuccessfulCheck"] = "[auto-merge:on-check-success]";
})(AutoMergeMode = exports.AutoMergeMode || (exports.AutoMergeMode = {}));
/**
 * Return a commit to master branch with the given message. Use with care!
 */
function commitToMaster(message) {
    return {
        message,
        branch: "master",
    };
}
exports.commitToMaster = commitToMaster;
function isBranchCommit(em) {
    const maybeBc = em;
    return !!maybeBc.branch && !!maybeBc.message;
}
exports.isBranchCommit = isBranchCommit;
/**
 * Captures basic information to create a commit
 */
class Commit {
    constructor(branch, message, autoMerge) {
        this.branch = branch;
        this.message = message;
        this.autoMerge = autoMerge;
    }
}
exports.Commit = Commit;
/**
 * Captures extra steps that must go into raising a PR
 */
class PullRequest extends Commit {
    constructor(branch, title, body = title, message = title, targetBranch, autoMerge, reviewers) {
        super(branch, message, autoMerge);
        this.branch = branch;
        this.title = title;
        this.body = body;
        this.message = message;
        this.targetBranch = targetBranch;
        this.autoMerge = autoMerge;
        this.reviewers = reviewers;
    }
}
exports.PullRequest = PullRequest;
function isPullRequest(em) {
    const maybePr = em;
    return !!maybePr.branch && !!maybePr.body && !!maybePr.title;
}
exports.isPullRequest = isPullRequest;
function isCustomExecutionEditMode(ei) {
    const maybeCei = ei;
    return !!maybeCei.edit;
}
exports.isCustomExecutionEditMode = isCustomExecutionEditMode;
//# sourceMappingURL=editModes.js.map