"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fileGlobs_1 = require("../fileGlobs");
const projectUtils_1 = require("./projectUtils");
const Separator = "-------------------";
/**
 * Use as a diagnostic step in an editor chain.
 * No op that dumps file information to the console.
 * @param stepName identification of the step in the process we're up to
 * @param globPattern optional glob pattern to select files. Match all if not supplied
 * @param stringifier function to convert files to strings. Default uses path
 */
function diagnosticDump(stepName, globPattern = fileGlobs_1.AllFiles, stringifier = f => f.path) {
    return project => projectUtils_1.gatherFromFiles(project, globPattern, (f) => __awaiter(this, void 0, void 0, function* () { return f; }))
        .then(files => 
    // tslint:disable-next-line:no-console
    console.log(`${Separator}\nProject name ${project.name}: Step=${stepName}; Files[${globPattern}]=\n` +
        `${files.map(f => "\t" + stringifier(f)).join("\n")}\n${Separator}`))
        .then(() => project);
}
exports.diagnosticDump = diagnosticDump;
//# sourceMappingURL=diagnosticUtils.js.map