import { HandleCommand } from "./HandleCommand";
import { HandlerContext } from "./HandlerContext";
import { HandlerResult } from "./HandlerResult";
import { Maker } from "./util/constructionUtils";
/**
 * Protect the given HandleCommand by only allowing access for certain slack users
 * @param {Maker<HandleCommand>} maker
 * @param {string} users
 * @returns {() => HandleCommand}
 */
export declare function slackUser(maker: Maker<HandleCommand>, ...users: string[]): () => HandleCommand;
/**
 * Protect the given HandleCommand by only allowing members of a certain GitHub Organization
 * @param {Maker<HandleCommand>} maker
 * @param {string} team
 * @returns {() => HandleCommand}
 */
export declare function githubOrg(maker: Maker<HandleCommand>, org: string): () => HandleCommand;
/**
 * Protect the given HandleCommand by only allowing members of a certain GitHub team
 * @param {Maker<HandleCommand>} maker
 * @param {string} team
 * @returns {() => HandleCommand}
 */
export declare function githubTeam(maker: Maker<HandleCommand>, gTeam: string): () => HandleCommand;
export declare function isGitHubTeamMember(owner: string, login: string, team: string, token: string, apiUrl?: string): Promise<boolean>;
export declare function sendUnauthorized(ctx: HandlerContext): Promise<HandlerResult>;
//# sourceMappingURL=secured.d.ts.map