"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const retry_1 = require("../../util/retry");
const axiosHttpClient_1 = require("./axiosHttpClient");
/**
 * Available HTTP request methods to use with HttpClient.
 */
var HttpMethod;
(function (HttpMethod) {
    HttpMethod["Get"] = "GET";
    HttpMethod["Post"] = "POST";
    HttpMethod["Put"] = "PUT";
    HttpMethod["Delete"] = "DELETE";
    HttpMethod["Options"] = "OPTIONS";
    HttpMethod["Patch"] = "PATCH";
    HttpMethod["Head"] = "HEAD";
})(HttpMethod = exports.HttpMethod || (exports.HttpMethod = {}));
/**
 * Default HTTP client options each implementation to should use.
 * @type {{method: HttpMethod; headers: {}; retry: WrapOptions}}
 */
exports.DefaultHttpClientOptions = {
    method: HttpMethod.Get,
    headers: {},
    retry: retry_1.DefaultRetryOptions,
};
/**
 * Default HttpClientFactory which gets registered in the automation-client if not a
 * different HttpClientFactory implementation is configured.
 * @see Configuration.http.client.factory
 * @type {HttpClientFactory}
 */
exports.defaultHttpClientFactory = () => new axiosHttpClient_1.AxiosHttpClientFactory();
//# sourceMappingURL=httpClient.js.map