"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const MatchPrefixResult_1 = require("./MatchPrefixResult");
function isSuccessfulMatchReport(fmr) {
    return fmr.successful;
}
exports.isSuccessfulMatchReport = isSuccessfulMatchReport;
function isFailedMatchReport(fmr) {
    return !fmr.successful;
}
exports.isFailedMatchReport = isFailedMatchReport;
function toPatternMatchOrDismatchReport(mr) {
    if (isSuccessfulMatchReport(mr)) {
        return mr.toPatternMatch();
    }
    else {
        if (isFailedMatchReport(mr)) {
            return {
                description: "Looks like it didn't match",
                explanationTree: mr.toExplanationTree(),
            };
        }
        throw new Error("not implemented");
    }
}
exports.toPatternMatchOrDismatchReport = toPatternMatchOrDismatchReport;
/**
 * Extract a tree describing how the match was parsed.
 * Each matcher adds a node, plus Concat adds nodes for all its properties
 * and Rep adds nodes for all its elements.
 * Synthetic elements are empty elements are not included; each node
 * maps to content in the file.
 * @param mr match report from Grammar.perfectMatch
 */
function toParseTree(mr) {
    if (!isSuccessfulMatchReport(mr)) {
        throw new Error("Match was not successful");
    }
    return mr.toParseTree();
}
exports.toParseTree = toParseTree;
/**
 * Return a tree which explains the activity of the matching, both
 * what matched and what didn't. Empty matches are included, and failed
 * matches are included. Use this to figure out why something matched (or didn't).
 * @param mr a MatchReport from Grammar.perfectMatch
 */
function toExplanationTree(mr) {
    return mr.toExplanationTree();
}
exports.toExplanationTree = toExplanationTree;
/**
 * Return the values extracted from a match. For any match constructed with
 * `microgrammar(...)` this will return an object with a property for each term.
 * Synthetic properties are included.
 * @param mr a MatchReport from Grammar.perfectMatch
 */
function toValueStructure(mr) {
    if (isSuccessfulMatchReport(mr)) {
        return mr.toValueStructure();
    }
    else {
        throw new Error("You can only get the value structure of successful matches");
    }
}
exports.toValueStructure = toValueStructure;
function toMatchPrefixResult(mr) {
    if (isSuccessfulMatchReport(mr)) {
        return mr.toPatternMatch();
    }
    return new MatchPrefixResult_1.MatchFailureReport(mr.matcher.$id, mr.offset, mr.matched, mr.description);
}
exports.toMatchPrefixResult = toMatchPrefixResult;
//# sourceMappingURL=MatchReport.js.map