"use strict";
/**
 * Returned when we failed to match prefix
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Represents a successful match. Contains microgrammar information
 * in fields with names beginning with $ and any user-defined fields.
 * To ensure this separation works cleanly, not bind user data to fields beginning with $.
 */
class PatternMatch {
    /**
     * Represents a match
     * @param $matcherId id of the matcher that matched
     * @param $matched the actual string content
     * @param $offset offset from 0 in input
     */
    constructor($matcherId, $matched, $offset) {
        this.$matcherId = $matcherId;
        this.$matched = $matched;
        this.$offset = $offset;
    }
    /**
     * Return just the structure that was matched, throwing away offset and matcher information.
     * This is useful if you want to store PatternMatches after you're done with their internal information.
     */
    matchedStructure() {
        return justTheData(this);
    }
}
exports.PatternMatch = PatternMatch;
function isPatternMatch(mpr) {
    return mpr != undefined && mpr !== undefined && mpr.$value !== undefined;
}
exports.isPatternMatch = isPatternMatch;
/**
 * Represents a complex pattern match. Sets properties to expose structure.
 * In the case of string properties, where we can't add provide the whole PatternMatch,
 * we store that in a parallel object $valueMatches
 * @Deprecated
 * Prefer SuccessfulMatchReport; use matchReport.toValueStructure() to get the object structure
 * or matchReport.toParseTree() to learn about how it matches
 */
class TreePatternMatch extends PatternMatch {
    constructor() {
        super(...arguments);
        this.$valueMatches = {};
    }
}
exports.TreePatternMatch = TreePatternMatch;
function isTreePatternMatch(om) {
    return om != undefined && om.submatches !== undefined;
}
exports.isTreePatternMatch = isTreePatternMatch;
/**
 * Return true if the member has a special meaning,
 * rather than being bound to the context. For example,
 * is a veto function or a private property.
 * @param name member name to test
 * @return {boolean}
 */
function isSpecialMember(name) {
    return name.indexOf("_") === 0;
}
exports.isSpecialMember = isSpecialMember;
function justTheData(match) {
    if (Array.isArray(match)) {
        return match.map(justTheData);
    }
    if (typeof match !== "object") {
        return match;
    }
    const output = {}; // it is not a const, I mutate it, but tslint won't let me declare otherwise :-(
    for (const p in match) {
        if (!(p.indexOf("_") === 0 || p.indexOf("$") === 0 || typeof match[p] === "function")) {
            output[p] = justTheData(match[p]);
        }
    }
    return output;
}
//# sourceMappingURL=PatternMatch.js.map