"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function replaceFirstAfter(content, offset, old, replacement) {
    return content.substring(0, offset) + content.substring(offset).replace(old, replacement);
}
// TODO this should also support streaming
class ChangeSet {
    constructor(initialContent) {
        this.initialContent = initialContent;
        this.changes = [];
    }
    change(match, to) {
        const change = new Change(match, to);
        // Don't add twice: update
        const found = this.changes.filter(c => c.match === match);
        if (found.length === 0) {
            this.changes.push(change);
            // Keep sorted with latest first
            this.changes.sort((pm1, pm2) => pm2.match.$offset - pm1.match.$offset);
        }
        else {
            const idx = this.changes.indexOf(found[0]);
            this.changes[idx] = change;
        }
    }
    updated() {
        if (this.changes.length === 0) {
            return this.initialContent;
        }
        let result = this.initialContent;
        for (const change of this.changes) {
            // console.log(`Applying change at ${change.match.$offset} to ${change.to}`)
            result = replaceFirstAfter(result, change.match.$offset, change.match.$matched, change.to);
        }
        if (!result) {
            throw new Error("Anomaly: !result");
        }
        return result;
    }
}
exports.ChangeSet = ChangeSet;
class Change {
    constructor(match, to) {
        this.match = match;
        this.to = to;
        if (match.$offset === undefined) {
            throw new Error(`No offset on match with id [${match.$matcherId}]: matched=[${match.$matched}]`);
        }
    }
}
//# sourceMappingURL=ChangeSet.js.map