"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const MatchReport_1 = require("../MatchReport");
const StringInputStream_1 = require("../spi/StringInputStream");
const InputStateFactory_1 = require("./InputStateFactory");
const failedMatchReport_1 = require("./matchReport/failedMatchReport");
function exactMatch(matcher, input, parseContext = {}, l) {
    return MatchReport_1.toPatternMatchOrDismatchReport(perfectMatch(matcher, input, parseContext, l));
}
exports.exactMatch = exactMatch;
function perfectMatch(matcher, input, parseContext = {}, l) {
    const inputState = InputStateFactory_1.inputStateFromStream(toInputStream(input), l);
    const result = matcher.matchPrefixReport(inputState, {}, parseContext);
    if (!MatchReport_1.isSuccessfulMatchReport(result)) {
        return result;
    }
    const trailingInputState = advanceTo(result.endingOffset, inputState);
    if (hasNothingLeft(trailingInputState)) {
        return result;
    }
    const extraInputDescription = describeExtraInput(trailingInputState);
    return failedMatchReport_1.failedMatchReport(matcher, {
        offset: 0,
        children: [result, trailingJunkMatchReport(result.endingOffset, extraInputDescription)],
        parseNodeName: "ExactMatch",
        reason: `Not all input was consumed: Left over [${extraInputDescription}]`,
    });
}
exports.perfectMatch = perfectMatch;
function advanceTo(endingOffset, inputState) {
    // is there a better way to move the input stream forward?
    let currentInputState = inputState;
    while (!currentInputState.exhausted() && currentInputState.offset <= endingOffset) {
        currentInputState = currentInputState.advance();
    }
    return currentInputState;
}
function hasNothingLeft(inputState) {
    // sometimes exhausted() returns false even when there is nothing left.
    return inputState.exhausted() || inputState.peek(1).length === 0;
}
function describeExtraInput(inputState) {
    const extraInput = inputState.peek(20);
    const nextInputState = inputState.consume(extraInput, "observing extra input in ExactMatch");
    return nextInputState.exhausted() ? extraInput : extraInput + "...";
}
function trailingJunkMatchReport(endingOffset, extraInputDescription) {
    return failedMatchReport_1.failedMatchReport({ $id: "EndOfInput" }, {
        offset: endingOffset,
        parseNodeName: "EndOfInput",
        reason: `Expected end of input. Saw: ${extraInputDescription}`,
    });
}
function toInputStream(input) {
    return (typeof input === "string") ?
        new StringInputStream_1.StringInputStream(input) :
        input;
}
//# sourceMappingURL=ExactMatch.js.map