"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DEFAULT_BUFFER_SIZE = 5000;
/**
 * Window over input. Enables us to use a simple InputStream
 * abstraction that does not need to support backtracking.
 */
class InputStateManager {
    constructor(stream, bufsize = DEFAULT_BUFFER_SIZE) {
        this.stream = stream;
        this.bufsize = bufsize;
        this.left = 0;
        this.window = "";
    }
    get(offset, n) {
        this.canSatisfy(offset + n);
        return this.window.substr(offset - this.left, n);
    }
    canSatisfy(offset) {
        if (offset < this.left) {
            throw new Error(`Cannot rewind to offset ${offset}: already at ${this.left}`);
        }
        if (offset > this.right()) {
            this.window += this.stream.read(Math.max(this.bufsize, offset - this.right()));
        }
        return this.right() >= offset;
    }
    /**
     * Drop characters left of the offset from window
     * @param offset leftmost offset we'll need
     */
    dropLeft(offset) {
        if (offset > this.left && offset <= this.right()) {
            this.window = this.window.substr(offset - this.left);
            this.left = offset;
        }
    }
    exhausted() {
        return this.window.length === 0 && this.stream.exhausted();
    }
    right() {
        return this.left + this.window.length;
    }
}
exports.InputStateManager = InputStateManager;
//# sourceMappingURL=InputStateManager.js.map