"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function wrappingMatchReport(matcher, params) {
    return new WrappingMatchReport(matcher, params.parseNodeName || matcher.$id, params.inner, params.additional);
}
exports.wrappingMatchReport = wrappingMatchReport;
class SuccessfulMatchReportWrapper {
    constructor(matcher, parseNodeName, inner) {
        this.matcher = matcher;
        this.parseNodeName = parseNodeName;
        this.inner = inner;
        this.kind = "real";
        this.successful = true;
    }
    get offset() {
        return this.inner.offset;
    }
    get endingOffset() {
        return this.inner.endingOffset;
    }
    toValueStructure() {
        return this.inner.toValueStructure();
    }
    toPatternMatch() {
        // the wrapping disappears
        return this.inner.toPatternMatch();
    }
    get matched() {
        return this.inner.matched;
    }
    toParseTree() {
        // only successful matches go into the parse tree
        return {
            $name: this.parseNodeName,
            $offset: this.inner.offset,
            $value: this.inner.matched,
            $children: [this.inner.toParseTree()],
        };
    }
    toExplanationTree() {
        return Object.assign({}, this.toParseTree(), { successful: true, $children: [this.inner.toExplanationTree()] });
    }
}
exports.SuccessfulMatchReportWrapper = SuccessfulMatchReportWrapper;
class WrappingMatchReport extends SuccessfulMatchReportWrapper {
    constructor(matcher, parseNodeName, inner, additional = []) {
        super(matcher, parseNodeName, inner);
        this.additional = additional;
    }
    toExplanationTree() {
        // all non-matches and matches go into the explanation tree
        return Object.assign({}, this.toParseTree(), { successful: true, $children: [...this.additional, this.inner].map(m => m.toExplanationTree()) });
    }
}
function wrappingFailedMatchReport(matcher, params) {
    return new WrappingFailedMatchReport(matcher, params.parseNodeName || matcher.$id, params.inner, params.reason, params.offset, params.matched);
}
exports.wrappingFailedMatchReport = wrappingFailedMatchReport;
class WrappingFailedMatchReport {
    constructor(matcher, parseNodeName, inner, reason, offsetOverride, matchedOverride) {
        this.matcher = matcher;
        this.parseNodeName = parseNodeName;
        this.inner = inner;
        this.reason = reason;
        this.offsetOverride = offsetOverride;
        this.matchedOverride = matchedOverride;
        this.kind = "real";
        this.successful = false;
    }
    get offset() {
        return (this.offsetOverride === undefined) ? this.inner.offset : this.offsetOverride;
    }
    get matched() {
        return (this.matchedOverride === undefined) ? this.inner.matched : this.matchedOverride;
    }
    toExplanationTree() {
        return {
            $name: this.parseNodeName,
            $offset: this.inner.offset,
            $value: this.inner.matched,
            successful: false,
            reason: this.reason,
            $children: [this.inner.toExplanationTree()],
        };
    }
}
//# sourceMappingURL=wrappingMatchReport.js.map