"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const wrappingMatchReport_1 = require("../internal/matchReport/wrappingMatchReport");
const MatchReport_1 = require("../MatchReport");
const PatternMatch_1 = require("../PatternMatch");
const Concat_1 = require("./Concat");
/**
 * Flatten this match, pulling up its only property with the name given to
 * this matcher.
 * @param o matcher
 * @return {FlatteningMatcher}
 */
function flatten(o) {
    return new FlatteningMatcher(Concat_1.toMatchingLogic(o));
}
exports.flatten = flatten;
class FlatteningMatcher {
    constructor(delegate) {
        this.delegate = delegate;
    }
    matchPrefix(is, thisMatchContext, parseContext) {
        return MatchReport_1.toMatchPrefixResult(this.matchPrefixReport(is, thisMatchContext, parseContext));
    }
    matchPrefixReport(is, thisMatchContext, parseContext) {
        const r = this.delegate.matchPrefixReport(is, thisMatchContext, parseContext);
        if (MatchReport_1.isSuccessfulMatchReport(r)) {
            return new FlatteningMatchReport(this, r);
        }
        return wrappingMatchReport_1.wrappingFailedMatchReport(this, { inner: (r), parseNodeName: "Flatten" });
    }
}
class FlatteningMatchReport extends wrappingMatchReport_1.SuccessfulMatchReportWrapper {
    constructor(matcher, inner) {
        super(matcher, "Flatten", inner);
    }
    toPatternMatch() {
        // it is possible this could be simplified but not sure how to get it compatible.
        const match = this.inner.toPatternMatch();
        if (PatternMatch_1.isTreePatternMatch(match)) {
            const propNames = Object.getOwnPropertyNames(match.submatches());
            if (propNames.length !== 1) {
                throw new Error(`Cannot flatten a structure with more than one property: Found [${propNames.join(",")}]`);
            }
            const onlyPropertyName = propNames[0];
            const relevantSubMatch = match.$valueMatches[onlyPropertyName];
            return successfulPatternMatch({
                matcherId: this.matcher.$id, matched: match.$matched, offset: match.$offset,
                valueRepresented: { value: relevantSubMatch.$value },
            });
        }
        else {
            return match;
        }
    }
    toValueStructure() {
        const innerVs = this.inner.toValueStructure();
        if (typeof innerVs === "object") {
            const propNames = Object.keys(innerVs);
            if (propNames.length !== 1) {
                throw new Error(`Cannot flatten a structure with more than one property: Found [${propNames.join(",")}]`);
            }
            return innerVs[propNames[0]];
        }
        else {
            return innerVs;
        }
    }
}
// consider moving this to terminalMatchReport and also calling it there
function successfulPatternMatch(params) {
    const { matched, offset } = params;
    const $value = params.valueRepresented ? params.valueRepresented.value : matched;
    const pm = {
        $matcherId: params.matcherId,
        $matched: matched,
        $offset: offset,
        $value,
        matchedStructure: () => $value,
    };
    return pm;
}
//# sourceMappingURL=Functions.js.map