"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const InputStateFactory_1 = require("../../../internal/InputStateFactory");
const terminalMatchReport_1 = require("../../../internal/matchReport/terminalMatchReport");
const MatchReport_1 = require("../../../MatchReport");
const Concat_1 = require("../../Concat");
const CFamilyStateMachine_1 = require("./CFamilyStateMachine");
/**
 * The rest of a C family block, going to a matching depth of +1 curlies or braces.
 * Does not read final curly
 */
class CBlock {
    constructor(stateMachineFactory, kind, inner) {
        this.stateMachineFactory = stateMachineFactory;
        this.inner = inner;
        switch (kind) {
            case "block":
                [this.push, this.pop] = ["{", "}"];
                break;
            case "parens":
                [this.push, this.pop] = ["(", ")"];
                break;
        }
    }
    matchPrefix(is, thisMatchContext, parseContext) {
        return MatchReport_1.toMatchPrefixResult(this.matchPrefixReport(is, thisMatchContext, parseContext));
    }
    matchPrefixReport(is, thisMatchContext, parseContext) {
        const sm = this.stateMachineFactory();
        let depth = 1;
        let currentIs = is;
        let matched = "";
        while (depth > 0) {
            const next = currentIs.peek(1);
            if (next.length === 0) {
                break;
            }
            sm.consume(next);
            if (sm.state.normal()) {
                switch (next) {
                    case this.push:
                        depth++;
                        break;
                    case this.pop:
                        depth--;
                        break;
                    default:
                }
            }
            if (depth > 0) {
                matched += next;
                currentIs = currentIs.advance();
            }
        }
        if (!this.inner) {
            return terminalMatchReport_1.successfulMatchReport(this, {
                parseNodeName: "CBlock",
                matched,
                offset: is.offset,
            });
        }
        // We supply the offset to preserve it in this match
        return this.inner.matchPrefixReport(InputStateFactory_1.inputStateFromString(matched, undefined, is.offset), thisMatchContext, parseContext);
    }
}
exports.CBlock = CBlock;
/**
 * Match a block with balanced curlies
 * @type {Term}
 */
function block(stateMachineFactory) {
    return Concat_1.Concat.of({
        $id: "{...}",
        _lp: "{",
        block: new CBlock(stateMachineFactory, "block"),
        _rp: "}",
    });
}
exports.block = block;
function blockContaining(m, stateMachineFactory = () => new CFamilyStateMachine_1.CFamilyStateMachine()) {
    return Concat_1.Concat.of({
        $id: "{...}",
        _lp: "{",
        block: new CBlock(stateMachineFactory, "block", m),
        _rp: "}",
    });
}
exports.blockContaining = blockContaining;
/**
 * Match a parenthesized expression including ()
 * @type {Concat}
 */
function parenthesizedExpression(stateMachineFactory = () => new CFamilyStateMachine_1.CFamilyStateMachine()) {
    return Concat_1.Concat.of({
        $id: "(...)",
        _lp: "(",
        block: new CBlock(stateMachineFactory, "parens"),
        _rp: ")",
    });
}
exports.parenthesizedExpression = parenthesizedExpression;
//# sourceMappingURL=CBlock.js.map