"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CFamilyStateMachine_1 = require("./CFamilyStateMachine");
const States_1 = require("./States");
class CFamilyLangHelper {
    /**
     * Strip C/C++ comments
     * @param source
     */
    stripComments(source) {
        let stripped = "";
        const sm = new CFamilyStateMachine_1.CFamilyStateMachine();
        for (const ch of source) {
            sm.consume(ch);
            switch (sm.state) {
                case States_1.SlashStarComment:
                case States_1.SlashSlashComment:
                    if (sm.state !== sm.previousState) {
                        // Get rid of the first /, which was written to the string
                        stripped = stripped.substring(0, stripped.length - 1);
                    }
                    break;
                case States_1.Normal:
                    if (!(ch === "/" && sm.previousState.comment)) {
                        stripped += ch;
                    }
                    break;
                default:
                    stripped += ch;
            }
        }
        return stripped;
    }
    stripWhitespace(source) {
        let stripped = "";
        let chunk = "";
        const sm = new CFamilyStateMachine_1.CFamilyStateMachine();
        for (const s of source) {
            sm.consume(s);
            switch (sm.state) {
                case States_1.DoubleString:
                    // If we've just entered a string, add the stripped chunk that preceded it
                    if (sm.previousState !== States_1.DoubleString) {
                        stripped += this.strip(chunk);
                        chunk = "";
                    }
                    // Take all characters from string without stripping whitespace
                    stripped += s;
                    break;
                default:
                    // Add to a chunk that we'll later strip
                    chunk += s;
            }
        }
        stripped += this.strip(chunk);
        return stripped;
    }
    canonicalize(src) {
        return this.stripWhitespace(this.stripComments(src));
    }
    strip(src) {
        let stripped = src.replace(/[\s]+/g, " ");
        // Get rid of syntactically unnecessary whitespace
        stripped = stripped.replace(/([{}.@;])\s/g, "$1");
        stripped = stripped.replace(/\s([{}.@;])/g, "$1");
        stripped = stripped.trim();
        return stripped;
    }
}
exports.CFamilyLangHelper = CFamilyLangHelper;
//# sourceMappingURL=CFamilyLangHelper.js.map