"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const LangStateMachine_1 = require("../LangStateMachine");
const States_1 = require("./States");
/**
 * State machine for recognizing C family strings and comments.
 * Directly usable for Java, C and C++
 */
class CFamilyStateMachine extends LangStateMachine_1.LangStateMachine {
    constructor(state = States_1.Normal) {
        super(state);
    }
    clone() {
        return new CFamilyStateMachine(this.state);
    }
    consume(ch) {
        this.previousState = this.state;
        switch (this.state) {
            case States_1.SlashSlashComment:
                if (ch === "\n") {
                    this.state = States_1.Normal;
                }
                break;
            case States_1.SlashStarComment:
                if (ch === "/" && this.previousChar === "*") {
                    this.state = States_1.Normal;
                }
                break;
            case States_1.Normal:
                switch (ch) {
                    case "\"":
                        this.state = States_1.DoubleString;
                        break;
                    case "/":
                        if (this.previousChar === "/") {
                            this.state = States_1.SlashSlashComment;
                        }
                        break;
                    case "*":
                        if (this.previousChar === "/") {
                            this.state = States_1.SlashStarComment;
                        }
                        break;
                    default:
                }
                break;
            case States_1.DoubleString:
                if (ch === "\"" && this.previousChar !== "\\") {
                    this.state = States_1.Normal;
                }
                break;
        }
        this.previousChar = ch;
    }
}
exports.CFamilyStateMachine = CFamilyStateMachine;
//# sourceMappingURL=CFamilyStateMachine.js.map