"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const failedMatchReport_1 = require("../../../internal/matchReport/failedMatchReport");
const terminalMatchReport_1 = require("../../../internal/matchReport/terminalMatchReport");
const MatchReport_1 = require("../../../MatchReport");
const LangStateMachine_1 = require("../LangStateMachine");
const States_1 = require("./States");
// TODO: pass in an inner matcher, support nesting
class DelimitedLiteral {
    constructor(delimiter, escapeChar = "\\") {
        this.delimiter = delimiter;
        this.escapeChar = escapeChar;
        this.$id = `${this.delimiter} ... ${this.delimiter}`;
        this.parseNodeName = "DelimitedLiteral";
        if (delimiter.length !== 1) {
            throw new Error("That is not gonna work. Delimiters are 1 char");
        }
        if (escapeChar.length !== 1) {
            throw new Error("That is not gonna work. escapeChar must be 1 char");
        }
    }
    matchPrefix(is, thisMatchContext, parseContext) {
        return MatchReport_1.toMatchPrefixResult(this.matchPrefixReport(is, thisMatchContext, parseContext));
    }
    matchPrefixReport(is, thisMatchContext, parseContext) {
        const delimiter = this.delimiter;
        const initialOffset = is.offset;
        let currentIs = is; // is this needed? seems likely.
        if (is.peek(1) !== delimiter) {
            return failedMatchReport_1.failedMatchReport(this, {
                offset: initialOffset,
                parseNodeName: this.parseNodeName,
                reason: `No opening ${delimiter}; saw ${is.peek(1)} instead`,
            });
        }
        currentIs = currentIs.consume(delimiter, "Opening delimiter");
        let matched = delimiter;
        const sm = new DelimiterWithEscapeChar(delimiter, this.escapeChar);
        while (sm.state !== Done) {
            const next = currentIs.peek(1);
            if (next.length === 0) {
                return failedMatchReport_1.failedMatchReport(this, {
                    offset: initialOffset,
                    parseNodeName: this.parseNodeName,
                    reason: `End of input before the closing ${delimiter}`,
                });
            }
            sm.consume(next);
            matched += next;
            currentIs = currentIs.consume(next, `Looking for a closing ${delimiter}`);
        }
        return terminalMatchReport_1.successfulMatchReport(this, {
            matched,
            parseNodeName: this.parseNodeName,
            offset: is.offset,
        });
    }
}
exports.DelimitedLiteral = DelimitedLiteral;
const Done = new LangStateMachine_1.LangState("DONE", false, false);
class DelimiterWithEscapeChar extends LangStateMachine_1.LangStateMachine {
    constructor(endChar, escapeChar, state = States_1.Normal) {
        super(state);
        this.endChar = endChar;
        this.escapeChar = escapeChar;
    }
    clone() {
        return new DelimiterWithEscapeChar(this.endChar, this.escapeChar, this.state);
    }
    consume(ch) {
        switch (this.state) {
            case Done:
                break;
            case States_1.EscapeNextCharacter:
                this.state = States_1.Normal;
                break;
            case States_1.Normal:
                switch (ch) {
                    case this.escapeChar:
                        this.state = States_1.EscapeNextCharacter;
                        break;
                    case this.endChar:
                        this.state = Done;
                        break;
                    default:
                    // no change
                }
        }
    }
}
//# sourceMappingURL=DelimitedLiteral.js.map