"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractStateMachine_1 = require("../../../support/AbstractStateMachine");
const CFamilyStateMachine_1 = require("./CFamilyStateMachine");
const States_1 = require("./States");
/**
 * Track depth of curlies and parentheses in C family languages
 */
class NestingDepthStateMachine extends AbstractStateMachine_1.AbstractStateMachine {
    constructor(kind = "block", factory = () => new CFamilyStateMachine_1.CFamilyStateMachine(), state = States_1.Normal, depth = 0) {
        super(state);
        this.kind = kind;
        this.factory = factory;
        this.depth = depth;
        this.stateMachine = this.factory();
        switch (kind) {
            case "block":
                [this.push, this.pop] = ["{", "}"];
                break;
            case "parens":
                [this.push, this.pop] = ["(", ")"];
                break;
        }
    }
    clone() {
        return new NestingDepthStateMachine(this.kind, this.factory, this.state, this.depth);
    }
    consume(char) {
        this.stateMachine.consume(char);
        this.state = this.stateMachine.state;
        if (this.state.normal()) {
            switch (char) {
                case this.push:
                    this.depth++;
                    break;
                case this.pop:
                    this.depth--;
                    break;
                default:
            }
        }
    }
}
exports.NestingDepthStateMachine = NestingDepthStateMachine;
//# sourceMappingURL=NestingDepthStateMachine.js.map