"use strict";
/**
 * Convenient operations to skip over input
 */
Object.defineProperty(exports, "__esModule", { value: true });
const Break_1 = require("../../internal/Break");
const terminalMatchReport_1 = require("../../internal/matchReport/terminalMatchReport");
const MatchReport_1 = require("../../MatchReport");
const Primitives_1 = require("../../Primitives");
const Concat_1 = require("../Concat");
/**
 * Match the rest of the input.
 */
exports.RestOfInput = {
    $id: "RestOfInput",
    matchPrefix(is) {
        return MatchReport_1.toMatchPrefixResult(restOfInputMatchPrefixReport(is));
    },
    matchPrefixReport: restOfInputMatchPrefixReport,
};
function restOfInputMatchPrefixReport(is) {
    const consumed = is.skipWhile(() => true, 1);
    return terminalMatchReport_1.successfulMatchReport(exports.RestOfInput, {
        parseNodeName: "RestOfInput",
        matched: consumed.skipped,
        offset: is.offset,
    });
}
/**
 * Match the rest of the current line
 */
exports.RestOfLine = new Break_1.Break(new Primitives_1.Literal("\n"));
/**
 * Match a string until the given matcher. Wraps Break.
 * Binds the content until the break.
 */
function takeUntil(what) {
    return new Break_1.Break(Concat_1.toMatchingLogic(what));
}
exports.takeUntil = takeUntil;
/**
 * Skip all content until the given matcher. Bind its match
 */
function skipTo(what) {
    return new Break_1.Break(Concat_1.toMatchingLogic(what), true);
}
exports.skipTo = skipTo;
/**
 * Return a match for the first thing if it doesn't
 * @param a desired match
 * @param b match we don't want.
 */
function yadaYadaThenThisButNotThat(a, b) {
    return new Break_1.Break(Concat_1.toMatchingLogic(a), true, Concat_1.toMatchingLogic(b));
}
exports.yadaYadaThenThisButNotThat = yadaYadaThenThisButNotThat;
/**
 * Anything, then the given matcher. Binds the terminal match
 * @param a desired match
 * @returns {Break}
 */
function yadaYadaThen(a) {
    return new Break_1.Break(Concat_1.toMatchingLogic(a), true);
}
exports.yadaYadaThen = yadaYadaThen;
//# sourceMappingURL=Skip.js.map