"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const failedMatchReport_1 = require("../../internal/matchReport/failedMatchReport");
const terminalMatchReport_1 = require("../../internal/matchReport/terminalMatchReport");
const MatchReport_1 = require("../../MatchReport");
/**
 * Inspired by Snobol SPAN: http://www.snobol4.org/docs/burks/tutorial/ch4.htm
 * SPAN(S) matches one or more subject characters from the set in S.
 * SPAN must match at least one subject character, and will match the LONGEST subject string possible.
 */
class Span {
    constructor(characters) {
        this.characters = characters;
    }
    get $id() {
        return `Span[${this.characters}]`;
    }
    matchPrefix(is, thisMatchContext, parseContext) {
        return MatchReport_1.toMatchPrefixResult(this.matchPrefixReport(is, thisMatchContext, parseContext));
    }
    matchPrefixReport(is, thisMatchContext, parseContext) {
        let currentIs = is;
        let matched = "";
        while (!currentIs.exhausted() && this.characters.indexOf(currentIs.peek(1)) > -1) {
            matched += currentIs.peek(1);
            currentIs = currentIs.advance();
        }
        return (currentIs !== is) ?
            terminalMatchReport_1.successfulMatchReport(this, 
            // why is the value an inputstate?
            { matched, offset: is.offset, valueRepresented: { value: currentIs } }) :
            failedMatchReport_1.failedMatchReport(this, {
                offset: is.offset, matched,
                reason: `No characters found. Looking for [${this.characters}], found [${currentIs.peek(1)}]`,
            });
    }
}
exports.Span = Span;
//# sourceMappingURL=Span.js.map