"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Microgrammar_1 = require("./Microgrammar");
/**
 * Create a microgrammar return matches with properties according
 * to the given interface.
 * @param definition full definition, including a phrase string, or terms definition
 * @return {Grammar<T>}
 */
function microgrammar(definition) {
    if (!definition.phrase && !definition.terms) {
        return Microgrammar_1.Microgrammar.fromDefinitions(definition);
    }
    if (!!definition.phrase) {
        return Microgrammar_1.Microgrammar.fromString(definition.phrase, definition.terms, definition.options);
    }
    return Microgrammar_1.Microgrammar.fromDefinitions(definition.terms);
}
exports.microgrammar = microgrammar;
/**
 * Create a microgrammar with return matches implementing an inferred interface
 * taking properties of type "any" from definitions.
 * Use microgrammar for stronger typing.
 * @return {Grammar<T>}
 */
function simpleMicrogrammar(definition) {
    return microgrammar(definition);
}
exports.simpleMicrogrammar = simpleMicrogrammar;
//# sourceMappingURL=microgrammarConstruction.js.map