"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sdm_1 = require("@atomist/sdm");
const fs = require("fs-extra");
const os = require("os");
const path = require("path");
/**
 * Goal archive store that stores the compressed archives into the SDM cache directory.
 */
class FileSystemGoalCacheArchiveStore {
    store(gi, classifier, archivePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheDir = yield FileSystemGoalCacheArchiveStore.getCacheDirectory(gi, classifier);
            const archiveName = FileSystemGoalCacheArchiveStore.archiveName;
            const archiveFileName = path.join(cacheDir, archiveName);
            yield sdm_1.spawnLog("mv", [archivePath, archiveFileName], {
                log: gi.progressLog,
            });
        });
    }
    delete(gi, classifier) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheDir = yield FileSystemGoalCacheArchiveStore.getCacheDirectory(gi, classifier);
            const archiveName = FileSystemGoalCacheArchiveStore.archiveName;
            const archiveFileName = path.join(cacheDir, archiveName);
            yield sdm_1.spawnLog("rm", ["-f", archiveFileName], {
                log: gi.progressLog,
            });
        });
    }
    retrieve(gi, classifier, targetArchivePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheDir = yield FileSystemGoalCacheArchiveStore.getCacheDirectory(gi, classifier);
            const archiveName = FileSystemGoalCacheArchiveStore.archiveName;
            const archiveFileName = path.join(cacheDir, archiveName);
            yield sdm_1.spawnLog("cp", [archiveFileName, targetArchivePath], {
                log: gi.progressLog,
            });
        });
    }
    static getCacheDirectory(gi, classifier = "default") {
        return __awaiter(this, void 0, void 0, function* () {
            const defaultCachePath = path.join(os.homedir(), ".atomist", "cache");
            const possibleCacheConfiguration = gi.configuration.sdm.cache;
            const sdmCacheDir = possibleCacheConfiguration ? (possibleCacheConfiguration.path || defaultCachePath) : defaultCachePath;
            const cacheDir = path.join(sdmCacheDir, classifier);
            yield fs.mkdirs(cacheDir);
            return cacheDir;
        });
    }
}
exports.FileSystemGoalCacheArchiveStore = FileSystemGoalCacheArchiveStore;
FileSystemGoalCacheArchiveStore.archiveName = "cache.tar.gz";
//# sourceMappingURL=FileSystemGoalCacheArchiveStore.js.map