"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_1 = require("@atomist/automation-client/lib/configuration");
const sdm_1 = require("@atomist/sdm");
const resolvePlaceholder_1 = require("../../machine/yaml/resolvePlaceholder");
const array_1 = require("../../util/misc/array");
const goalCaching_1 = require("../cache/goalCaching");
function item(name, registration, options = {}) {
    const { uniqueName, parameters, input, output, secrets } = options;
    const g = sdm_1.goal({ displayName: uniqueName, uniqueName: uniqueName || name }).with({
        name: name.replace(/ /g, "_"),
        registration,
    });
    if (!!parameters || !!input || !!output || !!secrets) {
        g.plan = (pli) => __awaiter(this, void 0, void 0, function* () {
            const { push } = pli;
            yield configuration_1.resolvePlaceholders(parameters, v => resolvePlaceholder_1.resolvePlaceholder(v, {
                sha: pli.push.after.sha,
                branch: pli.push.branch,
                repo: {
                    owner: push.repo.owner,
                    name: push.repo.name,
                    providerId: push.repo.org.provider.providerId,
                },
                push: pli.push,
            }, pli, {}, true));
            return {
                parameters: Object.assign(Object.assign({}, (parameters || {})), { [goalCaching_1.CacheInputGoalDataKey]: array_1.toArray(input), [goalCaching_1.CacheOutputGoalDataKey]: array_1.toArray(output), "@atomist/sdm/secrets": secrets }),
            };
        });
    }
    return g;
}
exports.item = item;
//# sourceMappingURL=item.js.map