import { LeveledLogMethod } from "@atomist/automation-client";
import { ExecuteGoalResult, GoalInvocation, ProgressLog, SdmContext, SdmGoalEvent } from "@atomist/sdm";
/**
 * Simple test to see if SDM is running in Kubernetes.  It is called
 * from a non-async function, so it must be non-async.
 *
 * @return `true` if process is running in Kubernetes, `false` otherwise.
 */
export declare function runningInK8s(): boolean;
/**
 * Simple test to see if SDM is running as a Google Cloud Function.
 *
 * @return `true` if process is running as Google Cloud Function,
 * `false` otherwise.
 */
export declare function runningAsGoogleCloudFunction(): boolean;
/**
 * Return environment variables required by the container goal
 * execution machinery.
 *
 * @param goalEvent SDM goal event being executed as a container goal
 * @param ctx SDM context for goal execution
 * @return SDM goal environment variables
 */
export declare function containerEnvVars(goalEvent: SdmGoalEvent, ctx: SdmContext, projectDir?: string, inputDir?: string, outputDir?: string): Promise<Array<{
    name: string;
    value: string;
}>>;
export declare function prepareInputAndOutput(input: string, output: string, gi: GoalInvocation): Promise<void>;
/**
 * Write to client and progress logs.  Add newline to progress log.
 *
 * @param msg Message to write, should not have newline at end
 * @param l Logger method, e.g., `logger.warn`
 * @param p Progress log
 */
export declare function loglog(msg: string, l: LeveledLogMethod, p: ProgressLog): void;
export declare function processResult(result: any, gi: GoalInvocation): Promise<ExecuteGoalResult | undefined>;
//# sourceMappingURL=util.d.ts.map