"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const slack_messages_1 = require("@atomist/slack-messages");
const sdmDeployEnablement_1 = require("../../ingesters/sdmDeployEnablement");
let SetDeployEnablementParameters = class SetDeployEnablementParameters {
};
__decorate([
    automation_client_1.Parameter({ required: false, displayable: false }),
    __metadata("design:type", String)
], SetDeployEnablementParameters.prototype, "msgId", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubOwner),
    __metadata("design:type", String)
], SetDeployEnablementParameters.prototype, "owner", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubRepository),
    __metadata("design:type", String)
], SetDeployEnablementParameters.prototype, "repo", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubRepositoryProvider),
    __metadata("design:type", String)
], SetDeployEnablementParameters.prototype, "providerId", void 0);
__decorate([
    automation_client_1.Value("name"),
    __metadata("design:type", String)
], SetDeployEnablementParameters.prototype, "name", void 0);
__decorate([
    automation_client_1.Value("version"),
    __metadata("design:type", String)
], SetDeployEnablementParameters.prototype, "version", void 0);
SetDeployEnablementParameters = __decorate([
    automation_client_1.Parameters()
], SetDeployEnablementParameters);
exports.SetDeployEnablementParameters = SetDeployEnablementParameters;
/**
 * Command to set deploy enablement on the currently mapped repo
 * @param {CommandListenerInvocation} cli
 * @param {boolean} enable
 * @returns {Promise<HandlerResult | HandlerError>}
 */
function setDeployEnablement(cli, enable) {
    const deployEnablement = {
        state: enable ? "requested" : "disabled",
        owner: cli.parameters.owner,
        repo: cli.parameters.repo,
        providerId: cli.parameters.providerId,
    };
    return cli.context.messageClient.send(deployEnablement, automation_client_1.addressEvent(sdmDeployEnablement_1.DeployEnablementRootType))
        .then(() => cli.context.messageClient.respond(sdm_1.slackSuccessMessage("Deploy Enablement", `Successfully ${enable ? "enabled" : "disabled"} deployment of ${slack_messages_1.bold(`${cli.parameters.owner}/${cli.parameters.repo}`)}`, {
        footer: `${cli.parameters.name}:${cli.parameters.version}`,
    }), { id: cli.parameters.msgId }))
        .then(() => automation_client_1.Success, automation_client_1.failure);
}
exports.setDeployEnablement = setDeployEnablement;
exports.EnableDeploy = {
    name: "EnableDeploy",
    intent: "enable deploy",
    description: "Enable deployment via Atomist SDM",
    paramsMaker: SetDeployEnablementParameters,
    listener: (cli) => __awaiter(void 0, void 0, void 0, function* () { return setDeployEnablement(cli, true); }),
};
exports.DisableDeploy = {
    name: "DisableDeploy",
    intent: "disable deploy",
    description: "Disable deployment via Atomist SDM",
    paramsMaker: SetDeployEnablementParameters,
    listener: (cli) => __awaiter(void 0, void 0, void 0, function* () { return setDeployEnablement(cli, false); }),
};
//# sourceMappingURL=SetDeployEnablement.js.map