"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const decorators_1 = require("@atomist/automation-client/lib/decorators");
const sdm_1 = require("@atomist/sdm");
const stringify = require("json-stringify-safe");
const validateGoal_1 = require("../../../../internal/delivery/goals/support/validateGoal");
const goalSigning_1 = require("../../../../internal/signing/goalSigning");
/**
 * Respond to a failure or success status by running listeners
 */
let RespondOnGoalCompletion = class RespondOnGoalCompletion {
    constructor(repoRefResolver, credentialsFactory, goalCompletionListeners, preferenceStoreFactory) {
        this.repoRefResolver = repoRefResolver;
        this.credentialsFactory = credentialsFactory;
        this.goalCompletionListeners = goalCompletionListeners;
        this.preferenceStoreFactory = preferenceStoreFactory;
    }
    handle(event, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const sdmGoal = event.data.SdmGoal[0];
            if (!validateGoal_1.shouldHandle(sdmGoal)) {
                automation_client_1.logger.debug(`Goal ${sdmGoal.uniqueName} skipped because not managed by this SDM`);
                return automation_client_1.Success;
            }
            yield goalSigning_1.verifyGoal(sdmGoal, this.configuration.sdm.goalSigning, context);
            const id = this.repoRefResolver.repoRefFromPush(sdmGoal.push);
            const goals = sdm_1.fetchGoalsFromPush(sdmGoal);
            const gsi = {
                id,
                context,
                credentials: yield sdm_1.resolveCredentialsPromise(this.credentialsFactory.eventHandlerCredentials(context, id)),
                addressChannels: sdm_1.addressChannelsFor(sdmGoal.push.repo, context),
                configuration: this.configuration,
                preferences: this.preferenceStoreFactory(context),
                allGoals: goals,
                completedGoal: sdmGoal,
            };
            try {
                yield Promise.all(this.goalCompletionListeners.map(l => l(gsi)));
            }
            catch (e) {
                automation_client_1.logger.warn(`Error occurred while running goal completion listener: ${stringify(e)}`);
            }
            return automation_client_1.Success;
        });
    }
};
__decorate([
    automation_client_1.Value(""),
    __metadata("design:type", Object)
], RespondOnGoalCompletion.prototype, "configuration", void 0);
RespondOnGoalCompletion = __decorate([
    decorators_1.EventHandler("Run a listener on goal failure or success", () => automation_client_1.GraphQL.subscription({
        name: "OnAnyCompletedSdmGoal",
        variables: { registration: () => { var _a, _b; return [(_b = (_a = automation_client_1.automationClientInstance()) === null || _a === void 0 ? void 0 : _a.configuration) === null || _b === void 0 ? void 0 : _b.name]; } },
    })),
    __metadata("design:paramtypes", [Object, Object, Array, Function])
], RespondOnGoalCompletion);
exports.RespondOnGoalCompletion = RespondOnGoalCompletion;
//# sourceMappingURL=RespondOnGoalCompletion.js.map