"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const decorators_1 = require("@atomist/automation-client/lib/decorators");
const sdm_1 = require("@atomist/sdm");
/**
 * Set up goalSet on a push (e.g. for delivery).
 */
let SetGoalsOnPush = class SetGoalsOnPush {
    /**
     * Configure goal setting
     * @param projectLoader use to load projects
     * @param repoRefResolver used to resolve repos from GraphQL return
     * @param goalSetter
     * @param goalsListeners listener to goals set
     * @param implementationMapping
     * @param credentialsFactory credentials factory
     */
    constructor(projectLoader, repoRefResolver, goalSetter, 
    // public for tests only
    goalsListeners, implementationMapping, credentialsFactory, preferenceStoreFactory, enrichGoal, tagGoalSet) {
        this.projectLoader = projectLoader;
        this.repoRefResolver = repoRefResolver;
        this.goalSetter = goalSetter;
        this.goalsListeners = goalsListeners;
        this.implementationMapping = implementationMapping;
        this.credentialsFactory = credentialsFactory;
        this.preferenceStoreFactory = preferenceStoreFactory;
        this.enrichGoal = enrichGoal;
        this.tagGoalSet = tagGoalSet;
    }
    handle(event, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const push = event.data.Push[0];
            const id = this.repoRefResolver.toRemoteRepoRef(push.repo, {});
            const credentials = yield sdm_1.resolveCredentialsPromise(this.credentialsFactory.eventHandlerCredentials(context, id));
            yield sdm_1.chooseAndSetGoals({
                projectLoader: this.projectLoader,
                repoRefResolver: this.repoRefResolver,
                goalsListeners: this.goalsListeners,
                goalSetter: this.goalSetter,
                implementationMapping: this.implementationMapping,
                preferencesFactory: this.preferenceStoreFactory,
                enrichGoal: this.enrichGoal,
                tagGoalSet: this.tagGoalSet,
            }, {
                context,
                credentials,
                push,
            });
            return automation_client_1.Success;
        });
    }
};
SetGoalsOnPush = __decorate([
    decorators_1.EventHandler("Set up goalSet on Push", automation_client_1.GraphQL.subscription("OnPushToAnyBranch")),
    __metadata("design:paramtypes", [Object, Object, Object, Array, Object, Object, Function, Function, Function])
], SetGoalsOnPush);
exports.SetGoalsOnPush = SetGoalsOnPush;
//# sourceMappingURL=SetGoalsOnPush.js.map