"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const decorators_1 = require("@atomist/automation-client/lib/decorators");
const sdm_1 = require("@atomist/sdm");
const validateGoal_1 = require("../../../../internal/delivery/goals/support/validateGoal");
const goalSigning_1 = require("../../../../internal/signing/goalSigning");
/**
 * Vote on approved goals.
 *
 * This allows GoalApprovalVoter instances to vote on the approved goal to decide
 * if this approval request can be granted or not.
 *
 * The final decision if the request should be granted based on all votes is delegated to the
 * configured instance of GoalApprovalRequestVoteDecisionManager.
 */
let VoteOnGoalApprovalRequest = class VoteOnGoalApprovalRequest {
    constructor(repoRefResolver, credentialsFactory, voters, decisionManager, implementationMapper, preferenceStoreFactory) {
        this.repoRefResolver = repoRefResolver;
        this.credentialsFactory = credentialsFactory;
        this.voters = voters;
        this.decisionManager = decisionManager;
        this.implementationMapper = implementationMapper;
        this.preferenceStoreFactory = preferenceStoreFactory;
        if (this.voters.length === 0) {
            this.voters.push(() => __awaiter(this, void 0, void 0, function* () { return ({ vote: sdm_1.GoalApprovalRequestVote.Granted }); }));
        }
        if (!this.decisionManager) {
            this.decisionManager = sdm_1.UnanimousGoalApprovalRequestVoteDecisionManager;
        }
    }
    handle(event, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const sdmGoal = event.data.SdmGoal[0];
            if (!validateGoal_1.shouldHandle(sdmGoal)) {
                automation_client_1.logger.debug(`Goal ${sdmGoal.name} skipped because not managed by this SDM`);
                return automation_client_1.Success;
            }
            yield goalSigning_1.verifyGoal(sdmGoal, this.configuration.sdm.goalSigning, context);
            const id = this.repoRefResolver.repoRefFromPush(sdmGoal.push);
            const credentials = yield sdm_1.resolveCredentialsPromise(this.credentialsFactory.eventHandlerCredentials(context, id));
            const preferences = this.preferenceStoreFactory(context);
            const garvi = {
                id,
                context,
                credentials,
                addressChannels: sdm_1.addressChannelsFor(sdmGoal.push.repo, context),
                configuration: this.configuration,
                preferences,
                goal: sdmGoal,
            };
            const votes = yield Promise.all(this.voters.map(v => v(garvi)));
            const decision = this.decisionManager(...votes);
            const goal = this.implementationMapper.findGoalBySdmGoal(sdmGoal);
            switch (decision) {
                case sdm_1.GoalApprovalRequestVote.Granted:
                    if (sdmGoal.state === sdm_1.SdmGoalState.pre_approved) {
                        let g = sdmGoal;
                        const cbs = this.implementationMapper.findFulfillmentCallbackForGoal(sdmGoal);
                        for (const cb of cbs) {
                            g = yield cb.callback(g, {
                                id,
                                addressChannels: undefined,
                                preferences,
                                credentials,
                                configuration: this.configuration,
                                context,
                            });
                        }
                        yield sdm_1.updateGoal(context, sdmGoal, {
                            state: sdm_1.SdmGoalState.requested,
                            description: !!sdmGoal.descriptions && !!sdmGoal.descriptions.requested
                                ? sdmGoal.descriptions.requested : goal.requestedDescription,
                            data: g.data,
                        });
                    }
                    else if (sdmGoal.state === sdm_1.SdmGoalState.approved) {
                        yield sdm_1.updateGoal(context, sdmGoal, {
                            state: sdm_1.SdmGoalState.success,
                            description: !!sdmGoal.descriptions && !!sdmGoal.descriptions.completed
                                ? sdmGoal.descriptions.completed : goal.successDescription,
                        });
                    }
                    break;
                case sdm_1.GoalApprovalRequestVote.Denied:
                    if (sdmGoal.state === sdm_1.SdmGoalState.pre_approved) {
                        const g = Object.assign(Object.assign({}, sdmGoal), { preApproval: undefined });
                        yield sdm_1.updateGoal(context, g, {
                            state: sdm_1.SdmGoalState.waiting_for_pre_approval,
                            description: `${!!sdmGoal.descriptions && !!sdmGoal.descriptions.waitingForPreApproval ?
                                sdmGoal.descriptions.waitingForPreApproval : goal.waitingForPreApprovalDescription} \u00B7 start by @${sdmGoal.preApproval.userId} denied`,
                        });
                    }
                    else if (sdmGoal.state === sdm_1.SdmGoalState.approved) {
                        const g = Object.assign(Object.assign({}, sdmGoal), { approval: undefined });
                        yield sdm_1.updateGoal(context, g, {
                            state: sdm_1.SdmGoalState.waiting_for_approval,
                            description: `${!!sdmGoal.descriptions && !!sdmGoal.descriptions.waitingForApproval ?
                                sdmGoal.descriptions.waitingForApproval : goal.waitingForApprovalDescription} \u00B7 approval by @${sdmGoal.approval.userId} denied`,
                        });
                    }
                    break;
                case sdm_1.GoalApprovalRequestVote.Abstain:
                    // We don't do anything if vote isn't either granted or denied
                    break;
            }
            return automation_client_1.Success;
        });
    }
};
__decorate([
    automation_client_1.Value(""),
    __metadata("design:type", Object)
], VoteOnGoalApprovalRequest.prototype, "configuration", void 0);
VoteOnGoalApprovalRequest = __decorate([
    decorators_1.EventHandler("Vote on started or approved goals", () => automation_client_1.GraphQL.subscription({
        name: "OnAnyApprovedSdmGoal",
        variables: { registration: () => { var _a, _b; return [(_b = (_a = automation_client_1.automationClientInstance()) === null || _a === void 0 ? void 0 : _a.configuration) === null || _b === void 0 ? void 0 : _b.name]; } },
    })),
    __metadata("design:paramtypes", [Object, Object, Array, Function, Object, Function])
], VoteOnGoalApprovalRequest);
exports.VoteOnGoalApprovalRequest = VoteOnGoalApprovalRequest;
//# sourceMappingURL=VoteOnGoalApprovalRequest.js.map