"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const cluster = require("cluster");
const fg = require("fast-glob");
const fs = require("fs-extra");
const path = require("path");
/**
 * Event listener that cleans up cached artifacts that are older then 2 hours.
 */
class CacheCleanupAutomationEventListener extends automation_client_1.AutomationEventListenerSupport {
    constructor(sdm) {
        super();
        this.sdm = sdm;
    }
    startupSuccessful(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const possibleCacheConfiguration = this.sdm.configuration.sdm.cache;
            if (cluster.isMaster && possibleCacheConfiguration && possibleCacheConfiguration.enabled) {
                const cachePath = possibleCacheConfiguration.path || "/opt/data";
                setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                    try {
                        const ts = Date.now() - (1000 * 60 * 60 * 2); // 2 hour threshold
                        if (fs.existsSync(cachePath)) {
                            const matches = yield fg("**/*", { cwd: cachePath });
                            for (const m of matches) {
                                const p = path.join(cachePath, m);
                                try {
                                    const st = yield fs.stat(p);
                                    if (st.mtimeMs < ts && st.isFile()) {
                                        automation_client_1.logger.debug(`Deleting cached file '${p}'`);
                                        yield fs.remove(p);
                                    }
                                }
                                catch (e) {
                                    automation_client_1.logger.debug("Failed to delete cached file '%s': %s", p, e.message);
                                }
                            }
                        }
                    }
                    catch (err) {
                        automation_client_1.logger.debug("Failed to clean cache directory '%s': %s", cachePath, err.message);
                    }
                }));
            }
        });
    }
}
exports.CacheCleanupAutomationEventListener = CacheCleanupAutomationEventListener;
//# sourceMappingURL=CacheCleanupAutomationEventListener.js.map