"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const decorators_1 = require("@atomist/automation-client/lib/decorators");
const sdm_1 = require("@atomist/sdm");
const _ = require("lodash");
/**
 * A pull request has been raised
 */
let OnPullRequest = class OnPullRequest {
    constructor(projectLoader, repoRefResolver, listeners, credentialsFactory, preferenceStoreFactory) {
        this.projectLoader = projectLoader;
        this.repoRefResolver = repoRefResolver;
        this.listeners = listeners;
        this.credentialsFactory = credentialsFactory;
        this.preferenceStoreFactory = preferenceStoreFactory;
    }
    handle(event, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const pullRequest = event.data.PullRequest[0];
            const repo = pullRequest.repo;
            const branch = _.get(pullRequest, "branch.name") || _.get(pullRequest, "head.pushes[0].branch");
            const id = this.repoRefResolver.toRemoteRepoRef(repo, {
                sha: pullRequest.head.sha,
                branch,
            });
            const credentials = yield sdm_1.resolveCredentialsPromise(this.credentialsFactory.eventHandlerCredentials(context, id));
            const addressChannels = sdm_1.addressChannelsFor(repo, context);
            const preferences = this.preferenceStoreFactory(context);
            yield this.projectLoader.doWithProject({ credentials, id, context, readOnly: true }, (project) => __awaiter(this, void 0, void 0, function* () {
                const prli = {
                    id,
                    context,
                    addressChannels,
                    preferences,
                    configuration: this.configuration,
                    credentials,
                    project,
                    pullRequest,
                };
                yield Promise.all(this.listeners
                    .map(l => l(prli)));
            }));
            return automation_client_1.Success;
        });
    }
};
__decorate([
    automation_client_1.Value(""),
    __metadata("design:type", Object)
], OnPullRequest.prototype, "configuration", void 0);
OnPullRequest = __decorate([
    decorators_1.EventHandler("On pull request", automation_client_1.GraphQL.subscription("OnPullRequest")),
    __metadata("design:paramtypes", [Object, Object, Array, Object, Function])
], OnPullRequest);
exports.OnPullRequest = OnPullRequest;
//# sourceMappingURL=OnPullRequest.js.map