"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const decorators_1 = require("@atomist/automation-client/lib/decorators");
const sdm_1 = require("@atomist/sdm");
/**
 * A user joined a channel
 */
let OnUserJoiningChannel = class OnUserJoiningChannel {
    constructor(listeners, repoRefResolver, credentialsFactory, preferenceStoreFactory) {
        this.listeners = listeners;
        this.repoRefResolver = repoRefResolver;
        this.credentialsFactory = credentialsFactory;
        this.preferenceStoreFactory = preferenceStoreFactory;
    }
    handle(event, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const joinEvent = event.data.UserJoinedChannel[0];
            const repos = joinEvent.channel.repos.map(repo => this.repoRefResolver.toRemoteRepoRef(repo, {}));
            const credentials = yield sdm_1.resolveCredentialsPromise(this.credentialsFactory.eventHandlerCredentials(context, repos[0]));
            const addressChannels = (msg, opts) => context.messageClient.addressChannels(msg, joinEvent.channel.name, opts);
            const preferences = this.preferenceStoreFactory(context);
            const invocation = {
                addressChannels,
                preferences,
                configuration: this.configuration,
                context,
                credentials,
                joinEvent,
                repos,
            };
            yield Promise.all(this.listeners
                .map(l => l(invocation)));
            return automation_client_1.Success;
        });
    }
};
__decorate([
    automation_client_1.Value(""),
    __metadata("design:type", Object)
], OnUserJoiningChannel.prototype, "configuration", void 0);
OnUserJoiningChannel = __decorate([
    decorators_1.EventHandler("On user joining channel", automation_client_1.GraphQL.subscription("OnUserJoiningChannel")),
    __metadata("design:paramtypes", [Array, Object, Object, Function])
], OnUserJoiningChannel);
exports.OnUserJoiningChannel = OnUserJoiningChannel;
//# sourceMappingURL=OnUserJoiningChannel.js.map