"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const AbstractPreferenceStore_1 = require("./AbstractPreferenceStore");
/**
 * Factory to create a new InMemoryPreferenceStore instance
 */
exports.InMemoryPreferenceStoreFactory = ctx => new InMemoryPreferenceStore(ctx);
/**
 * PreferenceStore implementation that simply stores preferences in-memory.
 * Note: This is implementation is not intended for production usage.
 */
class InMemoryPreferenceStore extends AbstractPreferenceStore_1.AbstractPreferenceStore {
    constructor(context) {
        super(context);
        this.store = {};
    }
    doGet(name, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = this.scopeKey(name, namespace);
            return this.store[key];
        });
    }
    doPut(pref) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = this.scopeKey(pref.name, pref.namespace);
            this.store[key] = Object.assign(Object.assign({}, pref), { ttl: typeof pref.ttl === "number" ? Date.now() + pref.ttl : undefined });
        });
    }
    doList(namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            const values = [];
            _.forEach(this.store, (v, k) => {
                if (!namespace || k.startsWith(`${namespace}_$_`)) {
                    values.push(v);
                }
            });
            return values;
        });
    }
    doDelete(name, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = this.scopeKey(name, namespace);
            delete this.store[key];
        });
    }
}
exports.InMemoryPreferenceStore = InMemoryPreferenceStore;
//# sourceMappingURL=InMemoryPreferenceStore.js.map