"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const AbstractPreferenceStore_1 = require("./AbstractPreferenceStore");
/**
 * Factory to create a new TeamConfigurationPreferenceStore instance
 */
exports.TeamConfigurationPreferenceStoreFactory = ctx => new TeamConfigurationPreferenceStore(ctx);
/**
 * PreferenceStore implementation that stores preferences in the backend GraphQL store.
 */
class TeamConfigurationPreferenceStore extends AbstractPreferenceStore_1.AbstractPreferenceStore {
    constructor(context) {
        super(context);
        this.context = context;
    }
    doGet(name, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.context.graphClient.query({
                name: "TeamConfigurationByNamespace",
                variables: {
                    namespace: normalizeNamespace(namespace),
                },
                options: automation_client_1.QueryNoCacheOptions,
            });
            const teamConfiguration = (result.TeamConfiguration || []).find(t => t.name === name);
            if (!!teamConfiguration) {
                return {
                    name,
                    namespace,
                    value: teamConfiguration.value,
                    ttl: undefined,
                };
            }
            return undefined;
        });
    }
    doPut(pref) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.context.graphClient.mutate({
                name: "SetTeamConfiguration",
                variables: {
                    name: pref.name,
                    namespace: normalizeNamespace(pref.namespace),
                    value: pref.value,
                    ttl: typeof pref.ttl === "number" ? Math.floor(pref.ttl / 1000) : undefined,
                },
                options: automation_client_1.MutationNoCacheOptions,
            });
        });
    }
    doList(namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.context.graphClient.query({
                name: "TeamConfigurationByNamespace",
                variables: {
                    namespace: normalizeNamespace(namespace),
                },
                options: automation_client_1.QueryNoCacheOptions,
            });
            if (!!result.TeamConfiguration) {
                return result.TeamConfiguration.map(t => ({
                    name: t.name,
                    namespace: t.namespace,
                    value: t.value,
                    ttl: undefined,
                }));
            }
            return [];
        });
    }
    doDelete(name, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.context.graphClient.mutate({
                name: "DeleteTeamConfiguration",
                variables: {
                    name,
                    namespace: normalizeNamespace(namespace),
                },
                options: automation_client_1.MutationNoCacheOptions,
            });
        });
    }
}
exports.TeamConfigurationPreferenceStore = TeamConfigurationPreferenceStore;
function normalizeNamespace(namespace) {
    if (!namespace || namespace.length === 0) {
        return "@atomist.global";
    }
    // Backend doesn't allow / inside a namespace
    return namespace.replace(/\//g, ".");
}
//# sourceMappingURL=TeamConfigurationPreferenceStore.js.map