"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const array_1 = require("../../util/misc/array");
const mapGoals_1 = require("./mapGoals");
const mapPushTests_1 = require("./mapPushTests");
const util_1 = require("./util");
function mapRules(rules, goalData, sdm, options, additionalGoals, goalMakers, testMakers) {
    return __awaiter(this, void 0, void 0, function* () {
        for (const rule of util_1.camelCase(array_1.toArray(rules))) {
            if (!rule.name) {
                throw new Error(`Property 'name' missing in push rule:\n${JSON.stringify(rule, undefined, 2)}`);
            }
            if (!rule.goals) {
                throw new Error(`Property 'goals' missing in push rule:\n${JSON.stringify(rule, undefined, 2)}`);
            }
            const test = !!rule.tests || !!rule.test ? yield mapPushTests_1.mapTests(rule.tests || rule.test, options.tests || {}, testMakers) : [];
            const goals = yield mapGoals_1.mapGoals(sdm, rule.goals, additionalGoals, goalMakers, options.tests || {}, testMakers);
            const dependsOn = rule.dependsOn;
            goalData[rule.name] = {
                test: array_1.toArray(test).length > 0 ? test : undefined,
                dependsOn,
                goals,
            };
        }
    });
}
exports.mapRules = mapRules;
//# sourceMappingURL=mapRules.js.map