"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const slack_messages_1 = require("@atomist/slack-messages");
const queryCommits_1 = require("../../util/graph/queryCommits");
let ResetGoalsParameters = class ResetGoalsParameters {
};
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubRepositoryProvider),
    __metadata("design:type", String)
], ResetGoalsParameters.prototype, "providerId", void 0);
__decorate([
    automation_client_1.Value("name"),
    __metadata("design:type", String)
], ResetGoalsParameters.prototype, "name", void 0);
__decorate([
    automation_client_1.Value("version"),
    __metadata("design:type", String)
], ResetGoalsParameters.prototype, "version", void 0);
ResetGoalsParameters = __decorate([
    automation_client_1.Parameters()
], ResetGoalsParameters);
exports.ResetGoalsParameters = ResetGoalsParameters;
function resetGoalsCommand(sdm, repoTargets = sdm_1.GitHubRepoTargets) {
    return {
        name: "ResetGoalsOnCommit",
        description: "Plan goals on a commit",
        paramsMaker: sdm_1.toRepoTargetingParametersMaker(ResetGoalsParameters, repoTargets),
        listener: resetGoalsOnCommit(sdm),
        intent: [
            `reset goals ${sdm.configuration.name.replace("@", "")}`,
            `plan goals ${sdm.configuration.name.replace("@", "")}`,
        ],
    };
}
exports.resetGoalsCommand = resetGoalsCommand;
function resetGoalsOnCommit(sdm) {
    return (cli) => __awaiter(this, void 0, void 0, function* () {
        const rules = {
            projectLoader: sdm.configuration.sdm.projectLoader,
            repoRefResolver: sdm.configuration.sdm.repoRefResolver,
            goalsListeners: [...sdm.goalsSetListeners],
            goalSetter: sdm.pushMapping,
            implementationMapping: sdm.goalFulfillmentMapper,
            preferencesFactory: sdm.configuration.sdm.preferenceStoreFactory,
        };
        const slug = `${cli.parameters.targets.repoRef.owner}/${cli.parameters.targets.repoRef.repo}`;
        let repoData;
        try {
            repoData = yield queryCommits_1.fetchBranchTips(cli.context, {
                providerId: cli.parameters.providerId,
                owner: cli.parameters.targets.repoRef.owner,
                repo: cli.parameters.targets.repoRef.repo,
            });
        }
        catch (e) {
            const text = `Repository ${slack_messages_1.bold(slug)} not found`;
            return cli.context.messageClient.respond(sdm_1.slackWarningMessage("Set Goal State", text, cli.context));
        }
        const branch = cli.parameters.targets.repoRef.branch || repoData.defaultBranch;
        let sha;
        try {
            sha = cli.parameters.targets.repoRef.sha || queryCommits_1.tipOfBranch(repoData, branch);
        }
        catch (e) {
            return cli.context.messageClient.respond(sdm_1.slackWarningMessage("Set Goal State", `Branch ${slack_messages_1.bold(branch)} not found on ${slack_messages_1.bold(slug)}`, cli.context));
        }
        const id = automation_client_1.GitHubRepoRef.from({
            owner: cli.parameters.targets.repoRef.owner,
            repo: cli.parameters.targets.repoRef.repo,
            sha,
            branch,
        });
        const push = yield queryCommits_1.fetchPushForCommit(cli.context, id, cli.parameters.providerId);
        const goals = yield sdm_1.chooseAndSetGoals(rules, {
            context: cli.context,
            credentials: cli.credentials,
            push,
        });
        const slugBranch = `${id.owner}/${id.repo}/${push.branch}`;
        if (goals) {
            yield cli.addressChannels(sdm_1.slackSuccessMessage("Plan Goals", `Successfully planned goals on ${slack_messages_1.codeLine(push.after.sha.slice(0, 7))} of ${slack_messages_1.bold(slugBranch)} to ${slack_messages_1.italic(goals.name)}`, {
                footer: `${cli.parameters.name}:${cli.parameters.version}`,
            }));
        }
        else {
            yield cli.addressChannels(sdm_1.slackWarningMessage("Plan Goals", `No goals found for ${slack_messages_1.codeLine(push.after.sha.slice(0, 7))} of ${slack_messages_1.bold(slugBranch)}`, cli.context, {
                footer: `${cli.parameters.name}:${cli.parameters.version}`,
            }));
        }
        return automation_client_1.Success;
    });
}
//# sourceMappingURL=resetGoals.js.map