"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const sdm_1 = require("@atomist/sdm");
const KubernetesGoalScheduler_1 = require("./KubernetesGoalScheduler");
const KubernetesJobDeletingGoalCompletionListener_1 = require("./KubernetesJobDeletingGoalCompletionListener");
/**
 * Extension pack to schedule goals as k8s jobs when marked as isolated = true.
 */
function k8sGoalSchedulingSupport() {
    return Object.assign(Object.assign({}, sdm_1.metadata("k8s-goal-scheduling")), { configure: sdm => {
            if (!process.env.ATOMIST_ISOLATED_GOAL && KubernetesGoalScheduler_1.isConfiguredInEnv("kubernetes", "kubernetes-all")) {
                sdm.configuration.sdm.goalScheduler = [new KubernetesGoalScheduler_1.KubernetesGoalScheduler()];
                sdm.addGoalCompletionListener(new KubernetesJobDeletingGoalCompletionListener_1.KubernetesJobDeletingGoalCompletionListenerFactory(sdm).create());
            }
        } });
}
exports.k8sGoalSchedulingSupport = k8sGoalSchedulingSupport;
/**
 * @deprecated use k8sGoalSchedulingSupport
 */
function goalScheduling() {
    return k8sGoalSchedulingSupport();
}
exports.goalScheduling = goalScheduling;
//# sourceMappingURL=goalScheduling.js.map