"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sdm_1 = require("@atomist/sdm");
const _ = require("lodash");
const array_1 = require("../../util/misc/array");
/**
 * Wrap provided generator to execute additional transformsAndParameters
 */
function universalGenerator(sdm, generator, transforms) {
    return Object.assign(Object.assign({}, generator), { startingPoint: (pi) => __awaiter(this, void 0, void 0, function* () {
            const repoLoader = (p) => sdm_1.projectLoaderRepoLoader(sdm.configuration.sdm.projectLoader || new sdm_1.CachingProjectLoader(), p.target.credentials, true);
            const project = yield sdm_1.computeStartingPoint(pi.parameters, pi.context, repoLoader(pi.parameters), Object.assign(Object.assign({}, generator), { redirecter: () => undefined }), generator.startingPoint, generator, sdm_1.toMachineOptions(sdm));
            const transformsToApply = [];
            for (const transform of transforms) {
                if (!!transform.test) {
                    if (yield transform.test(project)) {
                        transformsToApply.push(transform);
                    }
                }
                else {
                    transformsToApply.push(transform);
                }
            }
            yield enhanceWithSpecificParameters(transformsToApply, pi);
            // Safe the transformsAndParameters to invoke on the invocation to re-use later
            pi.parameters.__transforms = transformsToApply;
            return project;
        }), transform: (p, pi) => __awaiter(this, void 0, void 0, function* () {
            const universalTransforms = pi.parameters.__transforms;
            const computedTransforms = _.flatten(universalTransforms.map(t => array_1.toArray(t.transforms)));
            // tslint:disable-next-line:deprecation
            const trans = sdm_1.chainTransforms(...array_1.toArray(generator.transform || []), ...computedTransforms);
            return trans(p, pi, pi.parameters);
        }) });
}
exports.universalGenerator = universalGenerator;
/**
 * Enrich parameters with the extras if needed
 */
function enhanceWithSpecificParameters(universalTransforms, ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const unsatisfiedParameters = {};
        for (const universalTransform of universalTransforms) {
            _.forEach(universalTransform.parameters, (v, k) => {
                if (ctx.parameters[k] === undefined) {
                    unsatisfiedParameters[k] = v;
                }
            });
        }
        const newParams = yield ctx.promptFor(unsatisfiedParameters);
        for (const name of Object.getOwnPropertyNames(newParams)) {
            ctx.parameters[name] = newParams[name];
        }
    });
}
//# sourceMappingURL=generator.js.map