"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const sdm_1 = require("@atomist/sdm");
const array_1 = require("../../util/misc/array");
const generator_1 = require("./generator");
/**
 * Configure generators in the form GeneratorRegistrations to run additional CodeTransforms
 *
 * The CodeTransforms are getting registered with an optional test which can be used to
 * interrogate the seed project and determine if the CodeTransform should be executed
 * against the seed.
 * CodeTransforms can also register parameter definitions to ask for more parameters via a
 * promptFor question flow.
 */
function universalGeneratorSupport(options) {
    return Object.assign(Object.assign({}, sdm_1.metadata()), { configure: sdm => {
            array_1.toArray(options.generators)
                .map(g => generator_1.universalGenerator(sdm, g, array_1.toArray(options.transformsAndParameters || [])))
                .forEach(g => sdm.addGeneratorCommand(g));
        } });
}
exports.universalGeneratorSupport = universalGeneratorSupport;
//# sourceMappingURL=generatorSupport.js.map