"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const ActionResult_1 = require("@atomist/automation-client/lib/action/ActionResult");
const GitHubRepoRef_1 = require("@atomist/automation-client/lib/operations/common/GitHubRepoRef");
const sdm_1 = require("@atomist/sdm");
const _ = require("lodash");
const ghub_1 = require("./ghub");
/**
 * Persist tags to GitHub topics
 * @param {Tags} tags
 * @param params params to this command
 * @return {Promise<ActionResult<Tags>>}
 * @constructor
 */
exports.GitHubTagRouter = (tags, params) => __awaiter(void 0, void 0, void 0, function* () {
    const grr = GitHubRepoRef_1.isGitHubRepoRef(tags.repoId) ? tags.repoId : new automation_client_1.GitHubRepoRef(tags.repoId.owner, tags.repoId.repo, tags.repoId.sha);
    const url = `${grr.scheme}${grr.apiBase}/repos/${grr.owner}/${grr.repo}/topics`;
    const names = _.uniq(tags.tags);
    const httpClient = automation_client_1.configurationValue("http.client.factory", automation_client_1.defaultHttpClientFactory()).create();
    automation_client_1.logger.debug(`Request to '${url}' to raise tags: [${names.join()}]`);
    try {
        yield httpClient.exchange(url, {
            body: { names },
            headers: Object.assign(Object.assign({}, ghub_1.authHeaders(sdm_1.toToken(params.targets.credentials)).headers), { Accept: "application/vnd.github.mercy-preview+json" }),
            method: automation_client_1.HttpMethod.Put,
        });
    }
    catch (e) {
        e.message = `Failed to create GitHub topics '${names.join()}' on '${url}': ${e.message}`;
        automation_client_1.logger.error(e.message);
        throw e;
    }
    return ActionResult_1.successOn(tags);
});
//# sourceMappingURL=gitHubTagRouter.js.map