"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const stringify = require("json-stringify-safe");
function fetchPushForCommit(context, id, providerId) {
    return __awaiter(this, void 0, void 0, function* () {
        const commitResult = yield context.graphClient.query({
            name: "PushForCommit", variables: {
                owner: id.owner, repo: id.repo, providerId, branch: id.branch, sha: id.sha,
            },
        });
        if (!commitResult || !commitResult.Commit || commitResult.Commit.length === 0) {
            throw new Error("Could not find commit for " + stringify(id));
        }
        const commit = commitResult.Commit[0];
        if (!commit.pushes || commit.pushes.length === 0) {
            throw new Error("Could not find push for " + stringify(id));
        }
        return commit.pushes[0];
    });
}
exports.fetchPushForCommit = fetchPushForCommit;
function fetchBranchTips(ctx, repositoryId) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = yield ctx.graphClient.query({ name: "RepoBranchTips", variables: { name: repositoryId.repo, owner: repositoryId.owner } });
        if (!result || !result.Repo || result.Repo.length === 0) {
            throw new Error(`Repository not found: ${repositoryId.owner}/${repositoryId.repo}`);
        }
        const repo = result.Repo.find(r => r.org.provider.providerId === repositoryId.providerId);
        if (!repo) {
            throw new Error(`Repository not found: ${repositoryId.owner}/${repositoryId.repo} provider ${repositoryId.providerId}`);
        }
        return repo;
    });
}
exports.fetchBranchTips = fetchBranchTips;
function tipOfBranch(repo, branchName) {
    const branchData = repo.branches.find(b => b.name === branchName);
    if (!branchData) {
        throw new Error("Branch not found: " + branchName);
    }
    return branchData.commit.sha;
}
exports.tipOfBranch = tipOfBranch;
//# sourceMappingURL=queryCommits.js.map