"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automationClient_1 = require("@atomist/automation-client/lib/automationClient");
const configuration_1 = require("@atomist/automation-client/lib/configuration");
const sdm_core_1 = require("@atomist/sdm-core");
const _ = require("lodash");
const defaultWorkspaceContextResolver_1 = require("../../common/binding/defaultWorkspaceContextResolver");
const configureLocal_1 = require("../../sdm/configuration/configureLocal");
const fetchMetadataFromAutomationClient_1 = require("../invocation/http/fetchMetadataFromAutomationClient");
/**
 * Default port on which to start an embedded machine.
 * @type {number}
 */
exports.DefaultEmbeddedMachinePort = 2900;
const createMachine = (options) => (config) => {
    const sdm = sdm_core_1.createSoftwareDeliveryMachine({
        name: options.name || "Local bootstrap SDM",
        configuration: config,
    });
    sdm.addExtensionPacks(...(options.extensionPacks || []));
    options.configure(sdm);
    return sdm;
};
function configurationFor(options) {
    const cfg = configuration_1.defaultConfiguration();
    cfg.name = options.name || "bootstrap";
    cfg.workspaceIds = [defaultWorkspaceContextResolver_1.DefaultWorkspaceId];
    cfg.http.port = options.port;
    cfg.logging.level = "info";
    cfg.logging.file.enabled = false;
    cfg.ws.enabled = false;
    cfg.applicationEvents.enabled = false;
    cfg.commands = [];
    cfg.events = [];
    cfg.ingesters = [];
    cfg.listeners = [];
    cfg.apiKey = "not.your.apiKey";
    cfg.local = {
        repositoryOwnerParentDirectory: options.repositoryOwnerParentDirectory,
        mergeAutofixes: true,
        preferLocalSeeds: false,
    };
    cfg.postProcessors = [
        configureLocal_1.configureLocal({ forceLocal: true }),
        sdm_core_1.configureSdm(createMachine(options), {}),
    ];
    return cfg;
}
/**
 * Create an embedded SDM to perform bootstrap commands such as
 * generating a new SDM
 * @return {Promise<AutomationClientConnectionConfig>}
 */
function startEmbeddedMachine(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const optsToUse = Object.assign(Object.assign({ port: exports.DefaultEmbeddedMachinePort }, options), { repositoryOwnerParentDirectory: options.repositoryOwnerParentDirectory });
        process.env.ATOMIST_MODE = "local";
        const config = yield configuration_1.invokePostProcessors(configurationFor(optsToUse));
        _.set(config, "logging.level", "warn");
        const client = automationClient_1.automationClient(config);
        yield client.run();
        const coords = {
            baseEndpoint: `http://${config.local.hostname}:${optsToUse.port}`,
        };
        return fetchMetadataFromAutomationClient_1.fetchMetadataFromAutomationClient(coords);
    });
}
exports.startEmbeddedMachine = startEmbeddedMachine;
//# sourceMappingURL=embeddedMachine.js.map