"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const consoleOutput_1 = require("../../ui/consoleOutput");
const commandInvocationListeners_1 = require("./support/commandInvocationListeners");
const exposeParameters_1 = require("./support/exposeParameters");
const runCommandOnColocatedAutomationClient_1 = require("./support/runCommandOnColocatedAutomationClient");
const yargBuilder_1 = require("./support/yargBuilder");
/**
 * Add commands by name from the given client
 * @param {yargs.Argv} yargs
 * @param {boolean} allowUserInput whether to make all parameters optional, allowing user input to supply them
 */
function addCommandsByName(ai, yargs, workspaceResolver, allowUserInput = true) {
    yargs.command({
        command: "run",
        describe: "Run a command by name",
        builder: args => {
            ai.client.commands.forEach(hi => {
                args.withSubcommand({
                    command: hi.name,
                    describe: hi.description,
                    handler: (argv) => __awaiter(this, void 0, void 0, function* () {
                        return consoleOutput_1.logExceptionsToConsole(() => runByCommandName(ai, hi.name, argv, workspaceResolver.workspaceContext), true);
                    }),
                    parameters: exposeParameters_1.commandLineParametersFromCommandHandlerMetadata(hi, allowUserInput),
                    conflictResolution: yargBuilder_1.promptForAChoiceWhenNecessary(`run ${hi.name} (running command by name)`, ai.client.name),
                });
            });
            return args;
        },
    });
}
exports.addCommandsByName = addCommandsByName;
function runByCommandName(ai, name, command, workspaceContext) {
    return __awaiter(this, void 0, void 0, function* () {
        const hm = ai.client.commands.find(h => h.name === name);
        if (!hm) {
            process.stdout.write(`No command with name [${name}]: Known command names are \n${ai.client.commands
                .map(m => "\t" + m.name).sort().join("\n")}`);
            process.exit(1);
        }
        return runCommandOnColocatedAutomationClient_1.runCommandOnColocatedAutomationClient(ai.location, ai.localConfig, {
            workspaceName: workspaceContext.workspaceName,
            workspaceId: workspaceContext.workspaceId,
        }, hm, command, [commandInvocationListeners_1.ShowDescriptionListener]);
    });
}
//# sourceMappingURL=addCommandsByName.js.map