"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const consoleOutput_1 = require("../../ui/consoleOutput");
const commandInvocationListeners_1 = require("./support/commandInvocationListeners");
const exposeParameters_1 = require("./support/exposeParameters");
const runCommandOnColocatedAutomationClient_1 = require("./support/runCommandOnColocatedAutomationClient");
const yargBuilder_1 = require("./support/yargBuilder");
/**
 * Add commands for all intents
 * @param allowUserInput whether to make all parameters optional, allowing user input to supply them
 */
function addIntentsAsCommands(ai, yargBuilder, workSpaceContextResolver, allowUserInput = true) {
    const handlers = ai.client.commands
        .filter(hm => !!hm.intent && hm.intent.length > 0);
    handlers.forEach(h => h.intent.forEach(intent => yargBuilder.withSubcommand({
        command: intent,
        describe: h.description,
        handler: (argv) => __awaiter(this, void 0, void 0, function* () {
            automation_client_1.logger.debug("Args are %j", argv);
            return consoleOutput_1.logExceptionsToConsole(() => runByIntent(ai, h, argv, workSpaceContextResolver.workspaceContext), true);
        }),
        parameters: exposeParameters_1.commandLineParametersFromCommandHandlerMetadata(h, allowUserInput),
        conflictResolution: yargBuilder_1.promptForAChoiceWhenNecessary(`Intent '${intent}' on command ${h.name}`, ai.client.name),
    })));
}
exports.addIntentsAsCommands = addIntentsAsCommands;
function runByIntent(ai, hm, command, workspaceContext) {
    return __awaiter(this, void 0, void 0, function* () {
        return runCommandOnColocatedAutomationClient_1.runCommandOnColocatedAutomationClient(ai.location, ai.localConfig, {
            workspaceName: workspaceContext.workspaceName,
            workspaceId: workspaceContext.workspaceId,
        }, hm, command, [commandInvocationListeners_1.ShowDescriptionListener, commandInvocationListeners_1.PostToAtomistListenerListener]);
    });
}
//# sourceMappingURL=addIntentsAsCommands.js.map