"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const defaultWorkspaceContextResolver_1 = require("../../../common/binding/defaultWorkspaceContextResolver");
const consoleOutput_1 = require("../../ui/consoleOutput");
const defaultAutomationClientFinder_1 = require("../http/support/defaultAutomationClientFinder");
const addBootstrapCommands_1 = require("./addBootstrapCommands");
const addCloneCommand_1 = require("./addCloneCommand");
const addCommandsByName_1 = require("./addCommandsByName");
const addFeedCommand_1 = require("./addFeedCommand");
const addGitHooksCommands_1 = require("./addGitHooksCommands");
const addGitHubImportCommand_1 = require("./addGitHubImportCommand");
const addIntentsAsCommands_1 = require("./addIntentsAsCommands");
const addReplayCommand_1 = require("./addReplayCommand");
const addShowSdmsCommand_1 = require("./addShowSdmsCommand");
const addStartSdmDeliveryMachine_1 = require("./addStartSdmDeliveryMachine");
const showSkillsCommand_1 = require("./showSkillsCommand");
const yargBuilder_1 = require("./support/yargBuilder");
/**
 * Given a yargs instance, add commands based on local SDMs we can connect to
 * @param yargs instance to customize
 * @param finder strategy for finding running automation client instances
 * @return {yargs.Arguments}
 */
function addLocalSdmCommands(yargs, finder = defaultAutomationClientFinder_1.defaultAutomationClientFinder()) {
    return __awaiter(this, void 0, void 0, function* () {
        const workspaceContextResolver = defaultWorkspaceContextResolver_1.DefaultWorkspaceContextResolver;
        const yargBuilder = yargBuilder_1.isYargBuilder(yargs) ? yargs : yargBuilder_1.freshYargBuilder();
        addBootstrapCommands_1.addBootstrapCommands(yargBuilder);
        addStartSdmDeliveryMachine_1.addStartSdmDeliveryMachine(yargBuilder);
        addFeedCommand_1.addFeedCommand(yargBuilder);
        addGitHooksCommands_1.addAddGitHooksCommand(yargBuilder);
        addGitHooksCommands_1.addRemoveGitHooksCommand(yargBuilder);
        addReplayCommand_1.addReplayCommand(yargBuilder, finder, workspaceContextResolver);
        const clients = (yield finder.findAutomationClients())
            .filter(client => !!client.localConfig);
        addShowSdmsCommand_1.addShowSdmsCommand(clients, yargBuilder);
        showSkillsCommand_1.addShowSkillsCommand(clients, yargBuilder);
        addCloneCommand_1.addCloneCommand(clients, yargBuilder, workspaceContextResolver);
        addGitHubImportCommand_1.addGitHubImportCommand(clients, yargBuilder, workspaceContextResolver);
        // TODO filter on working directories
        for (const client of clients) {
            yield addCommandsToConnectTo(client, yargBuilder, workspaceContextResolver);
        }
        if (!yargBuilder_1.isYargBuilder(yargs)) {
            // we constructed this, so use it
            yargBuilder.build().save(yargs);
        }
    });
}
exports.addLocalSdmCommands = addLocalSdmCommands;
/**
 * Add commands to command to this automation client
 * @return {Promise<void>}
 */
function addCommandsToConnectTo(client, yargBuilder, workspaceContextResolver) {
    return __awaiter(this, void 0, void 0, function* () {
        verifyLocalSdm(client);
        // If we were able to connect to an SDM...
        if (!!client.client) {
            addCommandsByName_1.addCommandsByName(client, yargBuilder, workspaceContextResolver);
            addIntentsAsCommands_1.addIntentsAsCommands(client, yargBuilder, workspaceContextResolver);
        }
    });
}
function verifyLocalSdm(client) {
    if (!!client.location && !client.localConfig) {
        consoleOutput_1.infoMessage(`Detected non-local SDM '${client.client.name}:${client.client.version}'.
To make commands available here please restart this SDM with 'atomist start --local'.

`);
    }
}
//# sourceMappingURL=addLocalSdmCommands.js.map