"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sprintf_js_1 = require("sprintf-js");
const httpMessaging_1 = require("../../../common/ui/httpMessaging");
const consoleOutput_1 = require("../../ui/consoleOutput");
const handleGitHookEvent_1 = require("../git/handleGitHookEvent");
const triggerGitEvents_1 = require("../git/triggerGitEvents");
const suggestStartingAllMessagesListener_1 = require("./support/suggestStartingAllMessagesListener");
const yargBuilder_1 = require("./support/yargBuilder");
/**
 * Add a command to replay Atomist processing of a git event.
 * @param {YargBuilder} yargs
 */
function addReplayCommand(yargs, automationClientFinder, teamContextResolver) {
    yargs.withSubcommand(yargBuilder_1.yargCommandWithPositionalArguments({
        command: "replay <event> [depth]",
        describe: "Replay commit action on the current repository",
        positional: [{
                key: "event", opts: {
                    choices: Object.values(handleGitHookEvent_1.HookEvent),
                },
            }, {
                key: "depth", opts: {
                    type: "number",
                    default: 1,
                },
            }],
        handler: ya => {
            return consoleOutput_1.logExceptionsToConsole(() => __awaiter(this, void 0, void 0, function* () {
                const clients = yield automationClientFinder.findAutomationClients();
                const msg = sprintf_js_1.sprintf("Dispatching git event '%s' to %d clients...\n", ya.event, clients.length);
                consoleOutput_1.infoMessage(msg);
                yield httpMessaging_1.sendDiagnosticMessageToAllMessagesListener(msg);
                yield triggerGitEvents_1.triggerGitEvents(clients, ya.event, ya.depth, teamContextResolver);
                if (clients.length > 0) {
                    return suggestStartingAllMessagesListener_1.suggestStartingAllMessagesListener();
                }
            }), true);
        },
    }));
}
exports.addReplayCommand = addReplayCommand;
//# sourceMappingURL=addReplayCommand.js.map