"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const SdmCd_1 = require("../../../pack/sdm-cd/support/SdmCd");
const defaultLocalSoftwareDeliveryMachineConfiguration_1 = require("../../../sdm/configuration/defaultLocalSoftwareDeliveryMachineConfiguration");
const embeddedMachine_1 = require("../../embedded/embeddedMachine");
const consoleOutput_1 = require("../../ui/consoleOutput");
const renderClientInfo_1 = require("../../ui/renderClientInfo");
exports.DefaultSdmCdPort = 2901;
/**
 * Start an SDM dedicated to SDM CD
 * @param {YargBuilder} yargs
 */
function addStartSdmDeliveryMachine(yargs) {
    yargs.command({
        command: "deliver",
        describe: "DEPRECATED Start SDM delivery machine",
        builder: args => {
            return args.option("port", {
                required: false,
                default: exports.DefaultSdmCdPort,
                type: "number",
                description: `Port for the delivery SDM. Defaults to ${exports.DefaultSdmCdPort}`,
            }).option("base", {
                required: false,
                default: undefined,
                alias: "repositoryOwnerParentDirectory",
                description: `Base directory for the delivery SDM: Defaults to ${defaultLocalSoftwareDeliveryMachineConfiguration_1.determineDefaultRepositoryOwnerParentDirectory()}`,
            });
        },
        handler: argv => {
            consoleOutput_1.warningMessage(`The 'deliver' command is DEPRECATED and will be removed in a future release.\n`);
            return consoleOutput_1.logExceptionsToConsole(() => __awaiter(this, void 0, void 0, function* () {
                const client = yield startSdmMachine(argv.port, argv.base);
                consoleOutput_1.infoMessage("Started local SDM delivery machine \n\t%s\n", renderClientInfo_1.renderClientInfo(client));
                consoleOutput_1.infoMessage("The next push to any SDM under %s will trigger deployment\n", chalk_1.default.yellow(client.localConfig.repositoryOwnerParentDirectory));
                consoleOutput_1.infoMessage("Alternatively, go to the relevant directory and type %s to trigger deployment\n", chalk_1.default.yellow("atomist trigger post-commit"));
            }), true);
        },
    });
}
exports.addStartSdmDeliveryMachine = addStartSdmDeliveryMachine;
function startSdmMachine(host, port, repositoryOwnerParentDirectory) {
    return __awaiter(this, void 0, void 0, function* () {
        return embeddedMachine_1.startEmbeddedMachine({
            repositoryOwnerParentDirectory,
            port,
            configure: sdm => {
                sdm.addExtensionPacks(SdmCd_1.sdmCd({ host: sdm.configuration.local.hostname, port }));
            },
        });
    });
}
//# sourceMappingURL=addStartSdmDeliveryMachine.js.map