"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const NodeProjectCreationParameters_1 = require("./NodeProjectCreationParameters");
const updatePackageJsonIdentification_1 = require("./updatePackageJsonIdentification");
/**
 * Generator that can create a new Node project. Parameterized
 * by name, starting point and tags.
 */
function nodeGenerator(name, startingPoint, transforms = [updatePackageJsonIdentification_1.UpdatePackageJsonIdentification], ...tags) {
    return {
        name,
        startingPoint,
        parameters: NodeProjectCreationParameters_1.NodeProjectCreationParametersDefinition,
        transform: [
            ...transforms,
        ],
        tags,
    };
}
exports.nodeGenerator = nodeGenerator;
/**
 * Creates a new repo based on the content of an existing repo
 * without making any changes
 */
exports.superforkGenerator = {
    name: "superfork",
    startingPoint: params => new automation_client_1.GitHubRepoRef(params.owner, params.repo),
    parameters: {
        owner: Object.assign(Object.assign({}, automation_client_1.validationPatterns.GitHubNameRegExp), { description: "GitHub owner" }),
        repo: Object.assign(Object.assign({}, automation_client_1.validationPatterns.GitHubNameRegExp), { description: "GitHub repo" }),
    },
    transform: (p) => __awaiter(void 0, void 0, void 0, function* () { return p; }),
};
//# sourceMappingURL=bootstrapGenerators.js.map