"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const child_process_1 = require("@atomist/automation-client/lib/util/child_process");
/**
 * Code transform to update identification fields of package.json
 * @param project
 * @param context
 * @param params
 */
exports.UpdatePackageJsonIdentification = (project, context, params) => __awaiter(void 0, void 0, void 0, function* () {
    automation_client_1.logger.info("Updating JSON: params=%j", params);
    const author = {
        name: yield fetchGitConfig("user.name", params.screenName),
        email: yield fetchGitConfig("user.email", undefined),
    };
    return automation_client_1.doWithJson(project, "package.json", pkg => {
        const repoUrl = params.target.repoRef.url;
        pkg.name = `@${params.target.repoRef.owner}/${params.target.repoRef.repo}`;
        pkg.description = params.target.description;
        pkg.version = params.version;
        pkg.author = author;
        pkg.repository = {
            type: "git",
            url: `${repoUrl}.git`,
        };
        pkg.homepage = `${repoUrl}#readme`;
        pkg.bugs = {
            url: `${repoUrl}/issues`,
        };
        pkg.keywords = pkg.keywords.filter(keyword => keyword !== "seed");
        automation_client_1.logger.info("Updated JSON. Result is %j", pkg);
    });
});
function fetchGitConfig(property, defaultValue) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const result = yield child_process_1.execPromise("git", ["config", property]);
            return result.stdout.trim();
        }
        catch (_a) {
            automation_client_1.logger.info("Unable to retrieve git config " + property);
            return defaultValue;
        }
    });
}
//# sourceMappingURL=updatePackageJsonIdentification.js.map