"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const chalk_1 = require("chalk");
const _ = require("lodash");
const sprintf_js_1 = require("sprintf-js");
const consoleOutput_1 = require("../../ui/consoleOutput");
const MaxColumnWidth = 30;
/**
 * Display the show skills command, backed by the given skills
 * gathered from all connected clients
 * @param clientFinder help find connected clients
 * @param {YargBuilder} yargs
 */
function addShowSkillsCommand(clients, yargs) {
    yargs.command({
        command: "show skills",
        aliases: "s",
        describe: "Show skills",
        handler: () => __awaiter(this, void 0, void 0, function* () {
            return consoleOutput_1.logExceptionsToConsole(() => __awaiter(this, void 0, void 0, function* () { return printSkillsToConsole(clients); }), true);
        }),
    });
}
exports.addShowSkillsCommand = addShowSkillsCommand;
function printSkillsToConsole(clients) {
    const commands = _.flatten(clients.map(client => client.client.commands));
    consoleOutput_1.infoMessage("%s commands are available from %s connected SDM%s\n\n", commands.length, clients.length, clients.length === 1 ? "" : "s");
    const padLength = Math.max(longestSingleIntentString(commands), MaxColumnWidth);
    const longestCommandNameLength = Math.max(commands.length === 0 ? 0 : commands.map(c => c.name).reduce((a, b) => a.length > b.length ? a : b).length + 5, MaxColumnWidth);
    const separatorLength = 2 * padLength + longestCommandNameLength;
    consoleOutput_1.infoMessage("%s%s%s\n", chalk_1.default.italic("Intent".padEnd(padLength)), chalk_1.default.italic("Command name".padEnd(longestCommandNameLength)), chalk_1.default.italic("Description".padEnd(padLength)));
    consoleOutput_1.infoMessage("%s\n", "=".repeat(separatorLength));
    const commandChunks = commands.map(md => sprintf_js_1.sprintf("%s%s%s\n", chalk_1.default.bold(paddedIntentString(md, padLength)), chalk_1.default.gray(md.name.padEnd(longestCommandNameLength)), chalk_1.default.italic(md.description)));
    const separator = sprintf_js_1.sprintf("%s\n", chalk_1.default.gray("-".repeat(separatorLength)));
    consoleOutput_1.infoMessage(commandChunks.join(separator));
}
function paddedIntentString(md, n) {
    const arrays = !!md.intent && md.intent.length > 0 ?
        md.intent :
        [""];
    // Pad last element
    arrays[arrays.length - 1] = arrays[arrays.length - 1].padEnd(n);
    return arrays.join("\n");
}
function longestSingleIntentString(commands) {
    if (commands.length === 0) {
        return 0;
    }
    const winner = commands.reduce((a, b) => {
        const alen = longestIntentStringLength(a);
        const blen = longestIntentStringLength(b);
        return alen > blen ? a : b;
    });
    return longestIntentStringLength(winner);
}
function longestIntentStringLength(command) {
    const intents = automation_client_1.toStringArray(command.intent || []);
    return intents.length > 0 ?
        intents.reduce((a, b) => a.length > b.length ? a : b).length :
        0;
}
//# sourceMappingURL=showSkillsCommand.js.map