"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const fs = require("fs");
const util_1 = require("util");
const repoOnboardingEvents_1 = require("../../../../sdm/binding/event/repoOnboardingEvents");
const addGitHooks_1 = require("../../../setup/addGitHooks");
const consoleOutput_1 = require("../../../ui/consoleOutput");
const invokeEventHandlerUsingHttp_1 = require("../../http/invokeEventHandlerUsingHttp");
/**
 * Clone the repo, install Atomist git hooks and send onboarding messages
 */
function cloneAndAtomize(args) {
    return __awaiter(this, void 0, void 0, function* () {
        const { owner, repo } = args;
        const orgDir = args.repositoryOwnerDirectory + "/" + args.owner;
        if (!fs.existsSync(orgDir)) {
            fs.mkdirSync(orgDir);
        }
        consoleOutput_1.infoMessage(`Cloning git remote project: ${args.cloneCommand}\n`);
        yield util_1.promisify(child_process_1.exec)(args.cloneCommand, { cwd: orgDir });
        yield addGitHooks_1.addGitHooks(`${args.repositoryOwnerDirectory}/${owner}/${repo}`);
        for (const client of args.clients) {
            const eventSender = invokeEventHandlerUsingHttp_1.invokeEventHandlerUsingHttp(client.location, args.workspaceContext);
            yield repoOnboardingEvents_1.sendRepoOnboardingEvent(args.workspaceContext, { owner, repo, url: undefined }, eventSender);
            yield repoOnboardingEvents_1.sendChannelLinkEvent(args.workspaceContext, { owner, repo, url: undefined }, eventSender);
        }
    });
}
exports.cloneAndAtomize = cloneAndAtomize;
//# sourceMappingURL=cloneAndAtomize.js.map