"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const runCommandOnColocatedAutomationClient_1 = require("./runCommandOnColocatedAutomationClient");
/**
 * Expose the parameters for this command
 * @param {CommandHandlerMetadata} hi
 * @param {yargs.Argv} args
 * @param allowUserInput whether to make all parameters optional, allowing user input to supply them
 */
function exposeParameters(hi, args, allowUserInput) {
    commandLineParametersFromCommandHandlerMetadata(hi, allowUserInput)
        .forEach(p => {
        args.withParameter(p);
    });
    return args;
}
exports.exposeParameters = exposeParameters;
function commandLineParametersFromCommandHandlerMetadata(hi, allowUserInput) {
    const p1 = hi.parameters
        .map((p) => {
        const nameToUse = runCommandOnColocatedAutomationClient_1.convertToDisplayable(p.name);
        return {
            parameterName: nameToUse,
            required: !allowUserInput && p.required && !p.default_value,
        };
    });
    const p2 = (hi.mapped_parameters || []).map((mp) => {
        const nameToUse = runCommandOnColocatedAutomationClient_1.convertToDisplayable(mp.name);
        return {
            parameterName: nameToUse,
            required: false,
        };
    });
    return [...p1, ...p2];
}
exports.commandLineParametersFromCommandHandlerMetadata = commandLineParametersFromCommandHandlerMetadata;
//# sourceMappingURL=exposeParameters.js.map