"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const util_1 = require("util");
/**
 * Verify the result of a command. Useful to test
 * whether a dependency of an SDM is installed
 * @param {CommandVerificationRequest} opts
 * @return {Promise<void>}
 */
function verifyCommandResult(opts) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const r = yield util_1.promisify(child_process_1.exec)(opts.command, opts.execOptions);
            if (!opts.outputTest || opts.outputTest(r)) {
                if (!!opts.onVerified) {
                    opts.onVerified(r.stdout);
                }
            }
            else {
                opts.onWrongVersion(r);
            }
        }
        catch (e) {
            opts.onFailure(e);
        }
    });
}
exports.verifyCommandResult = verifyCommandResult;
//# sourceMappingURL=verifyCommandResult.js.map