"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
function parseCommandLine(s) {
    const allWords = s.split(" ");
    const firstPositionalArgument = allWords.findIndex(isPositionalArgument);
    if (firstPositionalArgument < 0) {
        return new CommandLineImpl(allWords, []);
    }
    const result = new CommandLineImpl(allWords.slice(0, firstPositionalArgument), allWords.slice(firstPositionalArgument));
    // could verify that all positionalArguments are positional arguments
    assert(s === result.toString(), `Command line parsed incorrectly: ${s} parsed to ${result}`);
    return result;
}
exports.parseCommandLine = parseCommandLine;
function dropFirstWord(commandLine) {
    return new CommandLineImpl(commandLine.words.splice(1), commandLine.positionalArguments);
}
exports.dropFirstWord = dropFirstWord;
function commandLineAlias(commandLine, alias) {
    assert(!alias.includes(" "), "multiword aliases not supported: " + alias);
    return new CommandLineImpl([alias], commandLine.positionalArguments);
}
exports.commandLineAlias = commandLineAlias;
function isPositionalArgument(w) {
    return /^<.*>$/.test(w) || /^\[.*\]/.test(w);
}
function verifyOneWord(commandLine) {
    if (commandLine.words.length > 1) {
        throw new Error("You can only have one word if there are positional arguments");
    }
}
exports.verifyOneWord = verifyOneWord;
class CommandLineImpl {
    constructor(words, positionalArguments) {
        this.words = words;
        this.positionalArguments = positionalArguments;
    }
    get firstWord() {
        return this.words[0];
    }
    toString() {
        return [...this.words, ...this.positionalArguments].join(" ");
    }
}
//# sourceMappingURL=commandLine.js.map