"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * If more than one of this command appear, then
 * - if all the others resolve with "drop with warnings," fine, this one wins
 * - otherwise, fail the whole program with an error, on startup, even if they didn't want to use this command.
 * This is the default because the others are opt-in, but you probably don't want to use it on anything dynamically added.
 * @param commandDescription
 */
function failEverythingInCaseOfConflict(commandDescription) {
    return { kind: "expected to be unique", failEverything: true, commandDescription };
}
exports.failEverythingInCaseOfConflict = failEverythingInCaseOfConflict;
/**
 * If more than one of this command appear, drop this one. Put a warning in the help message that this happened.
 * If all of this command resolve with "drop with warnings", none of them will be available.
 * @param commandDescription
 */
function dropWithWarningsInHelp(commandDescription) {
    return { kind: "drop with warnings", failEverything: false, commandDescription };
}
exports.dropWithWarningsInHelp = dropWithWarningsInHelp;
/**
 * If more than one of this command appear, ask the user to choose between them.
 * The uniqueChoice will distinguish them.
 * If two of the same command with the same uniqueChoice conflict, or if one of the commands insists on failing everything
 * in case of conflict, fallback to drop with warning.
 * @param commandDescription
 * @param choice
 */
function promptForAChoiceWhenNecessary(commandDescription, uniqueChoice) {
    return { kind: "prompt for a choice", commandDescription, uniqueChoice, failEverything: false };
}
exports.promptForAChoiceWhenNecessary = promptForAChoiceWhenNecessary;
function isPromptForChoice(cr) {
    return cr.kind === "prompt for a choice";
}
exports.isPromptForChoice = isPromptForChoice;
function isYargCommand(yc) {
    return yc.commandName !== undefined;
}
exports.isYargCommand = isYargCommand;
//# sourceMappingURL=interfaces.js.map