/// <reference types="yargs" />
import { HandleInstructions } from "./handleInstruction";
import { Built, CommandLineParameter, ConflictResolution, ParameterOptions, SupportedSubsetOfYargsCommandMethod, YargBuilder, YargCommand, YargCommandWordSpec, YargRunnableCommandSpec } from "./interfaces";
/**
 * This command might be runnable by itself, and might also have subcommands.
 *
 * It can be constructed whole, or built up yargs-style.
 */
export declare class YargCommandWord implements YargCommand {
    runnableCommand?: YargRunnableCommandSpec;
    readonly nestedCommands: YargCommand[];
    readonly conflictResolution: ConflictResolution;
    readonly description: string;
    readonly commandName: string;
    readonly warnings: string[];
    constructor(spec: YargCommandWordSpec);
    withSubcommand(c: YargCommand | SupportedSubsetOfYargsCommandMethod): this;
    addHelpMessages(strs: string[]): void;
    option(parameterName: string, opts: ParameterOptions): YargBuilder;
    demandCommand(): this;
    command(params: SupportedSubsetOfYargsCommandMethod): this;
    withParameter(p: CommandLineParameter): this;
    beRunnable(): void;
    /**
     * if true, this command has a handler function
     * if false, there are only subcommands
     */
    readonly isRunnable: boolean;
    build(): Built;
    readonly handleInstructions: HandleInstructions;
}
export declare function condenseDescriptions(allChildDescriptions: string[], myDescription?: string): string;
export declare function imitateYargsCommandMethod(params: SupportedSubsetOfYargsCommandMethod): YargCommand[];
