"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sdm_1 = require("@atomist/sdm");
/**
 * Transform to add a module to a project. Uses npm install.
 * @param opts dependency info. If not supplied, parameters will be used
 */
function addDependencyTransform(opts) {
    return {
        parameters: {
            name: {
                required: !opts.name,
                description: "module name",
            },
            version: {
                required: false,
                description: "npm version qualification. Goes after module name, e.g. @branch-master",
                defaultValue: opts.version || "",
            },
        },
        transform: (p, cli) => __awaiter(this, void 0, void 0, function* () {
            return sdm_1.spawnCodeTransform([{
                    command: "npm",
                    args: ["install",
                        `${cli.parameters.name || opts.name}${cli.parameters.version}`],
                }], opts.progressLog)(p, cli);
        }),
    };
}
exports.addDependencyTransform = addDependencyTransform;
//# sourceMappingURL=addDependencyTransform.js.map