"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const handlePushBasedEventOnRepo_1 = require("../../../common/git/handlePushBasedEventOnRepo");
const consoleOutput_1 = require("../../ui/consoleOutput");
const invokeEventHandlerUsingHttp_1 = require("../http/invokeEventHandlerUsingHttp");
/**
 * git hooks we support. All hooks are "post" hooks, meaning that
 * Atomist will never prevent completion of a git operation, but
 * will be invoked after it is complete.
 * @type {string[]}
 */
var HookEvent;
(function (HookEvent) {
    /**
     * Used to respond to commits. Main Atomist entry point.
     */
    HookEvent["PostCommit"] = "post-commit";
    /**
     * Used when a branch has been merged
     */
    HookEvent["PostMerge"] = "post-merge";
    /**
     * Server side hook. Atomist will clone local projects, using the
     * file location as a remote. If it makes a transform or autofix,
     * the push from the cloned repo will result in this server side hook
     * firing on the original local repo.
     */
    HookEvent["PostReceive"] = "post-receive";
})(HookEvent = exports.HookEvent || (exports.HookEvent = {}));
/**
 * Invoke an SDM to handle this event.
 * @param payload event data
 * @return {Promise<any>}
 */
function handleGitHookEvent(cc, lc, payload) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!payload) {
            return consoleOutput_1.errorMessage("Payload must be supplied");
        }
        if (!payload.event) {
            return consoleOutput_1.errorMessage("Invalid git hook invocation payload. Event is required: %j", payload);
        }
        if (!Object.values(HookEvent).includes(payload.event)) {
            return consoleOutput_1.errorMessage("Unknown git hook event '%s'", payload.event);
        }
        if (!lc) {
            return consoleOutput_1.errorMessage("LocalModeConfiguration must be supplied");
        }
        const target = {
            workspaceId: payload.workspaceId,
            workspaceName: payload.workspaceId,
        };
        return handlePushBasedEventOnRepo_1.handlePushBasedEventOnRepo(payload.workspaceId, invokeEventHandlerUsingHttp_1.invokeEventHandlerUsingHttp(cc, target), lc, payload, "SetGoalsOnPush");
    });
}
exports.handleGitHookEvent = handleGitHookEvent;
//# sourceMappingURL=handleGitHookEvent.js.map