"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable-next-line:import-blacklist
const axios_1 = require("axios");
/**
 * Call into an automation client at the given location and retrieve metadata
 * @return {Promise<AutomationClientInfo>}
 */
function fetchMetadataFromAutomationClient(location) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const resp = yield axios_1.default.get(location.baseEndpoint + "/registration", {
                timeout: 5 * 1000,
            });
            let localConfig;
            try {
                localConfig = (yield axios_1.default.get(location.baseEndpoint + "/local/configuration")).data;
            }
            catch (_a) {
                // Do nothing. The automation client we're talking to is not in local mode
            }
            const client = resp.data;
            return {
                client,
                localConfig,
                location,
            };
        }
        catch (e) {
            // errorMessage("Unable to connect to '%s': Is a Software Delivery Machine running?\n\t(%s)\n",
            //     location.baseEndpoint, e);
            return {
                client: undefined,
                localConfig: undefined,
                location,
            };
        }
    });
}
exports.fetchMetadataFromAutomationClient = fetchMetadataFromAutomationClient;
//# sourceMappingURL=fetchMetadataFromAutomationClient.js.map