"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const assert = require("power-assert");
const propertiesToArgs_1 = require("../../../common/util/propertiesToArgs");
const EnvironmentTokenCredentialsResolver_1 = require("../../../sdm/binding/EnvironmentTokenCredentialsResolver");
const httpInvoker_1 = require("./support/httpInvoker");
const newCorrelationId_1 = require("./support/newCorrelationId");
/**
 * Return a command invoker using HTTP to the given address
 * @param {AutomationClientConnectionRequest} location
 * @return {CommandHandlerInvoker}
 */
function invokeCommandHandlerUsingHttp(location) {
    return (invocation) => __awaiter(this, void 0, void 0, function* () {
        const parameters = propertiesToArgs_1.propertiesToArgs(invocation.parameters);
        const data = {
            command: invocation.name,
            parameters,
            mapped_parameters: propertiesToArgs_1.propertiesToArgs(invocation.mappedParameters || {}).concat([
                { name: "slackTeam", value: invocation.workspaceId },
            ]).concat(parameters),
            secrets: (invocation.secrets || []).concat([
                { uri: "github://user_token?scopes=repo,user:email,read:user", value: EnvironmentTokenCredentialsResolver_1.credentialsFromEnvironment().token },
            ]),
            correlation_id: invocation.correlationId || (yield newCorrelationId_1.newCliCorrelationId()),
            api_version: "1",
            team: {
                id: invocation.workspaceId,
                name: invocation.workspaceName,
            },
        };
        assert(!!location, "Config must be provided");
        assert(!!location.baseEndpoint, "Base endpoint must be provided: saw " + JSON.stringify(location));
        const url = `/command`;
        automation_client_1.logger.debug("Hitting %s to invoke command %s using %j", url, invocation.name, data);
        const resp = yield httpInvoker_1.postToSdm(location, url, data);
        if (resp.code !== 0) {
            automation_client_1.logger.error("Command handler did not succeed. Returned: " + JSON.stringify(resp, undefined, 2));
        }
        return resp;
    });
}
exports.invokeCommandHandlerUsingHttp = invokeCommandHandlerUsingHttp;
//# sourceMappingURL=invokeCommandHandlerUsingHttp.js.map