"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const string_1 = require("@atomist/automation-client/lib/internal/util/string");
const stringify = require("json-stringify-safe");
const EnvironmentTokenCredentialsResolver_1 = require("../../../sdm/binding/EnvironmentTokenCredentialsResolver");
const httpInvoker_1 = require("./support/httpInvoker");
const newCorrelationId_1 = require("./support/newCorrelationId");
const assert = require("assert");
/**
 * Invoke the event handler on all these clients
 * @param {AutomationClientFinder} clientFinder
 * @param {InvocationTarget} target
 * @return {EventSender}
 */
function invokeEventHandlerUsingHttpOnAll(clientFinder, target) {
    return __awaiter(this, void 0, void 0, function* () {
        const clients = yield clientFinder.findAutomationClients();
        return (invocation) => __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(clients.map(client => invokeEventHandlerUsingHttp(client.location, target)(invocation)));
            return automation_client_1.Success;
        });
    });
}
exports.invokeEventHandlerUsingHttpOnAll = invokeEventHandlerUsingHttpOnAll;
/**
 * Return an event invoker using HTTP to the given address
 * @return {Promise<HandlerResult>}
 */
function invokeEventHandlerUsingHttp(location, target) {
    assert(!!target);
    return (invocation) => __awaiter(this, void 0, void 0, function* () {
        const token = EnvironmentTokenCredentialsResolver_1.credentialsFromEnvironment().token;
        const data = {
            extensions: {
                operationName: invocation.name,
                query_id: "q-" + Date.now(),
                team_id: target.workspaceId,
                team_name: target.workspaceName,
                correlation_id: target.correlationId || (yield newCorrelationId_1.newCliCorrelationId()),
            },
            secrets: (invocation.secrets || []).concat([
                { uri: "github://user_token?scopes=repo,user:email,read:user", value: token },
                { uri: "github://org_token", value: token },
                { uri: automation_client_1.Secrets.OrgToken, value: token },
                { uri: automation_client_1.Secrets.UserToken, value: token },
            ]),
            api_version: "1",
            data: invocation.payload,
        };
        const url = `/event`;
        automation_client_1.logger.info("Calling url %s to handler %s using %s", url, invocation.name, stringify(data, string_1.replacer));
        assert(!!location, "HTTP location must be provided");
        assert(!!location.baseEndpoint, "Base endpoint must be provided: saw " + JSON.stringify(location));
        const resp = yield httpInvoker_1.postToSdm(location, url, data);
        assert(resp.code !== 0, "Event handler did not succeed. Returned: " + JSON.stringify(resp, undefined, 2));
        return resp;
    });
}
exports.invokeEventHandlerUsingHttp = invokeEventHandlerUsingHttp;
//# sourceMappingURL=invokeEventHandlerUsingHttp.js.map