"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const parseCorrelationId_1 = require("../../../../common/invocation/parseCorrelationId");
const portAllocation_1 = require("./portAllocation");
/**
 * Create a correctly formatted correlation ID. We encode the port that
 * the automation client's message client can communicate back to us on over HTTP,
 * and the channel to use to display messages.
 * @return {string}
 */
function newCliCorrelationId(opts = { channel: "general", encodeListenerPort: false }) {
    return __awaiter(this, void 0, void 0, function* () {
        const encodedPort = opts.encodeListenerPort ? `${yield portAllocation_1.portToListenOnFor(process.pid)}-` : "";
        const corrId = `${parseCorrelationId_1.ClientType}-${encodedPort}${opts.channel || "general"}-${new Date().getTime()}`;
        return corrId;
    });
}
exports.newCliCorrelationId = newCliCorrelationId;
//# sourceMappingURL=newCorrelationId.js.map