/**
 * Perform the given action, logging exceptions to the console
 * @param {() => Promise<any>} what
 * @param showStack whether or not to show the stack
 * @return {Promise<void>}
 */
export declare function logExceptionsToConsole(what: () => Promise<any>, showStack: boolean): Promise<void>;
/**
 * Display an error message to the console, supporting sprintf style
 * @param msg message
 * @param args arguments
 */
export declare function errorMessage(msg: string, ...args: any[]): void;
/**
 * Display a warning message to the console, supporting sprintf style
 * @param msg message
 * @param args arguments
 */
export declare function warningMessage(msg: string, ...args: any[]): void;
/**
 * Get the user's attention without implying there's a problem,
 * supporting sprintf style
 * @param {string} msg
 * @param args
 */
export declare function adviceMessage(msg: string, ...args: any[]): void;
/**
 * Display an info message to the console, supporting sprintf style
 * @param msg message
 * @param args arguments
 */
export declare function infoMessage(msg: string, ...args: any[]): void;
/**
 * Dynamically build an advice block from one or more documentation chunks
 * @param {string} relativePaths paths relative to the base of the current project
 */
export declare function adviceDoc(...relativePaths: string[]): void;
