"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const sprintf_js_1 = require("sprintf-js");
/**
 * Format information about this automation for the console
 * @param {AutomationClientInfo} aci
 * @return {string}
 */
function renderClientInfo(aci) {
    const local = aci.localConfig ? aci.localConfig.repositoryOwnerParentDirectory : "(remote)";
    const reg = aci.location.baseEndpoint + "/registration";
    return `${chalk_1.default.bold(aci.client.name)} @ ${chalk_1.default.underline(aci.location.baseEndpoint)} - ${local} - ${reg}`;
}
exports.renderClientInfo = renderClientInfo;
function renderEventDispatch(aci, what) {
    return sprintf_js_1.sprintf("Sending event %s to machine %s\n", chalk_1.default.yellow(JSON.stringify(what)), chalk_1.default.underline(aci.client.name));
}
exports.renderEventDispatch = renderEventDispatch;
function renderClientsInfo(clients) {
    let s = "";
    switch (clients.length) {
        case 0:
            s += sprintf_js_1.sprintf("There are no connected SDMs\n", clients.length);
            break;
        case 1:
            s += sprintf_js_1.sprintf("There is one connected SDM\n", clients.length);
            break;
        default:
            s += sprintf_js_1.sprintf("There are %d connected SDMs\n", clients.length);
            break;
    }
    s += sprintf_js_1.sprintf(`\t%s\n\n`, clients.map(renderClientInfo).join("\n\t"));
    return s;
}
exports.renderClientsInfo = renderClientsInfo;
//# sourceMappingURL=renderClientInfo.js.map