"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_1 = require("@atomist/automation-client/lib/configuration");
const _ = require("lodash");
const os = require("os");
const consoleOutput_1 = require("../../cli/ui/consoleOutput");
exports.DefaultLocalWorkspaceId = "local";
/**
 * Resolve team from the environment, within CLI
 */
class EnvConfigWorkspaceContextResolver {
    get workspaceContext() {
        let userConfig;
        try {
            userConfig = configuration_1.getUserConfig() || {};
        }
        catch (e) {
            consoleOutput_1.warningMessage(`Failed to load user configuration, ignoring: ${e.message}`);
            userConfig = {};
        }
        // This resolves workspaceIds from env variables or user configuration.
        configuration_1.resolveWorkspaceIds(userConfig);
        const workspaceId = (!_.isEmpty(userConfig.workspaceIds)) ?
            userConfig.workspaceIds[0] : exports.DefaultLocalWorkspaceId;
        const workspaceName = os.hostname();
        return {
            workspaceId,
            workspaceName,
        };
    }
}
exports.EnvConfigWorkspaceContextResolver = EnvConfigWorkspaceContextResolver;
//# sourceMappingURL=EnvConfigWorkspaceContextResolver.js.map