"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
// only local connections
// tslint:disable-next-line:import-blacklist
const axios_1 = require("axios");
const boxen = require("boxen");
const sprintf_js_1 = require("sprintf-js");
const defaultLocalSoftwareDeliveryMachineConfiguration_1 = require("../../sdm/configuration/defaultLocalSoftwareDeliveryMachineConfiguration");
exports.MessageRoute = "/message";
exports.GoalRoute = "/goal";
exports.AllMessagesPort = 6660;
/**
 * Write a raw message to the listener
 * @param {string} message
 * @return {Promise<void>}
 */
function sendDiagnosticMessageToAllMessagesListener(message, ...args) {
    return __awaiter(this, void 0, void 0, function* () {
        const url = `http://${defaultLocalSoftwareDeliveryMachineConfiguration_1.determineDefaultHostUrl()}:${exports.AllMessagesPort}/write`;
        const boxed = boxen(sprintf_js_1.sprintf(message, args), { padding: 1 }) + "\n";
        try {
            yield axios_1.default.post(url, { message: boxed });
        }
        catch (err) {
            // Ignore any error
        }
    });
}
exports.sendDiagnosticMessageToAllMessagesListener = sendDiagnosticMessageToAllMessagesListener;
/**
 * Convenient function to log in the present process and send a message
 * to the all messages listener, which will be displayed if it's in verbose mode.
 * Typically used when we want to send information about events within the
 * SDM process to a user debugging sdm-local.
 */
function logAndSend(msg, ...args) {
    return __awaiter(this, void 0, void 0, function* () {
        automation_client_1.logger.info(msg, args);
        return sendDiagnosticMessageToAllMessagesListener(msg, args);
    });
}
exports.logAndSend = logAndSend;
//# sourceMappingURL=httpMessaging.js.map