"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const globals_1 = require("@atomist/automation-client/lib/globals");
/**
 * Local graph client. Returns empty result set or throws an
 * exception on all calls. GraphQL is not presently supported in Local SDM.
 */
class LocalGraphClient {
    constructor(showErrorStacks) {
        this.showErrorStacks = showErrorStacks;
    }
    executeMutation(mutation, variables, options) {
        throw new Error();
    }
    executeMutationFromFile(path, variables, options, current) {
        throw new Error();
    }
    executeQuery(query, variables, options) {
        return __awaiter(this, void 0, void 0, function* () {
            automation_client_1.logger.warn("Returning empty object for query " + query);
            // tslint:disable-next-line:no-object-literal-type-assertion
            return {};
        });
    }
    executeQueryFromFile(path, variables, options, current) {
        throw new Error();
    }
    mutate(optionsOrName) {
        throw new Error();
    }
    query(optionsOrName) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO we are hard coding this to ensure that a particular query coming from automation-client is satisfied.
            // How do we do this in a better, more extensible way?
            const qo = optionsOrName;
            if (qo.name === "SdmGoalsForCommit" && qo.variables && qo.variables.offset === 0) {
                const v = qo.variables;
                const sha = v.sha;
                const goalSetId = v.goalSetId;
                const goals = globals_1.eventStore().messages()
                    .filter(m => m.value.sha === sha
                    && m.value.goalSet
                    && m.value.goalSetId
                    && (!goalSetId || m.value.goalSetId === goalSetId))
                    .map(m => m.value);
                return {
                    SdmGoal: goals,
                };
            }
            else if (qo.name === "PushForSdmGoal" && qo.variables) {
                const v = qo.variables;
                const sha = v.sha;
                const goal = globals_1.eventStore().messages()
                    .find(m => m.value.sha === sha && m.value.goalSet && m.value.goalSetId).value;
                return {
                    Commit: [{
                            pushes: [goal.push],
                        }],
                };
            }
            else if (qo.name === "SdmVersionForCommit" && qo.variables && qo.variables.sha) {
                const v = qo.variables;
                const sha = v.sha[0];
                const branch = v.branch[0];
                const version = globals_1.eventStore().messages()
                    .find(m => !m.value.goalSetId && m.value.sha === sha && m.value.version && m.value.branch === branch).value;
                return {
                    SdmVersion: [{
                            version: version.version,
                        }],
                };
            }
            else if (!!qo.query && qo.query.trim().startsWith("query ChatTeam")) {
                const ctr = {
                    ChatTeam: [{
                            id: "any",
                        }],
                };
                automation_client_1.logger.info("Returning hardcoded ChatTeam %j", ctr);
                return ctr;
            }
            const err = new Error("Warning: GraphClient not supported locally");
            if (this.showErrorStacks) {
                automation_client_1.logger.info("Returning empty object for query: %j, %s", optionsOrName, err.stack);
            }
            // tslint:disable-next-line:no-object-literal-type-assertion
            return {};
        });
    }
}
exports.LocalGraphClient = LocalGraphClient;
//# sourceMappingURL=LocalGraphClient.js.map