"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fileUrl = require("file-url");
const fs = require("fs-extra");
const path = require("path");
const snLogger = require("simple-node-logger");
const strip_ansi_1 = require("strip-ansi");
/**
 * Write log to a file using simple-node-logger in the repository
 * root of an SDM. Basically, we just
 * need any logging framework but Winston, as this is
 * separate from the SDM log itself.
 */
class SimpleNodeLoggerProgressLog {
    constructor(name, goalName, sdmRoot) {
        this.name = name;
        this.goalName = goalName;
        this.sdmRoot = sdmRoot;
        this.log = "";
        if (!fs.existsSync(sdmRoot)) {
            fs.mkdirsSync(sdmRoot);
        }
        this.logger = snLogger.createRollingFileLogger({
            logDirectory: sdmRoot,
            fileNamePattern: `${this.name.replace(/^.*\//, "")}-goals-<DATE>.log`,
            dateFormat: "YYYYMMDD",
        });
    }
    write(pWhat) {
        let what = pWhat || "";
        this.log += strip_ansi_1.default(what);
        if (what.endsWith("\n\r") || what.endsWith("\r\n")) {
            what = what.slice(0, -2);
        }
        if (what.endsWith("\n")) {
            what = what.slice(0, -1);
        }
        this.logger.info(`(${this.goalName}): ${what}`);
    }
    get url() {
        const appender = this.logger.getAppenders()[0];
        return fileUrl(path.join(this.sdmRoot, appender.createFileName()));
    }
    isAvailable() {
        return __awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
    flush() {
        return Promise.resolve();
    }
    close() {
        return Promise.resolve();
    }
}
exports.SimpleNodeLoggerProgressLog = SimpleNodeLoggerProgressLog;
//# sourceMappingURL=SimpleNodeLoggerProgressLog.js.map