"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const jsSHA = require("jssha");
const _ = require("lodash");
exports.ActionRoute = "/action";
function freshActionStore() {
    return new InMemoryActionStore();
}
exports.freshActionStore = freshActionStore;
class InMemoryActionStore {
    constructor() {
        this.actionByKey = {};
    }
    storeActions(sm) {
        return __awaiter(this, void 0, void 0, function* () {
            automation_client_1.logger.debug("Storing actions for message: %s", sm.key);
            _.flatMap(sm.attachments, a => a.actions)
                .forEach((action, i) => {
                this.actionByKey[actionKeyFor(sm, i)] = action;
            });
            return;
        });
    }
    getAction(key) {
        return __awaiter(this, void 0, void 0, function* () {
            automation_client_1.logger.info("Getting action: %s", key);
            return this.actionByKey[key];
        });
    }
}
function actionKeyFor(message, index) {
    const messageKey = `${message.key || message.ts || computeShortSha(message)}-${index}`;
    automation_client_1.logger.debug("Message key computed: %s", messageKey);
    return messageKey;
}
exports.actionKeyFor = actionKeyFor;
function computeShortSha(message) {
    const shaObj = new jsSHA("SHA-1", "TEXT");
    shaObj.update(JSON.stringify(message));
    return shaObj.getHash("HEX");
}
function actionDescription(action) {
    return encodeURIComponent(action.text);
}
exports.actionDescription = actionDescription;
//# sourceMappingURL=ActionStore.js.map