"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
/**
 * MessageClient implementation that delegates to many message clients
 */
class BroadcastingMessageClient {
    constructor(...delegates) {
        this.delegates = delegates.filter(d => !!d);
    }
    addressChannels(msg, channels, options) {
        automation_client_1.logger.log("silly", "Broadcast.addressChannels: %j", msg);
        return Promise.all(this.delegates.map(d => {
            try {
                return d.addressChannels(msg, channels, options);
            }
            catch (_a) {
                // Ignore and continue
                return undefined;
            }
        }));
    }
    addressUsers(msg, users, options) {
        automation_client_1.logger.log("silly", "Broadcast.addressUsers: %j", msg);
        return Promise.all(this.delegates.map(d => {
            try {
                return d.addressUsers(msg, users, options);
            }
            catch (_a) {
                // Ignore and continue
                return undefined;
            }
        }));
    }
    respond(msg, options) {
        automation_client_1.logger.log("silly", "Broadcast.respond: %j", msg);
        return Promise.all(this.delegates.map(d => {
            try {
                return d.respond(msg, options);
            }
            catch (_a) {
                // Ignore and continue
                return undefined;
            }
        }));
    }
    send(msg, destinations, options) {
        automation_client_1.logger.log("silly", "Broadcast.send: %j", msg);
        return Promise.all(this.delegates.map(d => {
            try {
                return d.send(msg, destinations, options);
            }
            catch (_a) {
                // Ignore and continue
                return undefined;
            }
        }));
    }
}
exports.BroadcastingMessageClient = BroadcastingMessageClient;
//# sourceMappingURL=BroadcastingMessageClient.js.map