"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const globals_1 = require("@atomist/automation-client/lib/globals");
const sdm_1 = require("@atomist/sdm");
const _ = require("lodash");
const defaultWorkspaceContextResolver_1 = require("../../../common/binding/defaultWorkspaceContextResolver");
const handlePushBasedEventOnRepo_1 = require("../../../common/git/handlePushBasedEventOnRepo");
const invokeEventHandlerInProcess_1 = require("../../invocation/invokeEventHandlerInProcess");
function isSdmGoalStoreOrUpdate(o) {
    const maybe = o;
    return !!maybe.name && !!maybe.environment;
}
exports.isSdmGoalStoreOrUpdate = isSdmGoalStoreOrUpdate;
/**
 * Runs inside an SDM in local mode. As Goal storage uses
 * a MessageClient, this enables a local SDM to react to goals that
 * it has set. This implementation ignores other messages and forwards
 * goals only.
 * Will onDispatch goal events to the appropriate event handler
 * within the SDM at the known address.
 */
class GoalEventForwardingMessageClient {
    respond(msg, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Ignore
        });
    }
    send(msg, destinations, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (isSdmGoalStoreOrUpdate(msg)) {
                automation_client_1.logger.log("silly", "Storing SDM goal or ingester payload %j", msg);
                if (handlePushBasedEventOnRepo_1.isValidSHA1((msg.branch))) {
                    throw new Error("Branch/sha confusion in " + JSON.stringify(msg));
                }
                if (!handlePushBasedEventOnRepo_1.isValidSHA1((msg.sha))) {
                    throw new Error("Invalid sha in " + JSON.stringify(msg));
                }
                let handlerNames = [];
                switch (msg.state) {
                    case sdm_1.SdmGoalState.requested:
                        handlerNames = ["FulfillGoalOnRequested"];
                        break;
                    case sdm_1.SdmGoalState.failure:
                    case sdm_1.SdmGoalState.canceled:
                    case sdm_1.SdmGoalState.stopped:
                        handlerNames = ["RespondOnGoalCompletion", "SkipDownstreamGoalsOnGoalFailure"];
                        break;
                    case sdm_1.SdmGoalState.success:
                        handlerNames = ["RespondOnGoalCompletion", "RequestDownstreamGoalsOnGoalSuccess"];
                        break;
                    case sdm_1.SdmGoalState.planned:
                    case sdm_1.SdmGoalState.pre_approved:
                    case sdm_1.SdmGoalState.skipped:
                    case sdm_1.SdmGoalState.approved:
                        break;
                    case sdm_1.SdmGoalState.waiting_for_approval:
                    case sdm_1.SdmGoalState.waiting_for_pre_approval:
                    case sdm_1.SdmGoalState.in_process:
                        handlerNames = ["RespondOnGoalCompletion"];
                        break;
                    default:
                        throw new Error(`Unexpected SdmGoalState '${msg.state}'`);
                }
                const goals = _.cloneDeep(globals_1.eventStore().messages()
                    .filter(m => m.value.sha === msg.sha
                    && m.value.goalSet
                    && m.value.goalSetId
                    && (!msg.goalSetId || m.value.goalSetId === msg.goalSetId))
                    .map(m => m.value));
                goals.forEach(g => delete g.push);
                msg.push.goals = goals;
                const payload = {
                    SdmGoal: [msg],
                };
                // We want to return to let this work in the background
                // tslint:disable-next-line:no-floating-promises
                Promise.all(handlerNames.map(name => 
                // TODO pass this in
                invokeEventHandlerInProcess_1.invokeEventHandlerInProcess(defaultWorkspaceContextResolver_1.DefaultWorkspaceContextResolver.workspaceContext)({
                    name,
                    payload,
                })));
            }
        });
    }
    addressChannels(msg, channels, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Ignore
        });
    }
    addressUsers(msg, users, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Ignore
        });
    }
}
exports.GoalEventForwardingMessageClient = GoalEventForwardingMessageClient;
//# sourceMappingURL=GoalEventForwardingMessageClient.js.map