import { Destination, MessageClient, MessageOptions, SlackMessageClient } from "@atomist/automation-client";
import { ActionStore } from "./ActionStore";
/**
 * Server-side Message client that POSTS to an Atomist client listener (which
 * is itself a server) and logs to a fallback otherwise.
 * There's one distinct HTTP message client for each address.
 */
export declare class HttpClientMessageClient implements MessageClient, SlackMessageClient {
    readonly options: {
        workspaceId: string;
        channel: string;
        port: number;
        transient: boolean;
        actionStore: ActionStore;
    };
    private readonly url;
    private readonly goalUrl;
    private dead;
    respond(message: any, options?: MessageOptions): Promise<any>;
    send(msg: any, destinations: Destination | Destination[], options?: MessageOptions): Promise<any>;
    addressChannels(message: any, channels: string | string[], options?: MessageOptions): Promise<any>;
    addressUsers(message: any, users: string | string[], options?: MessageOptions): Promise<any>;
    /**
     * Send the message, storing if necessary. Called by other methods.
     */
    private sendInternal;
    /**
     * Send the message to the client
     * @param {StreamedMessage} sm
     * @return {Promise<any>}
     */
    private stream;
    constructor(options: {
        workspaceId: string;
        channel: string;
        port: number;
        transient: boolean;
        actionStore: ActionStore;
    });
}
