"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
// this is calling a local address so it's fine
// tslint:disable-next-line:import-blacklist
const axios_1 = require("axios");
const HttpMessageListener_1 = require("../../ui/HttpMessageListener");
const currentMachineAddress_1 = require("../../util/currentMachineAddress");
const GoalEventForwardingMessageClient_1 = require("./GoalEventForwardingMessageClient");
/**
 * Server-side Message client that POSTS to an Atomist client listener (which
 * is itself a server) and logs to a fallback otherwise.
 * There's one distinct HTTP message client for each address.
 */
class HttpClientMessageClient {
    constructor(options) {
        this.options = options;
        this.url = HttpMessageListener_1.messageListenerEndpoint(options.port);
        this.goalUrl = HttpMessageListener_1.goalListenerEndpoint(options.port);
    }
    respond(message, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.addressChannels(message, this.options.channel, options);
        });
    }
    send(msg, destinations, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const dests = Array.isArray(destinations) ? destinations : [destinations];
            if (GoalEventForwardingMessageClient_1.isSdmGoalStoreOrUpdate(msg) || (msg.goals && msg.push && msg.goalSetId)) {
                return this.stream({
                    message: msg,
                    options,
                    machineAddress: currentMachineAddress_1.currentMachineAddress(),
                    destinations: dests,
                }, this.goalUrl);
            }
            return this.sendInternal(msg, dests, options);
        });
    }
    addressChannels(message, channels, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (automation_client_1.isSlackMessage(message)) {
                automation_client_1.logger.info("Storing any actions for message %j", message);
                yield this.options.actionStore.storeActions(message);
            }
            const destinations = [{
                    team: this.options.workspaceId,
                    channels: Array.isArray(channels) ? channels : [channels],
                    userAgent: undefined,
                    users: undefined,
                    addressUser: undefined,
                    addressChannel: undefined,
                }];
            return this.sendInternal(message, destinations, options);
        });
    }
    addressUsers(message, users, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.addressChannels(message, users, options);
        });
    }
    /**
     * Send the message, storing if necessary. Called by other methods.
     */
    sendInternal(message, destinations, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (automation_client_1.isSlackMessage(message)) {
                yield this.options.actionStore.storeActions(message);
            }
            return this.stream({
                message,
                options,
                machineAddress: currentMachineAddress_1.currentMachineAddress(),
                destinations,
            }, this.url);
        });
    }
    /**
     * Send the message to the client
     * @param {StreamedMessage} sm
     * @return {Promise<any>}
     */
    stream(sm, url) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.dead) {
                    automation_client_1.logger.log("silly", `Write to url ${url}: ${JSON.stringify(sm)}`);
                    yield axios_1.default.post(url, sm);
                    automation_client_1.logger.log("silly", `Wrote to url ${url}: ${JSON.stringify(sm)}`);
                }
            }
            catch (err) {
                if (this.options.transient) {
                    // Stop sending messages here. it must have gone away
                    this.dead = true;
                }
                automation_client_1.logger.log("silly", "Cannot POST to log service at [%s]: %s", this.url, err.message);
            }
        });
    }
}
exports.HttpClientMessageClient = HttpClientMessageClient;
//# sourceMappingURL=HttpClientMessageClient.js.map