"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const GitHubRepoRef_1 = require("@atomist/automation-client/lib/operations/common/GitHubRepoRef");
const os = require("os");
const defaultWorkspaceContextResolver_1 = require("../../../common/binding/defaultWorkspaceContextResolver");
const expandedTreeUtils_1 = require("./expandedTreeUtils");
/**
 * Resolve mapped parameters based on where we are in the directory tree
 * when the command was invoked.
 */
class ExpandedTreeMappedParameterResolver {
    constructor(repositoryOwnerParentDirectory, workspaceId = defaultWorkspaceContextResolver_1.DefaultWorkspaceId) {
        this.repositoryOwnerParentDirectory = repositoryOwnerParentDirectory;
        this.workspaceId = workspaceId;
    }
    resolve(md) {
        switch (md.uri) {
            case automation_client_1.MappedParameters.GitHubRepository:
                const { repo } = expandedTreeUtils_1.parseOwnerAndRepo(this.repositoryOwnerParentDirectory);
                return repo;
            case automation_client_1.MappedParameters.GitHubOwner:
                const { owner } = expandedTreeUtils_1.parseOwnerAndRepo(this.repositoryOwnerParentDirectory);
                return owner;
            case automation_client_1.MappedParameters.SlackTeam:
                // TODO fix this; the SlackTeam mapped parameters is !== atomist workspace id
                return this.workspaceId;
            case automation_client_1.MappedParameters.SlackUserName:
                return process.env.SLACK_USER_NAME || os.userInfo().username;
            case automation_client_1.MappedParameters.GitHubWebHookUrl:
                return "http://not.a.real.url";
            case automation_client_1.MappedParameters.GitHubApiUrl:
                return GitHubRepoRef_1.GitHubDotComBase;
            case automation_client_1.MappedParameters.GitHubRepositoryProvider:
                return "not-a-real-provider";
            default:
                automation_client_1.logger.warn("Mapped parameter %s not resolvable", md.uri);
                if (!md.required) {
                    return undefined;
                }
                throw new Error(`Mapped parameter ${md.uri} not resolvable`);
        }
    }
}
exports.ExpandedTreeMappedParameterResolver = ExpandedTreeMappedParameterResolver;
//# sourceMappingURL=ExpandedTreeMappedParameterResolver.js.map