"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const FileSystemRemoteRepoRef_1 = require("./FileSystemRemoteRepoRef");
/**
 * Resolve RepoRefs into our expanded tree structure
 */
class ExpandedTreeRepoRefResolver {
    constructor(opts) {
        this.opts = opts;
    }
    providerIdFromPush(push) {
        return "local";
    }
    repoRefFromPush(push) {
        return new FileSystemRemoteRepoRef_1.FileSystemRemoteRepoRef({
            repositoryOwnerParentDirectory: this.opts.repositoryOwnerParentDirectory,
            mergePullRequests: this.opts.mergePullRequests,
            owner: push.repo.owner,
            repo: push.repo.name,
            branch: push.branch,
            sha: push.commits[0].sha,
        });
    }
    repoRefFromSdmGoal(sdmGoal, provider) {
        const repo = sdmGoal.repo;
        return new FileSystemRemoteRepoRef_1.FileSystemRemoteRepoRef({
            repositoryOwnerParentDirectory: this.opts.repositoryOwnerParentDirectory,
            mergePullRequests: this.opts.mergePullRequests,
            owner: repo.owner,
            repo: repo.name,
            branch: sdmGoal.branch,
            sha: sdmGoal.sha,
        });
    }
    toRemoteRepoRef(repo, opts) {
        return new FileSystemRemoteRepoRef_1.FileSystemRemoteRepoRef({
            repositoryOwnerParentDirectory: this.opts.repositoryOwnerParentDirectory,
            mergePullRequests: this.opts.mergePullRequests,
            owner: repo.owner,
            repo: repo.name,
            branch: opts.branch,
            sha: opts.sha,
        });
    }
}
exports.ExpandedTreeRepoRefResolver = ExpandedTreeRepoRefResolver;
//# sourceMappingURL=ExpandedTreeRepoRefResolver.js.map