import { ProjectOperationCredentials, RemoteRepoRef, RepoRef } from "@atomist/automation-client";
import { ActionResult } from "@atomist/automation-client/lib/action/ActionResult";
import { AbstractRemoteRepoRef } from "@atomist/automation-client/lib/operations/common/AbstractRemoteRepoRef";
import { Configurable } from "@atomist/automation-client/lib/project/git/Configurable";
/**
 * RemoteRepoRef working against our expanded directory structure.
 * Supports cloning and pushing.
 */
export declare class FileSystemRemoteRepoRef extends AbstractRemoteRepoRef {
    kind: string;
    readonly repositoryOwnerParentDirectory: string;
    mergePullRequests: boolean;
    static fromDirectory(opts: {
        repositoryOwnerParentDirectory: string;
        mergePullRequests: boolean;
        baseDir: string;
        branch?: string;
        sha?: string;
    }): FileSystemRemoteRepoRef;
    /**
     * Remote RepoRef for the given owner and directory. Not guaranteed
     * to exist.
     * @param {string} repositoryOwnerParentDirectory
     * @param {string} owner
     * @param {string} repo
     * @return {RemoteRepoRef}
     */
    static implied(repositoryOwnerParentDirectory: string, mergePullRequests: boolean, owner: string, repo: string): RemoteRepoRef;
    createRemote(creds: ProjectOperationCredentials, description: string, visibility: any): Promise<ActionResult<this>>;
    deleteRemote(creds: ProjectOperationCredentials): Promise<ActionResult<this>>;
    raisePullRequest(creds: ProjectOperationCredentials, title: string, body: string, head: string, base: string): Promise<ActionResult<this>>;
    setUserConfig(credentials: ProjectOperationCredentials, project: Configurable): Promise<ActionResult<any>>;
    cloneUrl(): string;
    readonly url: string;
    readonly fileSystemLocation: string;
    constructor(opts: {
        repositoryOwnerParentDirectory: string;
        mergePullRequests: boolean;
        owner: string;
        repo: string;
        branch?: string;
        sha?: string;
    });
}
export declare function isFileSystemRemoteRepoRef(rr: RepoRef): rr is FileSystemRemoteRepoRef;
