"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const TargetsParams_1 = require("@atomist/automation-client/lib/operations/common/params/TargetsParams");
const repoFilter_1 = require("@atomist/automation-client/lib/operations/common/repoFilter");
const FileSystemRemoteRepoRef_1 = require("./FileSystemRemoteRepoRef");
/**
 * Repo targeting for local use.
 */
let LocalRepoTargets = class LocalRepoTargets extends TargetsParams_1.TargetsParams {
    constructor(opts) {
        super();
        this.opts = opts;
        this.repos = ".*";
        this.token = "this.is.not.your.token.and.does.not.matter";
        const orgFilter = rr => !!this.owner ? rr.owner === this.owner : true;
        this.test = repoFilter_1.andFilter(orgFilter, super.test);
    }
    get credentials() {
        return { token: this.token };
    }
    /**
     * Return a single RepoRef or undefined if we're not identifying a single repo
     * @return {RepoRef}
     */
    get repoRef() {
        const rr = (!!this.owner && !!this.repo && !this.usesRegex) ?
            new FileSystemRemoteRepoRef_1.FileSystemRemoteRepoRef({
                repositoryOwnerParentDirectory: this.opts.repositoryOwnerParentDirectory,
                mergePullRequests: this.opts.mergePullRequests,
                owner: this.owner,
                repo: this.repo,
                branch: this.branch,
                sha: this.sha,
            }) :
            undefined;
        automation_client_1.logger.debug("LocalRepoTargets returning %j: state=%j", rr, this);
        return rr;
    }
    bindAndValidate() {
        // nothing to do
    }
};
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubApiUrl, false),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "apiUrl", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubOwner, false),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "owner", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubRepository, false),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "repo", void 0);
__decorate([
    automation_client_1.Parameter(Object.assign(Object.assign({ description: "Branch or ref. Defaults to current HEAD" }, automation_client_1.validationPatterns.GitBranchRegExp), { required: false })),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "branch", void 0);
__decorate([
    automation_client_1.Parameter(Object.assign(Object.assign({ description: "Sha" }, automation_client_1.validationPatterns.GitShaRegExp), { required: false })),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "sha", void 0);
__decorate([
    automation_client_1.Parameter({ description: "regex", required: false }),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "repos", void 0);
__decorate([
    automation_client_1.Secret(automation_client_1.Secrets.userToken(["repo", "user:email", "read:user"])),
    __metadata("design:type", String)
], LocalRepoTargets.prototype, "token", void 0);
LocalRepoTargets = __decorate([
    automation_client_1.Parameters(),
    __metadata("design:paramtypes", [Object])
], LocalRepoTargets);
exports.LocalRepoTargets = LocalRepoTargets;
//# sourceMappingURL=LocalRepoTargets.js.map