"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const _ = require("lodash");
const path = require("path");
const util_1 = require("util");
const FileSystemRemoteRepoRef_1 = require("./FileSystemRemoteRepoRef");
/**
 * Find all repos under the given expanded directory structure
 * @param {string} repositoryOwnerParentDirectory
 * @return {RepoFinder}
 */
function expandedTreeRepoFinder(opts) {
    return () => __awaiter(this, void 0, void 0, function* () {
        const owners = yield allDirectoriesUnder(opts.repositoryOwnerParentDirectory);
        const projects = yield flatmapAsync(owners, allDirectoriesUnder);
        const gitProjects = yield filterAsync(projects, containsDirectory(".git"));
        const eligibleDirectories = gitProjects;
        return eligibleDirectories.map(dir => FileSystemRemoteRepoRef_1.FileSystemRemoteRepoRef.fromDirectory({
            repositoryOwnerParentDirectory: opts.repositoryOwnerParentDirectory,
            mergePullRequests: opts.mergePullRequests,
            baseDir: dir,
        }));
    });
}
exports.expandedTreeRepoFinder = expandedTreeRepoFinder;
const readdir = util_1.promisify(fs.readdir);
const filestat = util_1.promisify(fs.stat);
/* returns the whole path to each directory under `parent` */
function allDirectoriesUnder(parent) {
    return __awaiter(this, void 0, void 0, function* () {
        const allFilenamesUnder = yield readdir(parent);
        const allPathsUnder = allFilenamesUnder.map(filename => path.join(parent, filename));
        return filterAsync(allPathsUnder, pathIsDirectory);
    });
}
function pathIsDirectory(filepath) {
    return __awaiter(this, void 0, void 0, function* () {
        // will throw if filepath does not exist
        return filestat(filepath).then(stat => stat.isDirectory());
    });
}
function flatmapAsync(arr, f) {
    return __awaiter(this, void 0, void 0, function* () {
        return _.flatten(yield Promise.all(arr.map(f)));
    });
}
function filterAsync(arr, f) {
    return __awaiter(this, void 0, void 0, function* () {
        const enoughInfo = yield Promise.all(arr.map(t => f(t)
            .then((truthiness) => [t, truthiness])));
        return enoughInfo.filter(wt => wt[1]).map(wt => wt[0]);
    });
}
function containsDirectory(dirName) {
    return (parentDir) => __awaiter(this, void 0, void 0, function* () {
        return (yield filestat(path.join(parentDir, dirName))
            .catch(() => NotADirectory)).isDirectory();
    });
}
const NotADirectory = {
    isDirectory() {
        return false;
    },
};
//# sourceMappingURL=expandedTreeRepoFinder.js.map