/**
 * Return the directory in our expanded structure for the given directory
 * @param {string} repositoryOwnerParentDirectory
 * @param {string} owner
 * @param {string} repo
 * @return {string}
 */
export declare function dirFor(repositoryOwnerParentDirectory: string, owner: string, repo: string): string;
/**
 * Transform native win32 path to a cygwin path.  All backslashes in
 * the path are replaced forward slashes and any leading drive letter
 * is incorprated into the path as a lowercased first directory of the
 * path.
 *
 * @param dir original path
 * @return cygwinized path
 */
export declare function cygwinizePath(dir: string): string;
/**
 * Determine whether baseDir is under repositoryOwnerparentdirectory.
 * On win32, the comparison is done in a case-insensitive way and also
 * tries to deal with Cygwin-style paths.
 *
 * @param repositoryOwnerParentDirectory parent directory
 * @param baseDir putative child directory
 * @return true if baseDir is under repositoryownerparentdirectory
 */
export declare function isWithin(repositoryOwnerParentDirectory: string, baseDir: string): boolean;
/**
 * Find the owner and repo from the given directory, returning the empty
 * object if it isn't within our expanded directory structure
 * @param {string} repositoryOwnerParentDirectory
 * @param {string} baseDir directory to test
 * @return {{owner?: string; repo?: string}}
 */
export declare function parseOwnerAndRepo(repositoryOwnerParentDirectory: string, baseDir?: string): {
    owner?: string;
    repo?: string;
};
/**
 * Is the given directory within the expanded directory tree?
 * @param repositoryOwnerParentDirectory
 * @param {string} baseDir directory to test
 * @return {boolean}
 */
export declare function withinExpandedTree(repositoryOwnerParentDirectory: string, baseDir?: string): boolean;
/**
 * Where are we running? Respecting symlinks
 * @return {string}
 */
export declare function determineCwd(): string;
