"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const serializeError = require("serialize-error");
const CommandCompletionDestination_1 = require("../../../common/ui/CommandCompletionDestination");
/**
 * Event listener that sends an event on command termination
 */
class NotifyOnCompletionAutomationEventListener extends automation_client_1.AutomationEventListenerSupport {
    commandSuccessful(payload, ctx, result) {
        return ctx.messageClient.send("Success", CommandCompletionDestination_1.CommandCompletionDestination);
    }
    commandFailed(payload, ctx, error) {
        // Route the failure report to the client
        automation_client_1.logger.error("Sending error 'msg'", error.message, serializeError(error));
        return ctx.messageClient.send({
            kind: FailureKind,
            error: serializeError(error),
        }, CommandCompletionDestination_1.CommandCompletionDestination);
    }
}
exports.NotifyOnCompletionAutomationEventListener = NotifyOnCompletionAutomationEventListener;
const FailureKind = "failure";
function isFailureMessage(o) {
    return o.kind === FailureKind;
}
exports.isFailureMessage = isFailureMessage;
//# sourceMappingURL=NotifyOnCompletionAutomationEventListener.js.map