"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const slack_messages_1 = require("@atomist/slack-messages");
const newCorrelationId_1 = require("../../../cli/invocation/http/support/newCorrelationId");
/**
 * Notify the local feed that an SDM is now connected and ready to receive traffic.
 */
class NotifyOnStartupAutomationEventListener extends automation_client_1.AutomationEventListenerSupport {
    startupSuccessful(client) {
        return __awaiter(this, void 0, void 0, function* () {
            const correlationId = yield newCorrelationId_1.newCliCorrelationId({ channel: "general", encodeListenerPort: true });
            const messageClient = client.configuration.http.messageClientFactory({ context: { correlationId } });
            const message = `__${client.configuration.sdm.name ? client.configuration.sdm.name : "SDM"}__ ${slack_messages_1.codeLine(`${client.configuration.name}:${client.configuration.version}`)}`;
            yield messageClient.respond(`${message} is now connected`);
            automation_client_1.registerShutdownHook(() => __awaiter(this, void 0, void 0, function* () {
                yield messageClient.respond(`${message} disconnected`);
                return 0;
            }), 5000, "send shutdown chat message");
        });
    }
}
exports.NotifyOnStartupAutomationEventListener = NotifyOnStartupAutomationEventListener;
//# sourceMappingURL=NotifyOnStartupAutomationEventListener.js.map