"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const propertiesToArgs_1 = require("../../common/util/propertiesToArgs");
const EnvironmentTokenCredentialsResolver_1 = require("../binding/EnvironmentTokenCredentialsResolver");
const DefaultCommandHandlerCallback = () => { };
function invokeCommandHandlerInProcess(callback = DefaultCommandHandlerCallback) {
    return (invocation) => __awaiter(this, void 0, void 0, function* () {
        const parameters = propertiesToArgs_1.propertiesToArgs(invocation.parameters);
        const data = {
            command: invocation.name,
            parameters,
            mapped_parameters: propertiesToArgs_1.propertiesToArgs(invocation.mappedParameters || {}).concat([
                { name: "slackTeam", value: invocation.workspaceId },
            ]).concat(parameters),
            secrets: (invocation.secrets || []).concat([
                { uri: "github://user_token?scopes=repo,user:email,read:user", value: EnvironmentTokenCredentialsResolver_1.credentialsFromEnvironment().token },
            ]),
            correlation_id: invocation.correlationId,
            api_version: "1",
            team: {
                id: invocation.workspaceId,
                name: invocation.workspaceName,
            },
        };
        automation_client_1.automationClientInstance().processCommand(data, callback);
        return automation_client_1.Success;
    });
}
exports.invokeCommandHandlerInProcess = invokeCommandHandlerInProcess;
//# sourceMappingURL=invokeCommandHandlerInProcess.js.map