"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const string_1 = require("@atomist/automation-client/lib/internal/util/string");
const stringify = require("json-stringify-safe");
const assert = require("power-assert");
const newCorrelationId_1 = require("../../cli/invocation/http/support/newCorrelationId");
const EnvironmentTokenCredentialsResolver_1 = require("../binding/EnvironmentTokenCredentialsResolver");
/**
 * Invoke an event handler on the automation client at the given location
 * @return {Promise<HandlerResult>}
 */
function invokeEventHandlerInProcess(workspaceContext, correlationId) {
    return (invocation) => __awaiter(this, void 0, void 0, function* () {
        if (!automation_client_1.automationClientInstance()) {
            throw new Error("This function must be invoked inside an automation client locally");
        }
        // tslint:disable-next-line:variable-name
        const team_id = workspaceContext.workspaceId;
        // tslint:disable-next-line:variable-name
        const team_name = workspaceContext.workspaceName;
        const token = EnvironmentTokenCredentialsResolver_1.credentialsFromEnvironment().token;
        const data = {
            extensions: {
                operationName: invocation.name,
                query_id: "q-" + Date.now(),
                team_id,
                team_name,
                correlation_id: correlationId || (yield newCorrelationId_1.newCliCorrelationId()),
            },
            secrets: (invocation.secrets || []).concat([
                { uri: "github://user_token?scopes=repo,user:email,read:user", value: token },
                { uri: "github://org_token", value: token },
                { uri: automation_client_1.Secrets.OrgToken, value: token },
                { uri: automation_client_1.Secrets.UserToken, value: token },
            ]),
            api_version: "1",
            data: invocation.payload,
        };
        automation_client_1.logger.log("silly", "Invoking %s using %s", invocation.name, stringify(data, string_1.replacer));
        automation_client_1.automationClientInstance().processEvent(data, (result) => __awaiter(this, void 0, void 0, function* () {
            const results = (Array.isArray(result) ? result : [result]);
            assert(results.find(r => r.code !== 0), "Event handler did not succeed. Returned: " + JSON.stringify(result, undefined, 2));
            return results;
        }));
        return automation_client_1.Success;
    });
}
exports.invokeEventHandlerInProcess = invokeEventHandlerInProcess;
//# sourceMappingURL=invokeEventHandlerInProcess.js.map