/**
 * Construction arguments to HttpMessageListener
 */
export declare class HttpMessageListenerParameters {
    readonly port: number;
    /**
     * True if this is a short-lived listener that can be shut down
     * once it's seen a command completion event
     */
    readonly transient: boolean;
    /**
     * If this is set, a list of channels we should listen on
     */
    readonly channels?: string[] | string;
    /**
     * Whether to display information about the invocation of commands.
     * Useful when developing SDM commands.
     */
    readonly verbose?: boolean;
    readonly goals?: boolean;
}
/**
 * Start process to listen to HTTP POSTs from HttpClientMessageClient
 * and display them to the console. Supports both both structured
 * messages coming from a MessageClient and raw output sent
 * for diagnostic or other purposes.
 * The /message endpoint takes StreamedMessage
 * The /write endpoint takes a simple { message }
 */
export declare class HttpMessageListener {
    readonly parameters: HttpMessageListenerParameters;
    private seenCompletion;
    private server;
    private readonly goalRenderer;
    private readonly channels;
    /**
     * Could this process be terminated, based on its configuration
     * and the events it has seen?
     * @return {boolean}
     */
    readonly canTerminate: boolean;
    /**
     * Start the server process
     * @return {this}
     */
    start(): this;
    private addMessageRoute;
    private addGoalRoute;
    /**
     * Add raw write route
     */
    private addWriteRoute;
    /**
     * Start process to listen to HTTP messages from HttpClientMessageClient
     * and display them to the console
     */
    constructor(parameters: HttpMessageListenerParameters);
}
/**
 * What will the URL of this listener be?
 * @param {number} demonPort
 * @return {string}
 */
export declare function messageListenerEndpoint(demonPort: number): string;
export declare function goalListenerEndpoint(demonPort: number): string;
export declare function messageListenerRoot(demonPort: number): string;
/**
 * Is the all messages message listener running?
 * @param demonPort port to check on
 */
export declare function isFeedListenerRunning(demonPort?: number): Promise<boolean>;
