"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const runAndLog_1 = require("./runAndLog");
/**
 * Get the last git commit message
 * @param {GitProject} p
 * @return {Promise<string>}
 */
function lastCommitMessage(p) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = yield runAndLog_1.runAndLog("git log -1 --pretty=%B", { cwd: p.baseDir });
        return r.stdout.trim();
    });
}
exports.lastCommitMessage = lastCommitMessage;
function commitMessageForSha(sha, p) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = yield runAndLog_1.runAndLog(`git log --format=%B -n 1 ${sha}`, { cwd: p.baseDir });
        return r.stdout.trim();
    });
}
exports.commitMessageForSha = commitMessageForSha;
function timestampFromCommit(sha, p) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = yield runAndLog_1.runAndLog(`git log --format=%ct -n 1 ${sha}`, { cwd: p.baseDir });
        return r.stdout.trim();
    });
}
exports.timestampFromCommit = timestampFromCommit;
/**
 * Retrieve a number of git placeholders in a single call, for a particular sha
 * Return an indexed property by the codes, such as H
 * See https://git-scm.com/docs/git-log
 * @param {GitProject} p
 * @return {Promise<string>}
 */
function retrieveLogDataForSha(sha, baseDir, ...placeholders) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = yield runAndLog_1.runAndLog(`git log --format="${placeholders.map(ph => `%${ph}`).join(" ")}" -n 1 ${sha}`, { cwd: baseDir });
        const trimmed = r.stdout.trim();
        // We have an array of strings
        const result = {};
        trimmed.split(" ").forEach((retrieved, i) => result[placeholders[i]] = retrieved);
        automation_client_1.logger.info("Git result for %s is %j", placeholders, result);
        return result;
    });
}
exports.retrieveLogDataForSha = retrieveLogDataForSha;
function lastSha(p) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = yield runAndLog_1.runAndLog("git log -1 --format=format:%H", { cwd: p.baseDir });
        return r.stdout.trim();
    });
}
exports.lastSha = lastSha;
/**
 * Get the last shas for this project
 * @param {GitProject} p
 * @return {Promise<string[]>}
 */
function shaHistory(p) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = yield runAndLog_1.runAndLog("git log --format=format:%H", { cwd: p.baseDir });
        return r.stdout.trim().split("\n");
    });
}
exports.shaHistory = shaHistory;
//# sourceMappingURL=git.js.map