"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const child_process_1 = require("child_process");
const util_1 = require("util");
/**
 * Shell out to the given command showing stdout and stderr
 * @param {string} cmd
 * @param {module:child_process.ExecOptions} opts
 * @return {Promise<{stdout: string; stderr: string}>}
 */
function runAndLog(cmd, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = yield util_1.promisify(child_process_1.exec)(cmd, opts);
        automation_client_1.logger.info("[%s] %s stdout was \n%s", opts.cwd, cmd, result.stdout);
        if (!!result.stderr) {
            automation_client_1.logger.warn("[%s] %s stderr was \n%s", opts.cwd, cmd, result.stderr);
        }
        return result;
    });
}
exports.runAndLog = runAndLog;
//# sourceMappingURL=runAndLog.js.map