import { SdmGoalEvent } from "@atomist/sdm";
import { KubernetesApplication } from "../kubernetes/request";
export declare type ExternalUrls = Array<{
    label?: string;
    url: string;
}>;
/**
 * Return proper SDM goal externalUrls structure or `undefined` if
 * there is no externally accessible endpoint.
 */
export declare function appExternalUrls(ka: KubernetesApplication, ge: SdmGoalEvent): Promise<ExternalUrls | undefined>;
/**
 * Create the URL for a deployment using the protocol, host, and path
 * from the [[KubernetesApplication]] object.  If `ka.path` is not
 * truthy, no ingress was created so return `undefined`.  If the path
 * does not begin and end with a forward slash, /, add them.  If
 * protocol is not provided, use "https" if tlsSecret is provided,
 * otherwise "http".  If host is not provided and the SDM is running
 * in local mode, it attempts to find the IP address of any locally
 * running Kubernetes cluster and falls back to "127.0.0.1".  If the
 * host is not given and the SDM is _not_ in local mode, `undefined`
 * is returned.
 *
 * @param ka Kubernetes application
 * @return endpoint URL for deployment service
 */
export declare function endpointBaseUrl(ka: Pick<KubernetesApplication, "host" | "path" | "protocol" | "tlsSecret">): Promise<string | undefined>;
//# sourceMappingURL=externalUrls.d.ts.map