import { OnEvent } from "@atomist/automation-client";
import { EventHandlerRegistration, SoftwareDeliveryMachineConfiguration } from "@atomist/sdm";
import { KubernetesDeployRequestedSdmGoal } from "../typings/types";
/**
 * Parameters for the deploying an application to a Kubernetes cluster
 * via an event subscription.
 */
export declare class KubernetesDeployParameters {
    /**
     * Make the entire SDM configuration available to this event
     * handler.  The specific properties used are:
     *
     * `name`: Name of this SDM.  Only requested SDM Kubernetes
     * deployment goals whose fulfillment name match this name are
     * deployed by this SDM.
     *
     * `sdm.logFactory`: Used to generate a log sink to send progress
     * logs to.
     */
    configuration: SoftwareDeliveryMachineConfiguration;
}
/**
 * Event handler for deploying an application to a Kubernetes cluster.
 * The definition of the application to be deployed is handled by the
 * [[KubernetesDeploy]] goal of this or another SDM.  This SDM will
 * execute deployments configured for it, see [[eligibleDeployGoal]]
 * and [[verifyKubernetesApplicationDeploy]] for details.
 */
export declare const HandleKubernetesDeploy: OnEvent<KubernetesDeployRequestedSdmGoal.Subscription, KubernetesDeployParameters>;
/**
 * Create an event handler registration for this SDM to deploy
 * requested Kubernetes applications.
 */
export declare function kubernetesDeployHandler(self: string): EventHandlerRegistration<KubernetesDeployRequestedSdmGoal.Subscription, KubernetesDeployParameters>;
//# sourceMappingURL=kubernetesDeploy.d.ts.map