"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const stringify = require("json-stringify-safe");
const fulfiller_1 = require("../deploy/fulfiller");
/**
 * Parameters for the deploying an application to a Kubernetes cluster
 * via an event subscription.
 */
let KubernetesDeployParameters = class KubernetesDeployParameters {
};
__decorate([
    automation_client_1.Value("") // empty path returns the entire configuration
    ,
    __metadata("design:type", Object)
], KubernetesDeployParameters.prototype, "configuration", void 0);
KubernetesDeployParameters = __decorate([
    automation_client_1.Parameters()
], KubernetesDeployParameters);
exports.KubernetesDeployParameters = KubernetesDeployParameters;
/**
 * Event handler for deploying an application to a Kubernetes cluster.
 * The definition of the application to be deployed is handled by the
 * [[KubernetesDeploy]] goal of this or another SDM.  This SDM will
 * execute deployments configured for it, see [[eligibleDeployGoal]]
 * and [[verifyKubernetesApplicationDeploy]] for details.
 */
exports.HandleKubernetesDeploy = (ef, context, params) => __awaiter(void 0, void 0, void 0, function* () {
    if (!ef || !ef.data || !ef.data.SdmGoal) {
        automation_client_1.logger.warn(`Received event had no SdmGoal`);
        return automation_client_1.Success;
    }
    return Promise.all(ef.data.SdmGoal.map((g) => __awaiter(void 0, void 0, void 0, function* () {
        const goalEvent = g;
        const progressLog = new sdm_1.WriteToAllProgressLog(goalEvent.name, new sdm_1.LoggingProgressLog(goalEvent.name, "debug"), yield params.configuration.sdm.logFactory(context, goalEvent));
        try {
            const result = yield fulfiller_1.executeKubernetesDeployFulfill({ context, goalEvent, progressLog });
            const updateParams = {
                state: (result.code) ? sdm_1.SdmGoalState.failure : sdm_1.SdmGoalState.success,
                description: result.description,
                error: (result.code) ? new Error(result.message) : undefined,
                externalUrls: result.externalUrls,
            };
            try {
                yield sdm_1.updateGoal(context, goalEvent, updateParams);
            }
            catch (e) {
                const msg = `Failed to update SDM goal ${goalEventString(goalEvent)} with params '${stringify(updateParams)}': ${e.message}`;
                progressLog.write(msg);
                result.message = `${e.message}; ${msg}`;
            }
            if (!result.code) {
                result.code = 0;
            }
            return result;
        }
        catch (e) {
            return failGoal(context, goalEvent, e.message, progressLog);
        }
    })))
        .then(automation_client_1.reduceResults);
});
/**
 * Create an event handler registration for this SDM to deploy
 * requested Kubernetes applications.
 */
function kubernetesDeployHandler(self) {
    return {
        name: "KubernetesDeploy",
        description: "Deploy application resources to Kubernetes cluster",
        tags: ["deploy", "kubernetes"],
        subscription: automation_client_1.GraphQL.subscription({ name: "KubernetesDeployRequestedSdmGoal", variables: { fulfillmentName: self } }),
        paramsMaker: KubernetesDeployParameters,
        listener: exports.HandleKubernetesDeploy,
    };
}
exports.kubernetesDeployHandler = kubernetesDeployHandler;
/**
 * Fail the provided goal using the message to set the description and
 * error message.
 *
 * @param context handler context to use to send the update
 * @param goalEvent SDM goal to update
 * @param message informative error message
 * @return a failure handler result using the provided error message
 */
function failGoal(context, goalEvent, message, log) {
    return __awaiter(this, void 0, void 0, function* () {
        log.write(message);
        const params = {
            state: sdm_1.SdmGoalState.failure,
            description: message,
            error: new Error(message),
        };
        try {
            yield sdm_1.updateGoal(context, goalEvent, params);
        }
        catch (e) {
            const msg = `Failed to update SDM goal '${goalEventString(goalEvent)}' with params '${stringify(params)}': ${e.message}`;
            log.write(msg);
            return { code: 2, message: `${message}; ${msg}` };
        }
        return { code: 1, message };
    });
}
/** Unique string for goal event. */
function goalEventString(goalEvent) {
    return `${goalEvent.goalSetId}/${goalEvent.uniqueName}`;
}
//# sourceMappingURL=kubernetesDeploy.js.map