"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const role_1 = require("./role");
const roleBinding_1 = require("./roleBinding");
const serviceAccount_1 = require("./serviceAccount");
/**
 * Create requested RBAC resources if they do not exist.  If
 * `req.roleSpec` is truthy, the service account, role, and binding
 * are created.  If `req.roleSpect` is falsey but
 * `req.serviceAccountSpec` is truthy, only the service account is
 * created.  If any of the RBAC resources exist and their
 * corresponding partial spec is provided in `req`, the resource is
 * patched.
 *
 * @param req Kuberenetes application request
 * @return Kubernetes RBAC resource specs that were created or patched, some may be undefined
 */
function upsertRbac(req) {
    return __awaiter(this, void 0, void 0, function* () {
        if (req.roleSpec && !req.serviceAccountSpec) {
            req.serviceAccountSpec = {};
        }
        const resources = {};
        if (req.serviceAccountSpec) {
            resources.serviceAccount = yield serviceAccount_1.upsertServiceAccount(req);
        }
        if (req.roleSpec) {
            resources.role = yield role_1.upsertRole(req);
            resources.roleBinding = yield roleBinding_1.upsertRoleBinding(req);
        }
        return resources;
    });
}
exports.upsertRbac = upsertRbac;
//# sourceMappingURL=rbac.js.map