import * as request from "request";
/**
 * Extract message from a variety of error types.  If message is not
 * found in any of the standard places, safely stringify the error.
 *
 * @param e Some sort of Error or similar
 * @return Error message
 */
export declare function errMsg(e: any): string;
/** Interface to package request response and body. */
export interface RequestResponse {
    response: request.Response;
    body: any;
}
/**
 * Extract an error message from the error object, which may have
 * derived from a request response.
 *
 * @param e Some sort of Error or similar
 * @return Error message if found, `undefined` otherwise
 */
export declare function requestErrMsg(e: Partial<RequestResponse>): string | undefined;
/**
 * Create an Error object from a request response and body.  The
 * response and body will be included in the Error object.
 *
 * @param r Request response and body
 * @return Error object with meaningful message, if possible
 */
export declare function requestError(r: RequestResponse): Error & RequestResponse;
/**
 * Mask a string containing potentially sensitive information.
 *
 * @param raw String to mask
 * @return Masked string
 */
export declare function maskString(raw: string): string;
//# sourceMappingURL=error.d.ts.map