"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const sdm_core_1 = require("@atomist/sdm-core");
const config_1 = require("../kubernetes/config");
/**
 * If the SDM is running in local mode, the `DOCKER_HOST` environment
 * variable is not set, and using a minikube cluster, use `minikube
 * docker-env` to set the Docker environment variables.
 */
exports.minikubeStartupListener = (context) => __awaiter(void 0, void 0, void 0, function* () {
    if (!sdm_core_1.isInLocalMode()) {
        return;
    }
    automation_client_1.logger.debug(`Executing minikube startup listener`);
    if (process.env.DOCKER_HOST) {
        automation_client_1.logger.info(`Using provided Docker environment variables: DOCKER_HOST=${process.env.DOCKER_HOST}`);
        return;
    }
    const configContext = config_1.kubeConfigContext(context.sdm);
    if (configContext !== "minikube") {
        automation_client_1.logger.debug(`Context '${configContext}' is not minikube, not executing minikube startup listener`);
        return;
    }
    try {
        const result = yield sdm_1.execPromise("minikube", ["docker-env"]);
        const envVars = processMinikubeDockeEnv(result.stdout);
        Object.keys(envVars).forEach(e => process.env[e] = envVars[e]);
        automation_client_1.logger.info("Configured local minikube Docker environment");
    }
    catch (e) {
        automation_client_1.logger.warn(`Failed to configure local minikube Docker environment: ${e.message}`);
    }
});
/**
 * Parse output of `minikube docker-env` and return an object of
 * environment variable names and values.
 *
 * @param env Raw output of `minikube docker-env`
 * @return `{ ENV_VAR1: VALUE1, ENV_VAR2: VALUE2, … }`
 */
function processMinikubeDockeEnv(env) {
    const dockerEnv = {};
    if (!env) {
        return dockerEnv;
    }
    env.split("\n").filter(l => l && !l.startsWith("#")).forEach(l => {
        const [e, v] = l.replace("export ", "").replace(/="/, "=").replace(/"$/, "").split("=");
        dockerEnv[e] = v;
    });
    return dockerEnv;
}
exports.processMinikubeDockeEnv = processMinikubeDockeEnv;
//# sourceMappingURL=minikube.js.map