"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const pRetry = require("p-retry");
const error_1 = require("./error");
exports.DefaultLogRetryOptions = {
    retries: 5,
    factor: 2,
    minTimeout: 0.1 * 1000,
    maxTimeout: 3 * 1000,
    randomize: true,
};
/**
 * Add logging to promise-based retry.
 */
function logRetry(f, desc, options = exports.DefaultLogRetryOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const opts = Object.assign({ onFailedAttempt: e => {
                automation_client_1.logger.debug(`Error in ${desc} attempt ${e.attemptNumber}: ${error_1.errMsg(e)}`);
            } }, options);
        return pRetry((count) => __awaiter(this, void 0, void 0, function* () {
            automation_client_1.logger.debug(`Retry ${desc} attempt ${count}`);
            let r;
            try {
                r = yield f();
            }
            catch (e) {
                if (!(e instanceof Error)) {
                    const err = new Error(error_1.errMsg(e));
                    Object.keys(e).forEach(k => err[k] = e[k]);
                    throw err;
                }
                throw e;
            }
            return r;
        }), opts);
    });
}
exports.logRetry = logRetry;
//# sourceMappingURL=retry.js.map