"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const sdm_core_1 = require("@atomist/sdm-core");
const cluster = require("cluster");
const _ = require("lodash");
const repo_1 = require("./repo");
const sync_1 = require("./sync");
/**
 * If the SDM is registered with one or more workspaces and not
 * running in local mode, query cortex for the sync repo and replace
 * the sdm.k8s.options.sync.repo property with a RemoteRepoRef for
 * that repo.  If this is the cluster master, it will create a job to
 * perform a sync.  If this is the cluster master and the SDM
 * configuration has a [[KubernetesSyncOptions]] with a positive value
 * of `intervalMinutes`, it will set up an interval timer to apply the
 * specs from the sync repo periodically.
 */
exports.syncRepoStartupListener = (ctx) => __awaiter(void 0, void 0, void 0, function* () {
    if (sdm_core_1.isInLocalMode()) {
        return;
    }
    const sdm = ctx.sdm;
    if (!(yield repo_1.queryForScmProvider(sdm))) {
        return;
    }
    if (!cluster.isMaster) {
        return;
    }
    yield sdmRepoSync(sdm);
    const interval = _.get(sdm, "configuration.sdm.k8s.options.sync.intervalMinutes");
    if (interval && interval > 0) {
        automation_client_1.logger.info(`Creating sync repo trigger every ${interval} minutes`);
        setInterval(() => sdmRepoSync(sdm), interval * 60 * 1000);
    }
    return;
});
/**
 * Create the trappings required for executing a command and then
 * create a job to execute the [[kubernetesSync]] command.
 */
function sdmRepoSync(sdm) {
    return __awaiter(this, void 0, void 0, function* () {
        const workspaceId = sdm.configuration.workspaceIds[0];
        const context = sdm_1.fakeContext(workspaceId);
        context.graphClient = sdm.configuration.graphql.client.factory.create(workspaceId, sdm.configuration);
        automation_client_1.logger.info("Creating sync repo job");
        yield sdm_1.createJob({ command: sync_1.KubernetesSync, parameters: [{}] }, context);
        return;
    });
}
//# sourceMappingURL=startup.js.map