import { Severity } from "@atomist/automation-client";
import { ReviewListener } from "../../../api/listener/ReviewListener";
import { PushImpactResponse } from "../../../api/registration/PushImpactListenerRegistration";
import { ReviewListenerRegistration } from "../../../api/registration/ReviewListenerRegistration";
/**
 * Return review listener function that returns `pir` if there are any
 * review comments with the provided severity.  The method will short
 * circuit once any comment with the provided severity is found.
 *
 * @param pir push impact response to return if there are comments
 *            with provided severities
 * @param severity severity that should trigger a return of `pir`,
 *                 "error" by default
 * @return ReviewListener that returns `pir` if any comments have
 *         a severity of `severity`, proceed otherwise
 */
export declare function severityReviewListener(pir: PushImpactResponse, severity?: Severity): ReviewListener;
/**
 * Listener that fails the code inspection if the review has any
 * error comments.
 */
export declare const FailGoalIfErrorComments: ReviewListenerRegistration;
/**
 * Listener that requires approval on the code inspection if the
 * review has any error comments.
 */
export declare const ApproveGoalIfErrorComments: ReviewListenerRegistration;
/**
 * Listener that fails the code inspection if the review has any
 * warn comments.
 */
export declare const FailGoalIfWarnComments: ReviewListenerRegistration;
/**
 * Listener that requires approval on the code inspection if the
 * review has any warn comments.
 */
export declare const ApproveGoalIfWarnComments: ReviewListenerRegistration;
//# sourceMappingURL=goalReviewListener.d.ts.map