"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const _ = require("lodash");
/**
 * Check if current goal is already canceled
 */
function isGoalCanceled(sdmGoal, ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        // Validate that goal hasn't been canceled in the meantime
        const goalCanceled = yield ctx.graphClient.query({
            name: "CanceledSdmGoal",
            variables: {
                goalSetId: sdmGoal.goalSetId,
                uniqueName: sdmGoal.uniqueName,
            },
            options: automation_client_1.QueryNoCacheOptions,
        });
        return goalCanceled && goalCanceled.SdmGoal && goalCanceled.SdmGoal.length > 0;
    });
}
exports.isGoalCanceled = isGoalCanceled;
/**
 * Can goal be canceled
 */
function cancelableGoal(sdmGoal, configuration) {
    return __awaiter(this, void 0, void 0, function* () {
        const uncancelableGoals = _.get(configuration, "sdm.goal.uncancelable") || [];
        return !(uncancelableGoals.includes(sdmGoal.uniqueName) || uncancelableGoals.includes(sdmGoal.name));
    });
}
exports.cancelableGoal = cancelableGoal;
//# sourceMappingURL=cancelGoals.js.map