"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const minimalClone_1 = require("../goal/minimalClone");
const computeFingerprints_1 = require("./computeFingerprints");
const createPushImpactListenerInvocation_1 = require("./createPushImpactListenerInvocation");
const relevantCodeActions_1 = require("./relevantCodeActions");
/**
 * Execute fingerprinting and send fingerprints to Atomist
 * @param {FingerprinterRegistration} fingerprinters
 * @param listeners listeners to fingerprints
 */
function executeFingerprinting(fingerprinters, listeners) {
    return (goalInvocation) => __awaiter(this, void 0, void 0, function* () {
        const { goalEvent, configuration, id, credentials, context, preferences, addressChannels } = goalInvocation;
        if (fingerprinters.length === 0) {
            return automation_client_1.Success;
        }
        automation_client_1.logger.debug("About to fingerprint %j using %d fingerprinters", id, fingerprinters.length);
        yield configuration.sdm.projectLoader.doWithProject({
            credentials,
            id,
            readOnly: true,
            cloneOptions: minimalClone_1.minimalClone(goalEvent.push, { detachHead: true }),
        }, (project) => __awaiter(this, void 0, void 0, function* () {
            const cri = yield createPushImpactListenerInvocation_1.createPushImpactListenerInvocation(goalInvocation, project);
            const relevantFingerprinters = yield relevantCodeActions_1.relevantCodeActions(fingerprinters, cri);
            automation_client_1.logger.debug("Will invoke %d eligible fingerprinters of %d to %j", relevantFingerprinters.length, fingerprinters.length, cri.project.id);
            const fingerprints = yield computeFingerprints_1.computeFingerprints(cri, relevantFingerprinters.map(fp => fp.action));
            yield Promise.all(listeners.map(l => l({
                id,
                context,
                configuration,
                credentials,
                addressChannels,
                preferences,
                fingerprints,
            })));
        }));
        return automation_client_1.Success;
    });
}
exports.executeFingerprinting = executeFingerprinting;
//# sourceMappingURL=executeFingerprinting.js.map