"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const PushImpactListenerRegistration_1 = require("../../api/registration/PushImpactListenerRegistration");
const minimalClone_1 = require("../goal/minimalClone");
const createPushImpactListenerInvocation_1 = require("./createPushImpactListenerInvocation");
const relevantCodeActions_1 = require("./relevantCodeActions");
/**
 * Execute arbitrary code reactions against a codebase
 * @param {PushImpactListenerRegistration[]} registrations
 * @return {ExecuteGoal}
 */
function executePushImpact(registrations) {
    return (goalInvocation) => __awaiter(this, void 0, void 0, function* () {
        if (registrations.length === 0) {
            return automation_client_1.Success;
        }
        const { goalEvent, configuration, credentials, id, context } = goalInvocation;
        return configuration.sdm.projectLoader.doWithProject({
            credentials,
            id,
            context,
            readOnly: true,
            cloneOptions: minimalClone_1.minimalClone(goalEvent.push, { detachHead: true }),
        }, (project) => __awaiter(this, void 0, void 0, function* () {
            const cri = yield createPushImpactListenerInvocation_1.createPushImpactListenerInvocation(goalInvocation, project);
            const regs = registrations.map(PushImpactListenerRegistration_1.toPushReactionRegistration);
            const relevantCodeReactions = yield relevantCodeActions_1.relevantCodeActions(regs, cri);
            automation_client_1.logger.debug("Will invoke %d eligible code reactions of %d to %j: [%s] of [%s]", relevantCodeReactions.length, registrations.length, cri.id, relevantCodeReactions.map(a => a.name).join(), regs.map(a => a.name).join());
            const allReactions = yield Promise.all(relevantCodeReactions
                .map(reactionReg => reactionReg.action(cri)));
            const result = {
                code: allReactions.includes(PushImpactListenerRegistration_1.PushImpactResponse.failGoals) ? 1 : 0,
                requireApproval: allReactions.includes(PushImpactListenerRegistration_1.PushImpactResponse.requireApprovalToProceed),
            };
            automation_client_1.logger.debug("PushReaction responses are %j, result=%j", allReactions, result);
            return result;
        }));
    });
}
exports.executePushImpact = executePushImpact;
//# sourceMappingURL=executePushImpact.js.map