"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const _ = require("lodash");
const storeGoals_1 = require("../goal/storeGoals");
/**
 * Update the state of the SdmGoalSet as the goals progress
 * @param gcl
 */
exports.GoalSetGoalCompletionListener = (gcl) => __awaiter(void 0, void 0, void 0, function* () {
    const state = storeGoals_1.goalSetState(gcl.allGoals || []);
    automation_client_1.logger.debug(`GoalSet '${gcl.completedGoal.goalSetId}' now in state '${state}' because goal '${gcl.completedGoal.uniqueName}' was '${gcl.completedGoal.state}'`);
    const result = yield gcl.context.graphClient.query({
        name: "SdmGoalSetForId",
        variables: {
            goalSetId: [gcl.completedGoal.goalSetId],
        },
        options: automation_client_1.QueryNoCacheOptions,
    });
    const goalSet = _.get(result, "SdmGoalSet[0]");
    if (!!goalSet) {
        if (goalSet.state !== state) {
            const newGoalSet = Object.assign(Object.assign({}, goalSet), { state });
            yield storeGoals_1.storeGoalSet(gcl.context, newGoalSet);
        }
    }
});
//# sourceMappingURL=goalSetListener.js.map