"use strict";
/*
 * Copyright © 2020 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const format_1 = require("./format");
/**
 * Implementation of ProgressLog log that returns
 * an undefined link because it isn't actually persisted.
 * Used when we are not storing a local log.
 * Writes to logger at info level.
 */
class EphemeralProgressLog {
    constructor(name, writeToLog = true) {
        this.name = name;
        this.writeToLog = writeToLog;
        this.log = "";
        this.url = undefined;
    }
    isAvailable() {
        return __awaiter(this, void 0, void 0, function* () { return true; });
    }
    flush() {
        return Promise.resolve();
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.writeToLog) {
                automation_client_1.logger.info(`Progress log '${this.name}'\n${this.log.trim()}`);
            }
        });
    }
    write(what, ...args) {
        if (!what && what !== "") {
            return;
        }
        let line = format_1.format(what, ...args);
        if (!line.endsWith("\n")) {
            line += "\n";
        }
        this.log += line;
    }
}
exports.createEphemeralProgressLog = (context, sdmGoal) => __awaiter(void 0, void 0, void 0, function* () { return new EphemeralProgressLog(sdmGoal.uniqueName); });
//# sourceMappingURL=EphemeralProgressLog.js.map