"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const addressChannels_1 = require("../../api/context/addressChannels");
const format_1 = require("./format");
/**
 * Stream the ProgressLog output to any channels associated
 * with the current model element (such a repo) in Slack or elswhere.
 * @param name name for the log. Should relate to the activity we're logging
 * @param {HasChannels} hasChannels
 * @param {HandlerContext} ctx
 * @return {ProgressLog}
 */
function addressChannelsProgressLog(name, hasChannels, ctx) {
    const add = addressChannels_1.addressChannelsFor(hasChannels, ctx);
    return {
        name,
        isAvailable: () => __awaiter(this, void 0, void 0, function* () { return !!hasChannels.channels && hasChannels.channels.length > 0; }),
        write(msg, ...args) {
            return __awaiter(this, void 0, void 0, function* () {
                yield add(format_1.format(msg, ...args));
            });
        },
        flush() { return Promise.resolve(); },
        close() { return Promise.resolve(); },
    };
}
exports.addressChannelsProgressLog = addressChannelsProgressLog;
//# sourceMappingURL=addressChannelsProgressLog.js.map