"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:deprecation
// tslint:disable:max-file-line-count
const automation_client_1 = require("@atomist/automation-client");
const decoratorSupport_1 = require("@atomist/automation-client/lib/internal/metadata/decoratorSupport");
const onEvent_1 = require("@atomist/automation-client/lib/onEvent");
const repoFilter_1 = require("@atomist/automation-client/lib/operations/common/repoFilter");
const repoUtils_1 = require("@atomist/automation-client/lib/operations/common/repoUtils");
const editAll_1 = require("@atomist/automation-client/lib/operations/edit/editAll");
const projectEditor_1 = require("@atomist/automation-client/lib/operations/edit/projectEditor");
const projectEditorOps_1 = require("@atomist/automation-client/lib/operations/edit/projectEditorOps");
const GitHubRepoCreationParameters_1 = require("@atomist/automation-client/lib/operations/generate/GitHubRepoCreationParameters");
const Project_1 = require("@atomist/automation-client/lib/project/Project");
const constructionUtils_1 = require("@atomist/automation-client/lib/util/constructionUtils");
const slack_messages_1 = require("@atomist/slack-messages");
const _ = require("lodash");
const GitHubRepoTargets_1 = require("../../api/command/target/GitHubRepoTargets");
const TransformModeSuggestion_1 = require("../../api/command/target/TransformModeSuggestion");
const parameterPrompt_1 = require("../../api/context/parameterPrompt");
const preferenceStore_1 = require("../../api/context/preferenceStore");
const RepoTargets_1 = require("../../api/machine/RepoTargets");
const ParametersBuilder_1 = require("../../api/registration/ParametersBuilder");
const ParametersDefinition_1 = require("../../api/registration/ParametersDefinition");
const createCommand_1 = require("../command/createCommand");
const generatorCommand_1 = require("../command/generator/generatorCommand");
const chattyDryRunAwareEditor_1 = require("../command/transform/chattyDryRunAwareEditor");
const LoggingProgressLog_1 = require("../log/LoggingProgressLog");
const dateFormat_1 = require("../misc/dateFormat");
const createJob_1 = require("../misc/job/createJob");
const messages_1 = require("../misc/slack/messages");
const projectLoaderRepoLoader_1 = require("./projectLoaderRepoLoader");
const RepoTargetingParameters_1 = require("./RepoTargetingParameters");
const toMachineOptions_1 = require("./toMachineOptions");
exports.GeneratorTag = "generator";
exports.InspectionTag = "inspection";
exports.TransformTag = "transform";
function codeTransformRegistrationToCommand(sdm, ctr) {
    tagWith(ctr, exports.TransformTag);
    const mo = toMachineOptions_1.toMachineOptions(sdm);
    addDryRunParameters(ctr);
    addJobParameters(ctr);
    addParametersDefinedInBuilder(ctr);
    ctr.paramsMaker = toRepoTargetingParametersMaker(ctr.paramsMaker || automation_client_1.NoParameters, ctr.targets || mo.targets || GitHubRepoTargets_1.GitHubRepoTargets);
    const description = ctr.description || ctr.name;
    const asCommand = Object.assign(Object.assign({ description }, ctr), { listener: (ci) => __awaiter(this, void 0, void 0, function* () {
            ci.credentials = yield resolveCredentialsPromise(ci.credentials);
            const targets = ci.parameters.targets;
            const vr = targets.bindAndValidate();
            if (RepoTargets_1.isValidationError(vr)) {
                return ci.addressChannels(messages_1.slackErrorMessage(`Code Transform`, `Invalid parameters to code transform ${slack_messages_1.italic(ci.commandName)}:

${slack_messages_1.codeBlock(vr.message)}`, ci.context));
            }
            const repoFinder = !!ci.parameters.targets.repoRef ?
                () => Promise.resolve([ci.parameters.targets.repoRef]) :
                ctr.repoFinder || toMachineOptions_1.toMachineOptions(sdm).repoFinder;
            const repoLoader = !!ctr.repoLoader ?
                ctr.repoLoader(ci.parameters) :
                projectLoaderRepoLoader_1.projectLoaderRepoLoader(mo.projectLoader, ci.credentials, false, ci.context);
            const concurrency = Object.assign({ maxConcurrent: 2, requiresJob: false }, (ctr.concurrency || {}));
            try {
                const ids = yield repoUtils_1.relevantRepos(ci.context, repoFinder, repoFilter_1.andFilter(targets.test, ctr.repoFilter));
                const requiresJob = _.get(ci.parameters, "job.required", concurrency.requiresJob);
                if (ids.length > 1 || !!requiresJob) {
                    const params = Object.assign({}, ci.parameters);
                    params.targets.repos = undefined;
                    params.targets.repo = undefined;
                    delete params["job.name"];
                    delete params["job.description"];
                    delete params["job.required"];
                    yield createJob_1.createJob({
                        name: _.get(ci.parameters, "job.name") || `CodeTransform/${ci.commandName}`,
                        command: ci.commandName,
                        parameters: ids.map(id => (Object.assign(Object.assign({}, params), { "targets": {
                                owner: id.owner,
                                repo: id.repo,
                                branch: id.branch,
                                sha: id.sha,
                            }, "job.required": false }))),
                        description: _.get(ci.parameters, "job.description")
                            || `Running code transform ${slack_messages_1.italic(ci.commandName)} on ${ids.length} ${ids.length === 1 ? "repository" : "repositories"}`,
                        concurrentTasks: concurrency.maxConcurrent,
                    }, ci.context);
                }
                else {
                    const editMode = toEditModeOrFactory(ctr, ci);
                    const result = yield editAll_1.editOne(ci.context, ci.credentials, chattyDryRunAwareEditor_1.chattyDryRunAwareEditor(ctr, toScalarProjectEditor(ctr.transform, toMachineOptions_1.toMachineOptions(sdm), ctr.projectTest)), editMode, ids[0], ci.parameters, repoLoader);
                    if (!!ctr.onTransformResults) {
                        yield ctr.onTransformResults([result], Object.assign(Object.assign({}, ci), { progressLog: new LoggingProgressLog_1.LoggingProgressLog(ctr.name, "debug") }));
                    }
                    else if (!!result && !!result.error) {
                        const error = result.error;
                        return ci.addressChannels(messages_1.slackErrorMessage(`Code Transform`, `Code transform ${slack_messages_1.italic(ci.commandName)} failed${!!error.message ? ":\n\n" + slack_messages_1.codeBlock(error.message) : "."}`, ci.context));
                    }
                    else {
                        automation_client_1.logger.debug("No react function to react to result of code transformation '%s'", ctr.name);
                    }
                }
            }
            catch (e) {
                return ci.addressChannels(messages_1.slackErrorMessage(`Code Transform`, `Code transform ${slack_messages_1.italic(ci.commandName)} failed${!!e.message ? ":\n\n" + slack_messages_1.codeBlock(e.message) : "."}`, ci.context));
            }
        }) });
    return commandHandlerRegistrationToCommand(sdm, asCommand);
}
exports.codeTransformRegistrationToCommand = codeTransformRegistrationToCommand;
function codeInspectionRegistrationToCommand(sdm, cir) {
    tagWith(cir, exports.InspectionTag);
    const mo = toMachineOptions_1.toMachineOptions(sdm);
    addJobParameters(cir);
    addParametersDefinedInBuilder(cir);
    cir.paramsMaker = toRepoTargetingParametersMaker(cir.paramsMaker || automation_client_1.NoParameters, cir.targets || mo.targets || GitHubRepoTargets_1.GitHubRepoTargets);
    const description = cir.description || cir.name;
    const asCommand = Object.assign(Object.assign({ description }, cir), { listener: (ci) => __awaiter(this, void 0, void 0, function* () {
            ci.credentials = yield resolveCredentialsPromise(ci.credentials);
            const targets = ci.parameters.targets;
            const vr = targets.bindAndValidate();
            if (RepoTargets_1.isValidationError(vr)) {
                return ci.addressChannels(messages_1.slackErrorMessage(`Code Inspection`, `Invalid parameters to code inspection ${slack_messages_1.italic(ci.commandName)}:

${slack_messages_1.codeBlock(vr.message)}`, ci.context));
            }
            const action = (p) => __awaiter(this, void 0, void 0, function* () {
                if (!!cir.projectTest && !(yield cir.projectTest(p))) {
                    return { repoId: p.id, result: undefined };
                }
                return {
                    repoId: p.id,
                    result: yield cir.inspection(p, Object.assign(Object.assign({}, ci), { progressLog: new LoggingProgressLog_1.LoggingProgressLog(cir.name, "debug") })),
                };
            });
            const repoFinder = !!ci.parameters.targets.repoRef ?
                () => Promise.resolve([ci.parameters.targets.repoRef]) :
                cir.repoFinder || toMachineOptions_1.toMachineOptions(sdm).repoFinder;
            const repoLoader = !!cir.repoLoader ?
                cir.repoLoader(ci.parameters) :
                projectLoaderRepoLoader_1.projectLoaderRepoLoader(mo.projectLoader, ci.parameters.targets.credentials, true, ci.context);
            const concurrency = Object.assign({ maxConcurrent: 2, requiresJob: false }, (cir.concurrency || {}));
            try {
                const ids = yield repoUtils_1.relevantRepos(ci.context, repoFinder, repoFilter_1.andFilter(targets.test, cir.repoFilter));
                const requiresJob = _.get(ci.parameters, "job.required", concurrency.requiresJob);
                if (ids.length > 1 || !!requiresJob) {
                    const params = Object.assign({}, ci.parameters);
                    params.targets.repos = undefined;
                    params.targets.repo = undefined;
                    delete params["job.name"];
                    delete params["job.description"];
                    delete params["job.required"];
                    yield createJob_1.createJob({
                        name: `CodeInspection/${ci.commandName}`,
                        command: ci.commandName,
                        parameters: ids.map(id => (Object.assign(Object.assign({}, params), { "targets": {
                                owner: id.owner,
                                repo: id.repo,
                                branch: id.branch,
                                sha: id.sha,
                            }, "job.required": false }))),
                        description: `Running code inspection ${slack_messages_1.italic(ci.commandName)} on ${ids.length} repositories`,
                    }, ci.context);
                }
                else {
                    const project = yield repoLoader(ids[0]);
                    const result = yield action(project, ci.parameters);
                    if (!!cir.onInspectionResults) {
                        yield cir.onInspectionResults([result], ci);
                    }
                    else {
                        automation_client_1.logger.debug("No react function to react to results of code inspection '%s'", cir.name);
                    }
                }
            }
            catch (e) {
                return ci.addressChannels(messages_1.slackErrorMessage(`Code Inspection`, `Code Inspection ${slack_messages_1.italic(ci.commandName)} failed:

${slack_messages_1.codeBlock(e.message)}`, ci.context));
            }
        }) });
    return commandHandlerRegistrationToCommand(sdm, asCommand);
}
exports.codeInspectionRegistrationToCommand = codeInspectionRegistrationToCommand;
/**
 * Tag the command details with the given tag if it isn't already
 * @param {Partial<CommandDetails>} e
 * @param {string} tag
 */
function tagWith(e, tag) {
    if (!e.tags) {
        e.tags = [];
    }
    if (typeof e.tags === "string") {
        e.tags = [e.tags];
    }
    if (!e.tags.includes(tag)) {
        e.tags.push(tag);
    }
}
function generatorRegistrationToCommand(sdm, e) {
    tagWith(e, exports.GeneratorTag);
    if (!e.paramsMaker) {
        e.paramsMaker = automation_client_1.NoParameters;
    }
    if (e.startingPoint && Project_1.isProject(e.startingPoint) && !e.startingPoint.id) {
        // TODO should probably be handled in automation-client
        e.startingPoint.id = new automation_client_1.GitHubRepoRef("ignore", "this");
    }
    addParametersDefinedInBuilder(e);
    return () => generatorCommand_1.generatorCommand(sdm, () => toScalarProjectEditor(e.transform, toMachineOptions_1.toMachineOptions(sdm)), e.name, e.paramsMaker, e.fallbackTarget || GitHubRepoCreationParameters_1.GitHubRepoCreationParameters, e.startingPoint, e, // required because we redefine the afterAction
    e);
}
exports.generatorRegistrationToCommand = generatorRegistrationToCommand;
function commandHandlerRegistrationToCommand(sdm, c) {
    return () => createCommand_1.createCommand(sdm, toOnCommand(c), c.name, c.paramsMaker, c);
}
exports.commandHandlerRegistrationToCommand = commandHandlerRegistrationToCommand;
function eventHandlerRegistrationToEvent(sdm, e) {
    addParametersDefinedInBuilder(e);
    return () => onEvent_1.eventHandlerFrom(e.listener, e.paramsMaker || automation_client_1.NoParameters, e.subscription, e.name, e.description, e.tags);
}
exports.eventHandlerRegistrationToEvent = eventHandlerRegistrationToEvent;
class CommandListenerExecutionInterruptError extends Error {
    constructor(message) {
        super(message);
        this.message = message;
    }
}
exports.CommandListenerExecutionInterruptError = CommandListenerExecutionInterruptError;
function toOnCommand(c) {
    addParametersDefinedInBuilder(c);
    return sdm => (context, parameters) => __awaiter(this, void 0, void 0, function* () {
        const cli = toCommandListenerInvocation(c, context, parameters, toMachineOptions_1.toMachineOptions(sdm));
        cli.credentials = yield resolveCredentialsPromise(cli.credentials);
        try {
            yield c.listener(cli);
            return automation_client_1.Success;
        }
        catch (err) {
            if (err instanceof CommandListenerExecutionInterruptError) {
                return automation_client_1.Success;
            }
            else {
                automation_client_1.logger.error("Error executing command '%s': %s", cli.commandName, err.message);
                automation_client_1.logger.error(err.stack);
                return {
                    code: 1,
                    message: err.message,
                };
            }
        }
    });
}
function toCommandListenerInvocation(c, context, parameters, sdm) {
    // It may already be there
    let credentials = !!context ? context.credentials : undefined;
    let ids;
    if (generatorCommand_1.isSeedDrivenGeneratorParameters(parameters)) {
        credentials = parameters.target.credentials;
        ids = [parameters.target.repoRef];
    }
    else if (RepoTargetingParameters_1.isRepoTargetingParameters(parameters)) {
        credentials = parameters.targets.credentials;
        ids = !!parameters.targets.repoRef ? [parameters.targets.repoRef] : undefined;
    }
    if (!!sdm.credentialsResolver) {
        try {
            credentials = sdm.credentialsResolver.commandHandlerCredentials(context, ids ? ids[0] : undefined);
        }
        catch (e) {
            automation_client_1.logger.debug(`Failed to obtain credentials from credentialsResolver: ${e.message}`);
        }
    }
    let matches;
    if (c.intent instanceof RegExp) {
        matches = c.intent.exec(context.trigger.raw_message);
    }
    const addressChannels = (msg, opts) => context.messageClient.respond(msg, opts);
    const promptFor = sdm.parameterPromptFactory ? sdm.parameterPromptFactory(context) : parameterPrompt_1.NoParameterPrompt;
    const preferences = sdm.preferenceStoreFactory ? sdm.preferenceStoreFactory(context) : preferenceStore_1.NoPreferenceStore;
    const configuration = (context || {}).configuration;
    return {
        commandName: c.name,
        context,
        parameters,
        addressChannels,
        configuration,
        promptFor,
        preferences,
        credentials,
        ids,
        matches,
    };
}
exports.toCommandListenerInvocation = toCommandListenerInvocation;
exports.DryRunParameter = {
    name: "dry-run",
    description: "Run Code Transform in dry run mode so that changes aren't committed to the repository",
    required: false,
    defaultValue: false,
    type: "boolean",
};
exports.MsgIdParameter = {
    name: "msgId",
    description: "Id of the code transform message",
    required: false,
    type: "string",
    displayable: false,
};
/**
 * Add the dryRun parameter into the list of parameters
 */
function addDryRunParameters(c) {
    const params = toParametersListing(c.parameters || {});
    params.parameters.push(exports.DryRunParameter, exports.MsgIdParameter);
    c.parameters = params;
}
exports.JobNameParameter = {
    name: "job.name",
    description: "Name of the job to create",
    required: false,
    type: "string",
    displayable: false,
};
exports.JobDescriptionParameter = {
    name: "job.description",
    description: "Description of the job to create",
    required: false,
    type: "string",
    displayable: false,
};
exports.JobRequiredParameter = {
    name: "job.required",
    description: "Is job required",
    required: false,
    type: "boolean",
};
/**
 * Add the job parameter into the list of parameters
 */
function addJobParameters(c) {
    const params = toParametersListing(c.parameters || {});
    params.parameters.push(exports.JobNameParameter, exports.JobDescriptionParameter, exports.JobRequiredParameter);
    c.parameters = params;
}
/**
 * Add to the existing ParametersMaker any parameters defined in the builder itself
 * @param {CommandHandlerRegistration<PARAMS>} c
 */
function addParametersDefinedInBuilder(c) {
    const oldMaker = c.paramsMaker || automation_client_1.NoParameters;
    if (!!c.parameters) {
        c.paramsMaker = () => {
            const paramsInstance = constructionUtils_1.toFactory(oldMaker)();
            const paramListing = toParametersListing(c.parameters);
            paramListing.parameters.forEach(p => {
                paramsInstance[p.name] = p.defaultValue;
                decoratorSupport_1.declareParameter(paramsInstance, p.name, p);
            });
            paramListing.mappedParameters.forEach(mp => decoratorSupport_1.declareMappedParameter(paramsInstance, mp.name, mp.uri, mp.required));
            paramListing.secrets.forEach(s => decoratorSupport_1.declareSecret(paramsInstance, s.name, s.uri));
            paramListing.values.forEach(v => decoratorSupport_1.declareValue(paramsInstance, v.name, { path: v.path, required: v.required, type: v.type }));
            return paramsInstance;
        };
    }
}
function isMappedParameterOrSecretDeclaration(x) {
    const maybe = x;
    return !!maybe && !!maybe.declarationType;
}
function isValueDeclaration(x) {
    const maybe = x;
    return !!maybe && maybe.path !== undefined && maybe.path !== null;
}
function isParametersListing(p) {
    const maybe = p;
    return maybe.parameters !== undefined && maybe.mappedParameters !== undefined;
}
function toParametersListing(p) {
    if (isParametersListing(p)) {
        return p;
    }
    const builder = new ParametersBuilder_1.ParametersBuilder();
    for (const name of Object.getOwnPropertyNames(p)) {
        const value = p[name];
        if (isMappedParameterOrSecretDeclaration(value)) {
            switch (value.declarationType) {
                // tslint:disable-next-line:deprecation
                case ParametersDefinition_1.DeclarationType.mapped:
                case ParametersDefinition_1.DeclarationType.Mapped:
                    builder.addMappedParameters({ name, uri: value.uri, required: value.required });
                    break;
                // tslint:disable-next-line:deprecation
                case ParametersDefinition_1.DeclarationType.secret:
                case ParametersDefinition_1.DeclarationType.Secret:
                    builder.addSecrets({ name, uri: value.uri });
                    break;
            }
        }
        else if (isValueDeclaration(value)) {
            builder.addValues({ name, path: value.path, required: value.required, type: value.type });
        }
        else {
            builder.addParameters(Object.assign({ name }, (value)));
        }
    }
    return builder;
}
exports.toParametersListing = toParametersListing;
/**
 * Convert to legacy automation-client "editor" signature
 * @param {CodeTransformOrTransforms<PARAMS>} ctot
 * @param {ProjectPredicate} projectPredicate
 * @return {ProjectEditor<PARAMS>}
 */
function toScalarProjectEditor(ctot, sdm, projectPredicate) {
    const unguarded = Array.isArray(ctot) ?
        projectEditorOps_1.chainEditors(...ctot.map(c => toProjectEditor(c, sdm))) :
        toProjectEditor(ctot, sdm);
    if (!!projectPredicate) {
        // Filter out this project if it doesn't match the predicate
        return (p, context, params) => __awaiter(this, void 0, void 0, function* () {
            return (yield projectPredicate(p)) ?
                unguarded(p, context, params) :
                Promise.resolve({ success: true, edited: false, target: p });
        });
    }
    return unguarded;
}
exports.toScalarProjectEditor = toScalarProjectEditor;
// Convert to an old style, automation-client, ProjectEditor to allow
// underlying code to work for now
function toProjectEditor(ct, sdm) {
    return (p, ctx, params) => __awaiter(this, void 0, void 0, function* () {
        const ci = toCommandListenerInvocation(p, ctx, params, toMachineOptions_1.toMachineOptions(sdm));
        ci.credentials = yield resolveCredentialsPromise(ci.credentials);
        // Mix in handler context for old style callers
        const n = yield ct(p, Object.assign(Object.assign({}, ctx), ci), params);
        if (n === undefined) {
            // The transform returned void
            return { target: p, edited: yield isDirty(p), success: true };
        }
        const r = n;
        try {
            return Project_1.isProject(r) ? projectEditor_1.successfulEdit(r, yield isDirty(r)) : r;
        }
        catch (e) {
            return projectEditor_1.failedEdit(p, e);
        }
    });
}
function isDirty(p) {
    return __awaiter(this, void 0, void 0, function* () {
        if (isGitProject(p)) {
            try {
                const status = yield p.gitStatus();
                return !status.isClean;
            }
            catch (_a) {
                // Ignore
            }
        }
        return undefined;
    });
}
function isGitProject(p) {
    const maybe = p;
    return !!maybe.gitStatus;
}
/**
 * Return a parameters maker that is targeting aware
 * @param {Maker<PARAMS>} paramsMaker
 * @param targets targets parameters to set if necessary
 * @return {Maker<EditorOrReviewerParameters & PARAMS>}
 */
function toRepoTargetingParametersMaker(paramsMaker, targets) {
    const sampleParams = constructionUtils_1.toFactory(paramsMaker)();
    return RepoTargetingParameters_1.isRepoTargetingParameters(sampleParams) ?
        paramsMaker :
        () => {
            const rawParms = constructionUtils_1.toFactory(paramsMaker)();
            const allParms = rawParms;
            const targetsInstance = constructionUtils_1.toFactory(targets)();
            allParms.targets = targetsInstance;
            return allParms;
        };
}
exports.toRepoTargetingParametersMaker = toRepoTargetingParametersMaker;
function toEditModeOrFactory(ctr, ci) {
    const description = ctr.description || ctr.name;
    if (!!ctr.transformPresentation) {
        return (p) => ctr.transformPresentation(Object.assign(Object.assign({}, ci), { progressLog: new LoggingProgressLog_1.LoggingProgressLog(ctr.name, "debug") }), p);
    }
    // Get EditMode from parameters if possible
    if (TransformModeSuggestion_1.isTransformModeSuggestion(ci.parameters)) {
        const tms = ci.parameters;
        return new automation_client_1.editModes.PullRequest(tms.desiredBranchName, tms.desiredPullRequestTitle || description);
    }
    // Default it if not supplied
    return new automation_client_1.editModes.PullRequest(`transform-${gitBranchCompatible(ctr.name)}-${dateFormat_1.formatDate()}`, description);
}
/**
 * Takes a potential git branch name and returns a legalised iteration of it
 *
 * @param name the git branch name to sanitise.
 */
function gitBranchCompatible(name) {
    // handles spaces and .. ~ : ^ ? * [ @{
    let branchName = name.replace(/\s+|(\.\.)+|~+|:+|\^+|\?+|\*+|\[+|(\@\{)+/g, "_");
    // handles double slashes
    branchName = branchName.replace(/(\/\/)+|(\\)+/g, "/");
    // handles back slashes
    branchName = branchName.replace(/\\+/g, "/");
    if (branchName.startsWith(".") || branchName.startsWith("/")) {
        branchName = branchName.substring(1);
    }
    if (branchName.endsWith(".") || branchName.endsWith("/")) {
        branchName = branchName.slice(0, -1);
    }
    const lock = ".lock";
    if (branchName.endsWith(lock)) {
        branchName = branchName.slice(0, -lock.length);
    }
    if (branchName === "@") {
        branchName = "at";
    }
    // handles ascii control characters
    branchName = branchName.replace(/[\x00-\x1F\x7F]+/g, "");
    return branchName;
}
exports.gitBranchCompatible = gitBranchCompatible;
function resolveCredentialsPromise(creds) {
    return __awaiter(this, void 0, void 0, function* () {
        if (creds instanceof Promise) {
            try {
                return yield creds;
            }
            catch (e) {
                automation_client_1.logger.debug(e.message);
            }
        }
        else if (!!creds) {
            return creds;
        }
        return undefined;
    });
}
exports.resolveCredentialsPromise = resolveCredentialsPromise;
//# sourceMappingURL=handlerRegistrations.js.map