"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const findUp = require("find-up");
const path = require("path");
const trace = require("stack-trace");
/**
 * Read ExtensionPackMetadata from the modules package.json.
 * @param {string} name
 * @returns {ExtensionPackMetadata}
 */
function metadata(name) {
    const pathToCallingFunction = trace.get()[1].getFileName();
    const pj = require(findUp.sync("package.json", { cwd: path.resolve(path.dirname(pathToCallingFunction)) }));
    return {
        name: name ? `${pj.name}:${name.toLocaleLowerCase()}` : pj.name,
        vendor: pj.author && pj.author.name ? pj.author.name : pj.author,
        version: pj.version ? pj.version : "",
        tags: pj.keywords ? pj.keywords : [],
    };
}
exports.metadata = metadata;
//# sourceMappingURL=extensionPack.js.map