"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Return an editor to append the given content to the end of the file at the specified path,
 * creating the file with only this content if it doesn't exist.
 * Adds no whitespace.
 * @param command command
 * @return {SimpleProjectEditor}
 */
function appendOrCreateFileContent(command) {
    const commandToUse = Object.assign({ leaveAlone: content => content.includes(command.toAppend) }, command);
    return (p) => __awaiter(this, void 0, void 0, function* () {
        const target = yield p.getFile(commandToUse.path);
        if (!!target) {
            const oldContent = yield target.getContent();
            if (!commandToUse.leaveAlone(oldContent)) {
                yield target.setContent(oldContent + commandToUse.toAppend);
            }
        }
        else {
            yield p.addFile(commandToUse.path, commandToUse.toAppend);
        }
        return p;
    });
}
exports.appendOrCreateFileContent = appendOrCreateFileContent;
//# sourceMappingURL=appendOrCreate.js.map