"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const GitCommandGitProject_1 = require("@atomist/automation-client/lib/project/git/GitCommandGitProject");
const fs = require("fs-extra");
/**
 * Non caching ProjectLoader that uses a separate clone for each project accessed
 */
exports.CloningProjectLoader = {
    doWithProject(coords, action) {
        return __awaiter(this, void 0, void 0, function* () {
            // coords.depth is deprecated; populate it for backwards compatibility
            // tslint:disable-next-line:deprecation
            const cloneOptions = coords.cloneOptions ? coords.cloneOptions : { depth: coords.depth };
            // If we get a cloneDir we need to wrap the DirectoryManager to return the directory
            const directoryManager = !!coords.cloneDir ? new ExplicitDirectoryManager(coords.cloneDir) : GitCommandGitProject_1.DefaultDirectoryManager;
            const p = yield automation_client_1.GitCommandGitProject.cloned(coords.credentials, coords.id, cloneOptions, directoryManager);
            if (p.id.sha === "HEAD") {
                const gs = yield p.gitStatus();
                p.id.sha = gs.sha;
            }
            if (!!coords.readOnly) {
                p.shouldCache = true;
            }
            else {
                p.shouldCache = false;
            }
            return action(p);
        });
    },
};
class ExplicitDirectoryManager {
    constructor(cloneDir) {
        this.cloneDir = cloneDir;
    }
    directoryFor(owner, repo, branch, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            yield fs.emptyDir(this.cloneDir);
            return {
                path: this.cloneDir,
                release: () => this.cleanup(this.cloneDir, opts.keep),
                invalidate: () => Promise.resolve(),
                transient: opts.keep === false,
                provenance: "explicit-directory",
                type: "empty-directory",
            };
        });
    }
    cleanup(p, keep) {
        return __awaiter(this, void 0, void 0, function* () {
            if (keep) {
                return;
            }
            try {
                yield fs.remove(p);
            }
            catch (e) {
                automation_client_1.logger.warn(`Failed to remove '${p}': ${e.message}`);
            }
        });
    }
}
//# sourceMappingURL=cloningProjectLoader.js.map