"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const yaml = require("js-yaml");
const _ = require("lodash");
/**
 * Gather a configuration value from:
 *
 * 1) within a file .atomist/config.json or .atomist/config.yaml within the Project
 * 2) from within the `sdm` property in the wider, globally available configuration. For where these come from, see:
 *  https://atomist.github.io/automation-client/modules/_lib_configuration_.html#loadconfiguration
 * 3) the default passed in, if any
 * 4) ... or else throw.
 *
 * This will not merge the configuration objects. It gives the first one that is not undefined.
 */
function projectConfigurationValue(path, p, defaultValue) {
    return __awaiter(this, void 0, void 0, function* () {
        // Project specific configuration first
        let cf = yield p.getFile(".atomist/config.json");
        if (!!cf) {
            try {
                const conf = JSON.parse(yield cf.getContent());
                const value = _.get(conf, path);
                if (value !== undefined) {
                    return value;
                }
            }
            catch (e) {
                automation_client_1.logger.warn(`Failed to load/parse '.atomist/config.json' from '${p.id.owner}/${p.id.repo}': ${e.message}`);
            }
        }
        cf = yield p.getFile(".atomist/config.yaml");
        if (!!cf) {
            try {
                const conf = yaml.safeLoad(yield cf.getContent());
                const value = _.get(conf, path);
                if (value !== undefined) {
                    return value;
                }
            }
            catch (e) {
                automation_client_1.logger.warn(`Failed to load/parse '.atomist/config.yaml' from '${p.id.owner}/${p.id.repo}': ${e.message}`);
            }
        }
        // SDM configuration as fallback
        const cfg = automation_client_1.configurationValue(`sdm.${path}`, defaultValue);
        if (!!cfg) {
            return cfg;
        }
        // Lastly use the defaultValue if provided
        if (defaultValue !== undefined) {
            return defaultValue;
        }
        throw new Error(`Required project configuration value '${path}' not available`);
    });
}
exports.projectConfigurationValue = projectConfigurationValue;
//# sourceMappingURL=projectConfiguration.js.map