"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
/**
 * Add the downloaded content to the given project
 * @param url url of the content. Must be publicly accessible
 * @param path
 */
function copyFileFromUrl(url, path) {
    return (p) => __awaiter(this, void 0, void 0, function* () {
        const http = automation_client_1.configurationValue("http.client.factory", automation_client_1.defaultHttpClientFactory());
        const response = yield http.create(url).exchange(url, { method: automation_client_1.HttpMethod.Get });
        return p.addFile(path, response.body);
    });
}
exports.copyFileFromUrl = copyFileFromUrl;
/**
 * Take the specified files from the donor project
 * @param {RemoteRepoRef} donorProjectId
 * @param {FileMapping[]} fileMappings
 * @param {ProjectOperationCredentials} credentials
 * @return {SimpleProjectEditor}
 */
function copyFilesFrom(donorProjectId, fileMappings, credentials) {
    return (p, i) => __awaiter(this, void 0, void 0, function* () {
        const donorProject = yield automation_client_1.GitCommandGitProject.cloned(credentials, donorProjectId);
        return copyFiles(donorProject, fileMappings)(p, i);
    });
}
exports.copyFilesFrom = copyFilesFrom;
function copyFiles(donorProject, fileMappings) {
    return (p) => __awaiter(this, void 0, void 0, function* () {
        for (const m of fileMappings) {
            const fm = typeof m === "string" ? { donorPath: m, recipientPath: m } : m;
            const found = yield donorProject.getFile(fm.donorPath);
            if (found) {
                yield p.addFile(fm.recipientPath, yield found.getContent());
            }
            else {
                automation_client_1.logger.debug("Path '%s' not found in donor project %s:%s", fm.donorPath, donorProject.id.owner, donorProject.id.repo);
            }
        }
        return p;
    });
}
exports.copyFiles = copyFiles;
/**
 * Take the specified files from the donor project
 * @param {RemoteRepoRef} donorProjectId
 * @param {FileGlobMapping} fileGlobMapping - treated as globs as defined in Project.streamFiles
 * @return {SimpleProjectEditor}
 */
function streamFilesFrom(donorProjectId, fileGlobMapping) {
    return (p, i) => __awaiter(this, void 0, void 0, function* () {
        const donorProject = yield automation_client_1.GitCommandGitProject.cloned(i.credentials, donorProjectId);
        return streamFiles(donorProject, fileGlobMapping)(p, i);
    });
}
exports.streamFilesFrom = streamFilesFrom;
function streamFiles(donorProject, fileGlobMapping) {
    return (p) => __awaiter(this, void 0, void 0, function* () {
        const fileStream = donorProject.streamFiles(...fileGlobMapping.globPatterns);
        yield new Promise((resolve, reject) => {
            fileStream
                .on("end", () => {
                automation_client_1.logger.debug("end of file stream reached, using glob: ", fileGlobMapping);
                resolve();
            })
                .on("data", donorFile => {
                const newPath = (fileGlobMapping.recipientPath || "") + donorFile.path;
                p.addFileSync(newPath, donorFile.getContentSync());
                automation_client_1.logger.log("silly", "file added: ", donorFile.path);
            })
                .on("error", e => {
                automation_client_1.logger.debug("Error copying file: ", e);
                reject(e);
            });
        });
    });
}
exports.streamFiles = streamFiles;
//# sourceMappingURL=fileCopy.js.map