"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
/**
 * Perform an action on all accessible repos in parallel up to a configurable max concurrency limit.
 * This is normally readonly.
 * If you want to perform an update, use a CodeTransform.
 * @param {SoftwareDeliveryMachine} sdm
 * @param {SdmContext} i
 * @param {WithLoadedProject<any>} action
 * @return {Promise<any>}
 */
function doWithRepos(sdm, i, action) {
    return __awaiter(this, void 0, void 0, function* () {
        const repos = yield sdm.configuration.sdm.repoFinder(i.context);
        automation_client_1.logger.debug("doWithRepos working on %d repos", repos.length);
        yield automation_client_1.executeAll(repos.map(id => () => {
            return sdm.configuration.sdm.projectLoader.doWithProject({ credentials: i.credentials, id: id, readOnly: true }, action)
                .catch(err => {
                automation_client_1.logger.debug("Project err: %s", err);
            });
        }));
    });
}
exports.doWithRepos = doWithRepos;
//# sourceMappingURL=withRepos.js.map