"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Convenient function to allow creating fake contexts.
 * Useful for testing
 * @param {string} workspaceId
 * @return {any}
 */
function fakeContext(workspaceId = "T123") {
    const correlationId = "foo";
    return {
        workspaceId,
        messageClient: new DevNullMessageClient(),
        graphClient: {
            mutate(optionsOrName) {
                return undefined;
            },
        },
        correlationId,
        context: {
            name: "test-context",
            workspaceId,
            workspaceName: workspaceId,
            operation: "operation",
            version: "0.1.0",
            invocationId: "inv-id",
            ts: new Date().getTime(),
            correlationId,
        },
        trigger: {},
    };
}
exports.fakeContext = fakeContext;
/**
 * Throw it all away
 */
class DevNullMessageClient {
    addressChannels(msg, channels, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return;
        });
    }
    addressUsers(msg, users, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return;
        });
    }
    respond(msg, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return;
        });
    }
    send(msg, destinations, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return;
        });
    }
}
//# sourceMappingURL=fakeContext.js.map