"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const LoggingProgressLog_1 = require("../../api-helper/log/LoggingProgressLog");
const preferenceStore_1 = require("../../api/context/preferenceStore");
const Goal_1 = require("../../api/goal/Goal");
const environment_1 = require("../../api/goal/support/environment");
const types_1 = require("../../typings/types");
const fakeContext_1 = require("./fakeContext");
/**
 * Useful testing support
 * @param {RemoteRepoRef} id
 * @return {GoalInvocation}
 */
function fakeGoalInvocation(id, options) {
    return {
        credentials: { token: "foobar" },
        context: fakeContext_1.fakeContext("T1111"),
        id,
        addressChannels: (m) => __awaiter(this, void 0, void 0, function* () {
            automation_client_1.logger.info("channels > " + m);
        }),
        preferences: preferenceStore_1.NoPreferenceStore,
        progressLog: new LoggingProgressLog_1.LoggingProgressLog("fake"),
        sdmGoal: fakeSdmGoal(id),
        goalEvent: fakeSdmGoal(id),
        goal: fakeGoal("fake goal"),
        configuration: {
            sdm: Object.assign({}, options),
        },
    };
}
exports.fakeGoalInvocation = fakeGoalInvocation;
function fakeSdmGoal(id) {
    return {
        uniqueName: "hi",
        name: "Hello",
        goalSet: "goal set",
        registration: "foo",
        goalSetId: "xuf",
        ts: 42,
        provenance: [],
        preConditions: [],
        environment: "0-code",
        fulfillment: {
            method: "other",
            registration: "@atomist/sdm",
            name: "something",
        },
        repo: {
            name: id.repo,
            owner: id.owner,
            providerId: "asdfdas",
        },
        sha: "abc",
        branch: "master",
        state: types_1.SdmGoalState.requested,
        push: {
            repo: {
                org: {
                    owner: id.owner,
                    provider: {
                        providerId: "skdfjasd",
                        apiUrl: "https://api.github.com",
                    },
                },
                name: id.repo,
                channels: [{
                        name: "foo",
                        id: "1",
                        team: {
                            id: "T357",
                        },
                    }],
            },
            commits: [{ sha: automation_client_1.guid() }],
        },
    };
}
function fakeGoal(name) {
    return new Goal_1.Goal({
        uniqueName: name,
        displayName: name,
        environment: environment_1.IndependentOfEnvironment,
    });
}
//# sourceMappingURL=fakeGoalInvocation.js.map