"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const TargetsParams_1 = require("@atomist/automation-client/lib/operations/common/params/TargetsParams");
/**
 * Targets for working with BitBucket repo(s).
 * Allows use of regex.
 */
let BitBucketRepoTargets = class BitBucketRepoTargets extends TargetsParams_1.TargetsParams {
    constructor() {
        super();
        this.branch = "master";
    }
    get credentials() {
        return undefined;
    }
    /**
     * Return a single RepoRef or undefined if we're not identifying a single repo
     * @return {RepoRef}
     */
    get repoRef() {
        return (!!this.owner && !!this.repo && !this.usesRegex) ?
            new automation_client_1.BitBucketServerRepoRef(this.apiUrl, this.owner, this.repo, true, this.branch ? this.branch : this.sha) :
            undefined;
    }
    bindAndValidate() {
        if (!this.repo) {
            if (!this.repos) {
                return {
                    message: "If not executing in a mapped channel, must identify a repo via: `targets.owner` and `targets.repo`, " +
                        "or a repo name regex via `targets.repos`",
                };
            }
            this.repo = this.repos;
        }
    }
};
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubApiUrl, false),
    __metadata("design:type", String)
], BitBucketRepoTargets.prototype, "apiUrl", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubOwner, false),
    __metadata("design:type", String)
], BitBucketRepoTargets.prototype, "owner", void 0);
__decorate([
    automation_client_1.MappedParameter(automation_client_1.MappedParameters.GitHubRepository, false),
    __metadata("design:type", String)
], BitBucketRepoTargets.prototype, "repo", void 0);
__decorate([
    automation_client_1.Parameter(Object.assign(Object.assign({ description: "Ref" }, automation_client_1.validationPatterns.GitShaRegExp), { required: false })),
    __metadata("design:type", String)
], BitBucketRepoTargets.prototype, "sha", void 0);
__decorate([
    automation_client_1.Parameter(Object.assign(Object.assign({ description: "Branch Defaults to 'master'" }, automation_client_1.validationPatterns.GitBranchRegExp), { required: false })),
    __metadata("design:type", String)
], BitBucketRepoTargets.prototype, "branch", void 0);
__decorate([
    automation_client_1.Parameter({ description: "regex", required: false }),
    __metadata("design:type", String)
], BitBucketRepoTargets.prototype, "repos", void 0);
BitBucketRepoTargets = __decorate([
    automation_client_1.Parameters(),
    __metadata("design:paramtypes", [])
], BitBucketRepoTargets);
exports.BitBucketRepoTargets = BitBucketRepoTargets;
//# sourceMappingURL=BitBucketRepoTargets.js.map