"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const handlerRegistrations_1 = require("../../api-helper/machine/handlerRegistrations");
/**
 * No-op NoParameterPrompt implementation that never prompts for new parameters
 * @constructor
 */
exports.NoParameterPrompt = () => __awaiter(void 0, void 0, void 0, function* () { return ({}); });
exports.AtomistContinuationMimeType = "application/x-atomist-continuation+json";
/**
 * Default ParameterPromptFactory that uses the WebSocket connection to send parameter prompts to the backend.
 * @param ctx
 */
function commandRequestParameterPromptFactory(ctx) {
    return (parameters, options = {}) => __awaiter(this, void 0, void 0, function* () {
        const trigger = ctx.trigger;
        const existingParameters = trigger.parameters;
        const newParameters = _.cloneDeep(parameters);
        // Find out if - and if - which parameters are actually missing
        let requiredMissing = false;
        const params = {};
        for (const parameter in parameters) {
            if (parameters.hasOwnProperty(parameter)) {
                const existingParameter = existingParameters.find(p => p.name === parameter);
                if (!existingParameter) {
                    // If required isn't defined it means the parameter is required
                    if (newParameters[parameter].required || newParameters[parameter].required === undefined) {
                        requiredMissing = true;
                    }
                }
                else {
                    params[parameter] = existingParameter.value;
                    delete newParameters[parameter];
                }
            }
        }
        // If no parameters are missing we can return the already collected parameters
        if (!requiredMissing) {
            return params;
        }
        // Set up the thread_ts for this response message
        let threadTs;
        if (options.thread === true && !!trigger.source) {
            threadTs = _.get(trigger.source, "slack.message.ts");
        }
        else if (typeof options.thread === "string") {
            threadTs = options.thread;
        }
        const destination = _.cloneDeep(trigger.source);
        _.set(destination, "slack.thread_ts", threadTs);
        // Create a continuation message using the existing HandlerResponse and mixing in parameters
        // and parameter_specs
        const response = {
            api_version: "1",
            correlation_id: trigger.correlation_id,
            team: trigger.team,
            command: trigger.command,
            source: trigger.source,
            destinations: [destination],
            parameters: [...(trigger.parameters || []), ...(trigger.mapped_parameters || [])],
            question: !!options.parameterStyle ? options.parameterStyle.toString() : undefined,
            parameter_specs: _.map(newParameters, (v, k) => (Object.assign(Object.assign({}, v), { name: k, required: v.required !== undefined ? v.required : true, pattern: v.pattern ? v.pattern.source : undefined }))),
            content_type: exports.AtomistContinuationMimeType,
        };
        yield ctx.messageClient.respond(response);
        throw new handlerRegistrations_1.CommandListenerExecutionInterruptError(`Prompting for new parameters: ${_.map(newParameters, (v, k) => k).join(", ")}`);
    });
}
exports.commandRequestParameterPromptFactory = commandRequestParameterPromptFactory;
//# sourceMappingURL=parameterPrompt.js.map