"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const PredicateMapping_1 = require("../mapping/PredicateMapping");
const PredicateMappingTerm_1 = require("../mapping/support/PredicateMappingTerm");
/**
 * Predicate mapping DSL method. Allows use of booleans and functions
 * returning boolean in predicate expressions
 * @param {PushTest} pred1
 * @param {PushTest} preds
 */
function allOf(pred1, ...preds) {
    const asPredicateMappings = [PredicateMappingTerm_1.toPredicateMapping(pred1)].concat(preds.map(PredicateMappingTerm_1.toPredicateMapping));
    return {
        name: asPredicateMappings.map(c => c.name).join(" & "),
        mapping: (pu) => __awaiter(this, void 0, void 0, function* () {
            const result = yield Promise.all(asPredicateMappings.map(pm => pm.mapping(pu)));
            return !result.includes(false);
        }),
        structure: {
            components: asPredicateMappings,
            compositionStyle: PredicateMapping_1.PredicateMappingCompositionStyle.And,
        },
    };
}
exports.allOf = allOf;
//# sourceMappingURL=allOf.js.map