"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const executeAutoInspects_1 = require("../../../api-helper/listener/executeAutoInspects");
const logInterpreters_1 = require("../../../api-helper/log/logInterpreters");
const GoalNameGenerator_1 = require("../GoalNameGenerator");
const GoalWithFulfillment_1 = require("../GoalWithFulfillment");
const environment_1 = require("../support/environment");
const DefaultAutoCodeInspectionOptions = {
    reportToSlack: true,
};
/**
 * Goal that runs code inspections
 */
class AutoCodeInspection extends GoalWithFulfillment_1.FulfillableGoalWithRegistrationsAndListeners {
    constructor(details = {}, ...dependsOn) {
        super(Object.assign({}, GoalWithFulfillment_1.getGoalDefinitionFrom(details, GoalNameGenerator_1.DefaultGoalNameGenerator.generateName("code-inspection"), CodeInspectionDefinition)), ...dependsOn);
        const optsToUse = Object.assign({ reportToSlack: DefaultAutoCodeInspectionOptions.reportToSlack }, details);
        this.addFulfillment({
            name: `code-inspections-${this.definition.uniqueName}`,
            goalExecutor: executeAutoInspects_1.executeAutoInspects({
                reportToSlack: optsToUse.reportToSlack,
                cloneOptions: optsToUse.cloneOptions,
                registrations: this.registrations,
                listeners: this.listeners,
            }),
            logInterpreter: logInterpreters_1.LogSuppressor,
        });
    }
}
exports.AutoCodeInspection = AutoCodeInspection;
const CodeInspectionDefinition = {
    uniqueName: "code-inspection",
    displayName: "code inspection",
    environment: environment_1.IndependentOfEnvironment,
    workingDescription: "Running code inspections",
    completedDescription: "Code inspections passed",
};
//# sourceMappingURL=AutoCodeInspection.js.map