"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const executeFingerprinting_1 = require("../../../api-helper/listener/executeFingerprinting");
const GoalNameGenerator_1 = require("../GoalNameGenerator");
const GoalWithFulfillment_1 = require("../GoalWithFulfillment");
const environment_1 = require("../support/environment");
/**
 * Goal that performs fingerprinting. Typically invoked early in a delivery flow.
 * @deprecated use fingerprintSupport with a PushImpact goal
 */
class Fingerprint extends GoalWithFulfillment_1.FulfillableGoalWithRegistrationsAndListeners {
    constructor(goalDetailsOrUniqueName = GoalNameGenerator_1.DefaultGoalNameGenerator.generateName("fingerprint"), ...dependsOn) {
        super(Object.assign({}, GoalWithFulfillment_1.getGoalDefinitionFrom(goalDetailsOrUniqueName, GoalNameGenerator_1.DefaultGoalNameGenerator.generateName("fingerprint"), FingerprintDefinition)), ...dependsOn);
        this.addFulfillment({
            name: `fingerprint-${this.definition.uniqueName}`,
            goalExecutor: executeFingerprinting_1.executeFingerprinting(this.registrations, this.listeners),
        });
        this.listeners.push(exports.SendFingerprintToAtomist);
    }
}
exports.Fingerprint = Fingerprint;
const FingerprintDefinition = {
    uniqueName: "fingerprint",
    displayName: "fingerprint",
    environment: environment_1.IndependentOfEnvironment,
    workingDescription: "Running fingerprint calculations",
    completedDescription: "Fingerprinted",
};
/**
 * Publish the given fingerprint to Atomist in the given team
 * @return {Promise<void>}
 */
exports.SendFingerprintToAtomist = fli => {
    const baseUrl = process.env.ATOMIST_WEBHOOK_BASEURL || "https://webhook.atomist.com";
    const url = `${baseUrl}/atomist/fingerprints/teams/${fli.context.workspaceId}`;
    const payload = {
        commit: {
            provider: fli.id.providerType,
            owner: fli.id.owner,
            repo: fli.id.repo,
            sha: fli.id.sha,
        },
        fingerprints: fli.fingerprints,
    };
    try {
        automation_client_1.logger.debug("Sending fingerprint to %s: %j", url, JSON.stringify(payload));
    }
    catch (err) {
        automation_client_1.logger.error(`Unable to serialize fingerprint: %s`, err.message);
    }
    const http = fli.configuration.http.client.factory.create(url);
    return http.exchange(url, { method: automation_client_1.HttpMethod.Post, body: payload })
        .catch(err => {
        automation_client_1.logger.error(`Failed to send fingerprint: %s`, err.message);
        return Promise.reject(err);
    });
};
//# sourceMappingURL=Fingerprint.js.map