"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const executePushImpact_1 = require("../../../api-helper/listener/executePushImpact");
const GoalNameGenerator_1 = require("../GoalNameGenerator");
const GoalWithFulfillment_1 = require("../GoalWithFulfillment");
const environment_1 = require("../support/environment");
/**
 * Goal that invokes PushImpactListener instances. Typically invoked early in a delivery flow.
 */
class PushImpact extends GoalWithFulfillment_1.FulfillableGoalWithRegistrations {
    constructor(goalDetailsOrUniqueName = GoalNameGenerator_1.DefaultGoalNameGenerator.generateName("push-impact"), ...dependsOn) {
        super(Object.assign({}, GoalWithFulfillment_1.getGoalDefinitionFrom(goalDetailsOrUniqueName, GoalNameGenerator_1.DefaultGoalNameGenerator.generateName("push-impact"), PushImpactDefinition)), ...dependsOn);
        this.addFulfillment({
            name: `push-impact-${this.definition.uniqueName}`,
            goalExecutor: executePushImpact_1.executePushImpact(this.registrations),
        });
    }
    withListener(listener) {
        return this.with({
            name: GoalNameGenerator_1.DefaultGoalNameGenerator.generateName("push-impact-listener"),
            action: listener,
        });
    }
}
exports.PushImpact = PushImpact;
const PushImpactDefinition = {
    uniqueName: "push-impact",
    displayName: "push impact",
    environment: environment_1.IndependentOfEnvironment,
    workingDescription: "Running push impact analysis",
    completedDescription: "Completed push impact analysis",
};
//# sourceMappingURL=PushImpact.js.map