import { NoParameters } from "@atomist/automation-client";
import { CodeInspectionRegistration } from "../registration/CodeInspectionRegistration";
import { CodeTransformRegistration } from "../registration/CodeTransformRegistration";
import { CommandHandlerRegistration } from "../registration/CommandHandlerRegistration";
import { GeneratorRegistration } from "../registration/GeneratorRegistration";
/**
 * Manage command registrations.
 */
export interface CommandRegistrationManager {
    /**
     * Add a generic command to this machine
     * @return {this}
     */
    addCommand<PARAMS = NoParameters>(command: CommandHandlerRegistration<PARAMS>): this;
    /**
     * Add a generator to this machine to enable project creation
     * @return {this}
     */
    addGeneratorCommand<PARAMS = NoParameters>(generator: GeneratorRegistration<PARAMS>): this;
    /**
     * Add a code transformation to this machine.
     * @return {this}
     */
    addCodeTransformCommand<PARAMS = NoParameters>(ctr: CodeTransformRegistration<PARAMS>): this;
    /**
     * Add a code inspection to this machine.
     * Unlike code transformations, code inspections cannot mutate projects.
     * @return {this}
     */
    addCodeInspectionCommand<R, PARAMS = NoParameters>(cir: CodeInspectionRegistration<R, PARAMS>): this;
}
//# sourceMappingURL=CommandRegistrationManager.d.ts.map