import { GoalSetter } from "../mapping/GoalSetter";
import { ConfigurationValues } from "./ConfigurationValues";
import { MachineConfigurer } from "./MachineConfigurer";
export interface ExtensionPackMetadata {
    /**
     * Name of the extension pack
     */
    name: string;
    /**
     * Vendor or author of this extension pack
     */
    vendor: string;
    /**
     * Version of extension pack
     */
    version: string;
    /**
     * Optional tags of extension pack
     */
    tags?: string | string[];
}
/**
 * Primary unit of extensibility in SDMs.
 * Implemented to expose a capability that can be added to a
 * software delivery machine in a consistent manner.
 * Facilitates modularity at a higher level than FunctionUnit or handlers.
 * For example, a Node module can export an ExtensionPack.
 * ExtensionPacks can optional contribute goal setting, which will be added to existing goal setting.
 */
export interface ExtensionPack extends MachineConfigurer, ExtensionPackMetadata, ConfigurationValues {
    /**
     * Human-readable description of this extension pack
     */
    description?: string;
    /**
     * Optional goal setting contributions that will be added into SDM goal setting.
     * Decorates other goal setting behavior.
     */
    goalContributions?: GoalSetter;
}
//# sourceMappingURL=ExtensionPack.d.ts.map