"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const RequestProcessor_1 = require("@atomist/automation-client/lib/internal/transport/RequestProcessor");
const _ = require("lodash");
const commonPushTests_1 = require("./support/commonPushTests");
function isGoal(options = {}) {
    return goalTest(`is goal ${JSON.stringify(options)}`, (g) => __awaiter(this, void 0, void 0, function* () {
        if (!!options.name && !options.name.test(g.name)) {
            return false;
        }
        if (!!options.state && options.state !== g.state) {
            return false;
        }
        if (!!options.output) {
            const data = JSON.parse(g.data || "{}");
            const outputs = data["@atomist/sdm/output"];
            if (!outputs) {
                return false;
            }
            else if (!outputs.some(o => options.output.test(o.classifier))) {
                return false;
            }
        }
        if (!!options.data && !options.data.test(g.data)) {
            return false;
        }
        return true;
    }), options.pushTest);
}
exports.isGoal = isGoal;
function goalTest(name, goalMapping, pushTest = commonPushTests_1.AnyPush) {
    return {
        name,
        mapping: (pli) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const trigger = (_b = (_a = pli) === null || _a === void 0 ? void 0 : _a.context) === null || _b === void 0 ? void 0 : _b.trigger;
            if (!!trigger && RequestProcessor_1.isEventIncoming(trigger)) {
                const goal = _.get(trigger, "data.SdmGoal[0]");
                if (!!goal) {
                    const match = yield goalMapping(goal, pli);
                    if (!!match) {
                        if (!pli.project) {
                            return true;
                        }
                        else {
                            return pushTest.mapping(pli);
                        }
                    }
                }
            }
            return false;
        }),
        pushTest,
    };
}
exports.goalTest = goalTest;
/**
 * Wrap a PushTest to make sure it doesn't get the change to match on goal planning
 * based on goal events
 */
function notGoalTest(pushTest) {
    return Object.assign(Object.assign({}, pushTest), { mapping: (pli) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const trigger = (_b = (_a = pli) === null || _a === void 0 ? void 0 : _a.context) === null || _b === void 0 ? void 0 : _b.trigger;
            if (!!trigger && RequestProcessor_1.isEventIncoming(trigger)) {
                const goal = _.get(trigger, "data.SdmGoal[0]");
                if (!!goal) {
                    return false;
                }
            }
            return pushTest.mapping(pli);
        }) });
}
exports.notGoalTest = notGoalTest;
function wrapPredicateMapping(guards) {
    return wrapTest(guards);
}
exports.wrapPredicateMapping = wrapPredicateMapping;
function wrapTest(test) {
    if (!!test.pushTest) {
        return test;
    }
    else {
        return notGoalTest(test);
    }
}
exports.wrapTest = wrapTest;
//# sourceMappingURL=goalTest.js.map