import { PredicatePushTest, PushTest } from "../PushTest";
export declare const ToDefaultBranch: PushTest;
/**
 * Is this a push originated by Atomist? Note that we can't look at the committer,
 * as if a user invoked a command handler, their credentials will be used
 * @param {PushListenerInvocation} p
 * @return {boolean}
 * @constructor
 */
export declare const FromAtomist: PushTest;
/**
 * Match on any push
 * @param {PushListenerInvocation} p
 * @constructor
 */
export declare const AnyPush: PushTest;
/**
 * Return a PushTest testing for the existence of the given file
 * @param {string} path
 * @return {PushTest}
 */
export declare function hasFile(path: string): PredicatePushTest;
/**
 * PushTest that returns true if project is non empty
 * @type {PredicatePushTest}
 */
export declare const NonEmpty: PredicatePushTest;
/**
 * Is there at least one file with the given extension?
 * @param {string} extension
 * @return {PredicatePushTest}
 */
export declare function hasFileWithExtension(extension: string): PredicatePushTest;
/**
 * Is this push to a non-default branch that has an open pull request?
 */
export declare const IsPushToBranchWithPullRequest: PushTest;
/**
 * Return a push test that matches the repository owner/repo slug
 * against regular expression.
 * @param re Regular expression to match against using RegExp.test()
 * @return Push test performing the match
 */
export declare function isRepo(re: RegExp): PushTest;
/**
 * Return a push test that matches the repository branch
 * against regular expression.
 * @param re Regular expression to match against using RegExp.test()
 * @return Push test performing the match
 */
export declare function isBranch(re: RegExp): PushTest;
/**
 * Return a PushTest testing for the existence of the given file containing the pattern
 * @param {string} path
 * @param pattern regex to look for
 * @return {PushTest}
 */
export declare function hasFileContaining(pattern: string | string[], content?: RegExp): PushTest;
/**
 * Return a PushTest that checks if a certain resource provider exists in the graph
 */
export declare function hasResourceProvider(type: "docker" | "npm" | "maven2", name?: string): PushTest;
//# sourceMappingURL=commonPushTests.d.ts.map