import { NoParameters } from "@atomist/automation-client";
import { ParametersInvocation } from "../listener/ParametersInvocation";
import { CodeInspection } from "./CodeInspectionRegistration";
import { PushImpactResponse } from "./PushImpactListenerRegistration";
import { PushSelector } from "./PushRegistration";
/**
 * Register an automatic inspection.
 */
export interface AutoInspectRegistration<R, PARAMS = NoParameters> extends PushSelector {
    /**
     * Inspection function to run on each project
     */
    inspection: CodeInspection<R, PARAMS>;
    /**
     * Parameters used for all inspections
     */
    parametersInstance?: PARAMS;
    /**
     * Invoked after each inspection result, if provided.
     * A void return means keep processing this push. Return a
     * PushImpactResponse to demand approval or fail goals.
     * @param {R} result
     * @param {ParametersInvocation<PARAMS>} ci
     * @return {Promise<void>}
     */
    onInspectionResult?(result: R, ci: ParametersInvocation<PARAMS>): Promise<PushImpactResponse | void>;
}
//# sourceMappingURL=AutoInspectRegistration.d.ts.map