"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A code action response that affects delivery:
 * failing the current goal or requiring approval,
 * causing dependent goals to fail or wait.
 */
var PushImpactResponse;
(function (PushImpactResponse) {
    /**
     * Everything's good. Keep going.
     */
    PushImpactResponse["proceed"] = "proceed";
    /**
     * Fail execution of the present goalset. Any dependent goals will stop.
     * Will not stop execution of non-dependent goals.
     */
    PushImpactResponse["failGoals"] = "fail";
    /**
     * Require approval to proceed to dependent goals in the present goalset.
     */
    PushImpactResponse["requireApprovalToProceed"] = "requireApproval";
})(PushImpactResponse = exports.PushImpactResponse || (exports.PushImpactResponse = {}));
function isPushReactionRegistration(a) {
    const maybe = a;
    return !!maybe.name && !!maybe.action;
}
/**
 * Convert an action function to a PushImpactListener if necessary
 * @param {PushImpactListenerRegisterable<any>} prr
 * @return {PushImpactListenerRegistration}
 */
function toPushReactionRegistration(prr) {
    return isPushReactionRegistration(prr) ? prr : {
        name: "Raw push reaction",
        action: prr,
    };
}
exports.toPushReactionRegistration = toPushReactionRegistration;
//# sourceMappingURL=PushImpactListenerRegistration.js.map