"use strict";
/*
 * Copyright © 2019 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const GoalInvocation_1 = require("../goal/GoalInvocation");
const commonPushTests_1 = require("../mapping/support/commonPushTests");
/**
 * Combine these transforms into a single transform,
 * where they execute it in order
 * @deprecated use array of CodeTransforms instead when constructing a ProjectOperationRegistration
 */
function chainTransforms(...transforms) {
    return (p, sdmc, params) => __awaiter(this, void 0, void 0, function* () {
        let cumulativeResult = {
            target: p,
            success: true,
            edited: false,
        };
        try {
            for (const t of transforms) {
                const lastResult = yield t(p, sdmc, params);
                cumulativeResult = combineResults(toTransformResult(p, lastResult), cumulativeResult);
            }
            return cumulativeResult;
        }
        catch (error) {
            automation_client_1.logger.warn("Failure in transform chain: %s", error);
            return { target: p, edited: cumulativeResult.edited, success: false, error };
        }
    });
}
exports.chainTransforms = chainTransforms;
function isTransformResult(tr) {
    const maybe = tr;
    return maybe && maybe.success !== undefined;
}
function toTransformResult(p, tr) {
    if (isTransformResult(tr)) {
        return tr;
    }
    else {
        return { target: p, success: true, edited: undefined };
    }
}
/* tslint:disable */ // Disable tslint from incorrectly breaking checks for false vs undefined
function combineResults(r1, r2) {
    return Object.assign(Object.assign(Object.assign({}, r1), r2), { edited: (r1.edited || r2.edited) ? true :
            (r1.edited === false && r2.edited === false) ? false : undefined, success: r1.success && r2.success });
}
/**
 * Convert a CodeTransform to a GoalProjectListener
 * @param transform
 * @param name
 * @param pushTest
 */
function transformToProjectListener(transform, name, pushTest = commonPushTests_1.AnyPush) {
    return {
        name,
        pushTest,
        events: [GoalInvocation_1.GoalProjectListenerEvent.before],
        listener: (p, gi) => __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield transform(p, Object.assign({}, gi), {});
                if (isTransformResult(result)) {
                    return {
                        code: result.success === true ? 0 : 1,
                    };
                }
            }
            catch (e) {
                return {
                    code: 1,
                    message: e.message,
                };
            }
            return automation_client_1.Success;
        }),
    };
}
exports.transformToProjectListener = transformToProjectListener;
//# sourceMappingURL=transformUtils.js.map