import { CloneOptions, GitProject, HandlerContext, ProjectOperationCredentials, RemoteRepoRef } from "@atomist/automation-client";
/**
 * Operation on loaded project
 */
export declare type WithLoadedProject<T = any> = (p: GitProject) => Promise<T>;
export interface ProjectLoadingParameters {
    credentials: ProjectOperationCredentials;
    id: RemoteRepoRef;
    cloneOptions?: CloneOptions;
    context?: HandlerContext;
    /** Return true to get optimized behavior for read only */
    readOnly: boolean;
    /**
     * @deprecated use cloneOptions.depth
     */
    depth?: number;
    /**
     * Explicitly configured target clone dir
     */
    cloneDir?: string;
}
/**
 * Common interface for project loading that allows caching etc.
 */
export interface ProjectLoader {
    /**
     * Perform an action with the given project
     * @param {ProjectLoadingParameters} params
     * @param {WithLoadedProject<T>} action
     */
    doWithProject<T>(params: ProjectLoadingParameters, action: WithLoadedProject<T>): Promise<T>;
}
//# sourceMappingURL=ProjectLoader.d.ts.map