import { RemoteRepoRef } from "@atomist/automation-client";
import { SdmGoalEvent } from "../../api/goal/SdmGoalEvent";
import { CoreRepoFieldsAndChannels, OnPushToAnyBranch, ScmProvider } from "../../typings/types";
/**
 * Resolve a RemoteRepoRef from data in our model
 */
export interface RepoRefResolver {
    /**
     * Obtain a RemoteRepoRef from the given push, correctly
     * resolving provider.
     * @param {OnPushToAnyBranch.Push} push
     * @return {any}
     */
    repoRefFromPush(push: OnPushToAnyBranch.Push): RemoteRepoRef;
    providerIdFromPush(push: OnPushToAnyBranch.Push): string | null;
    /**
     * get a repoRef from an SdmGoal.
     * @param {SdmGoalEvent} sdmGoal
     * @param {ScmProvider.ScmProvider} provider: No longer needed; the SdmGoalEvent now contains this information
     * @returns {RemoteRepoRef}
     */
    repoRefFromSdmGoal(sdmGoal: SdmGoalEvent, provider?: ScmProvider.ScmProvider): RemoteRepoRef;
    /**
     * Convert GraphQL return to our remote repo ref, instantiating
     * the correct type based on provider
     * @param {CoreRepoFieldsAndChannels.Fragment} repo
     * @param opts options - sha or branch
     * @return {RemoteRepoRef}
     */
    toRemoteRepoRef(repo: CoreRepoFieldsAndChannels.Fragment, opts: {
        sha?: string;
        branch?: string;
    }): RemoteRepoRef;
}
//# sourceMappingURL=RepoRefResolver.d.ts.map