/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import {
    Issue,
    ProjectOperationCredentials,
    raiseIssue,
    RemoteRepoRef,
} from "@atomist/automation-client";
import { isTokenCredentials } from "@atomist/automation-client/lib/operations/common/ProjectOperationCredentials";
import { IssueRouter } from "../../../spi/issue/IssueRouter";

/**
 * GitHub implementation of IssueRouter
 */
export class GitHubIssueRouter implements IssueRouter {

    public async raiseIssue(credentials: ProjectOperationCredentials,
                            id: RemoteRepoRef,
                            issue: Issue): Promise<void> {
        if (!isTokenCredentials(credentials)) {
            throw new Error("Only token credentials are supported");
        }
        await raiseIssue(credentials.token, id, issue);
    }

}
