"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Return the path from the root down to the target node.
 * Path includes root and target node.
 * @param {TreeNode} from start node
 * @param seen: Nodes we've already seen in this path
 * @param {TreeNode} node
 * @return {TreeNode[]}
 */
function pathDownTo(node, from, seen = []) {
    if (from === node) {
        return seen.concat(from);
    }
    if (!from.$children) {
        return undefined;
    }
    if (from.$children.includes(node)) {
        return seen.concat([from, node]);
    }
    for (const kid of from.$children) {
        const path = pathDownTo(node, kid, seen.concat(from));
        if (path) {
            return path;
        }
    }
    return undefined;
}
exports.pathDownTo = pathDownTo;
/**
 * Find the parent of the target node.
 * @param {TreeNode} root
 * @param {TreeNode} node we are searching down to
 * @return {TreeNode}
 */
function parentOf(node, root) {
    if (node.$parent) {
        return node.$parent;
    }
    const path = pathDownTo(node, root);
    return !!path && path.length >= 2 ? path[path.length - 2] : undefined;
}
exports.parentOf = parentOf;
//# sourceMappingURL=treeUtils.js.map