import { concatAST, visit } from 'graphql';
import { CompatabilityPluginVisitor } from './visitor';
const REACT_APOLLO_PLUGIN_NAME = 'typescript-react-apollo';
export const plugin = async (schema, documents, config, additionalData) => {
    const allAst = concatAST(documents.reduce((prev, v) => {
        return [...prev, v.content];
    }, []));
    const reactApollo = ((additionalData || {}).allPlugins || []).find(p => Object.keys(p)[0] === REACT_APOLLO_PLUGIN_NAME);
    const visitor = new CompatabilityPluginVisitor(config, schema, {
        reactApollo: reactApollo
            ? {
                ...(config || {}),
                ...reactApollo[REACT_APOLLO_PLUGIN_NAME],
            }
            : null,
    });
    const visitorResult = visit(allAst, {
        leave: visitor,
    });
    const discriminateUnion = `type DiscriminateUnion<T, U> = T extends U ? T : never;\n`;
    const requireField = `type RequireField<T, TNames extends string> = T & { [P in TNames]: (T & { [name: string]: never })[P] };\n`;
    const result = visitorResult.definitions.filter(a => a && typeof a === 'string').join('\n');
    return result.includes('DiscriminateUnion') ? [discriminateUnion, requireField, result].join('\n') : result;
};
//# sourceMappingURL=index.js.map