"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const visitor_1 = require("./visitor");
exports.TypeScriptDocumentsVisitor = visitor_1.TypeScriptDocumentsVisitor;
exports.plugin = (schema, documents, config) => {
    const allAst = graphql_1.concatAST(documents.reduce((prev, v) => {
        return [...prev, v.content];
    }, []));
    const allFragments = [
        ...allAst.definitions.filter(d => d.kind === graphql_1.Kind.FRAGMENT_DEFINITION).map(fragmentDef => ({ node: fragmentDef, name: fragmentDef.name.value, onType: fragmentDef.typeCondition.name.value, isExternal: false })),
        ...(config.externalFragments || []),
    ];
    const visitorResult = graphql_1.visit(allAst, {
        leave: new visitor_1.TypeScriptDocumentsVisitor(schema, config, allFragments),
    });
    const result = visitorResult.definitions.join('\n');
    if (config.globalNamespace) {
        return `
    declare global { 
      ${result} 
    }
          `;
    }
    else {
        return result;
    }
};
//# sourceMappingURL=index.js.map