import { SelectionSetToObject } from '@graphql-codegen/visitor-plugin-common';
import { isNonNullType, isListType } from 'graphql';
export class TypeScriptSelectionSetToObject extends SelectionSetToObject {
    constructor(_scalars, _schema, _convertName, _addTypename, _preResolveTypes, _nonOptionalTypename, _loadedFragments, _config, _parentSchemaType, _selectionSet) {
        super(_scalars, _schema, _convertName, _addTypename, _preResolveTypes, _nonOptionalTypename, _loadedFragments, _config.namespacedImportName, _config.dedupeOperationSuffix, _config.enumPrefix, _parentSchemaType, _selectionSet);
        this._config = _config;
    }
    createNext(parentSchemaType, selectionSet) {
        return new TypeScriptSelectionSetToObject(this._scalars, this._schema, this._convertName, this._addTypename, this._preResolveTypes, this._nonOptionalTypename, this._loadedFragments, this._config, parentSchemaType, selectionSet);
    }
    clearOptional(str) {
        const prefix = this._config.namespacedImportName ? `${this._config.namespacedImportName}\.` : '';
        const rgx = new RegExp(`^${prefix}Maybe<(.*?)>$`, 'is');
        if (str.startsWith(`${this._config.namespacedImportName ? `${this._config.namespacedImportName}.` : ''}Maybe`)) {
            return str.replace(rgx, '$1');
        }
        return str;
    }
    formatNamedField(name) {
        return this._config.immutableTypes ? `readonly ${name}` : name;
    }
    wrapTypeWithModifiers(baseType, type) {
        const prefix = this._config.namespacedImportName ? `${this._config.namespacedImportName}.` : '';
        if (isNonNullType(type)) {
            return this.clearOptional(this.wrapTypeWithModifiers(baseType, type.ofType));
        }
        else if (isListType(type)) {
            const innerType = this.wrapTypeWithModifiers(baseType, type.ofType);
            return `${prefix}Maybe<${this._config.immutableTypes ? 'ReadonlyArray' : 'Array'}<${innerType}>>`;
        }
        else {
            return `${prefix}Maybe<${baseType}>`;
        }
    }
}
//# sourceMappingURL=ts-selection-set-to-object.js.map